#include "atelem_c.h"


// Create a new mxArray
static mxArray* create_mxArray(const char *fieldname, enum enum_d_type d_type, int rows, int cols, void *data) 
{
   mxArray* elem = (mxArray*) malloc(sizeof(mxArray));
   int flen = strlen(fieldname);
   if (flen > 64)
      flen = 64;
   memcpy(elem->fieldname, fieldname,flen);
   if (flen < 64)
      elem->fieldname[flen] = 0;

   elem->d_type = d_type;
   elem->rows = rows;
   elem->cols = cols;
   if (d_type == AT_DOUBLE_STR) {
    	elem->malloc_data_size = data_size_elem_mxArray(d_type) * rows * cols;
    	elem->data = malloc(elem->malloc_data_size);
	char **string = (char **)malloc(rows * cols * sizeof(int*) * 2);
	char delimit[]=" \t\r\n\v\f;,"; 
	int i = 0, j = 0;
	string[i]=strtok((char *)data, delimit);    
	while(string[i]!=NULL) {
		i++;
		if (i > (cols * rows))
			break;
		string[i]=strtok(NULL,delimit);
	}
	for (j = 0; j < i; j++) {
		double d;
		sscanf(string[j], "%lf", &d);
		* (double *) ((char *)elem->data + j * data_size_elem_mxArray(d_type)) = d;
	}
	free(string);   
   }
   else if (d_type == AT_FUNCTION) {
   	elem->data = data;
   }	   
   else if ((d_type == AT_POINTER) || (d_type == AT_POINTER_F) || (d_type == AT_POINTER_NODE)) {
   	elem->data = data;
   }   
   else if (d_type == AT_CHAR) {
   	// size rounded to next multiple of 8
   	elem->malloc_data_size = (strlen((char*)data)+ 1 + 7) & (-8);
	elem->data = calloc(1,elem->malloc_data_size);
   	memcpy(elem->data, data, strlen((char*)data));
   	*(char *)((char *)elem->data+strlen((char*)data)) = 0; // end of string
   }	
   else {
   	elem->malloc_data_size = (data_size_elem_mxArray(d_type) * rows * cols + 7) & (-8);
      	elem->data = malloc(elem->malloc_data_size);
	memcpy(elem->data, data, data_size_elem_mxArray(d_type) * rows * cols);			   		
   }
   	
   elem->next = NULL;
   return elem;
}


// Create a new mxArray
static mxArray* clone_field_mxArray(mxArray* orig_elem) 
{
   mxArray* elem = (mxArray*) malloc(sizeof(mxArray));
   memcpy(elem->fieldname, orig_elem->fieldname,64);
   elem->d_type = orig_elem->d_type;
   elem->rows = orig_elem->rows;
   elem->cols = orig_elem->cols;

   if ((elem->d_type == AT_FUNCTION) || (elem->d_type == AT_POINTER) || (elem->d_type == AT_POINTER_NODE)) {
   	elem->data = orig_elem->data;
   }	   
   else if (elem->d_type== AT_POINTER_F) {
   	elem->data = NULL;
   }   
   else {
   	elem->malloc_data_size = orig_elem->malloc_data_size;
    	elem->data = malloc(elem->malloc_data_size);
	memcpy(elem->data, orig_elem->data, orig_elem->malloc_data_size);		   		
   }
   	
   
   	
   elem->next = NULL;
   return elem;
}



// Create a new mxArray
static mxArray* clone_elem_mxArray(mxArray* orig_elem) 
{
   int cnt = 0;
   mxArray *elem_head, *new_elem, *new_elem_prev;

   elem_head = NULL;

   // check if elemS is not empty
   if (orig_elem) {
      do {
         new_elem = clone_field_mxArray(orig_elem); 
         if (cnt == 0) {
         	elem_head = new_elem;
         	new_elem_prev = new_elem;
         }		
         else {
         	new_elem_prev->next = new_elem;
         	new_elem->next = NULL;
         	new_elem_prev = new_elem;
         }
         cnt++;
         	
         if (orig_elem->next != NULL)
            orig_elem = orig_elem->next;
         else
            break;   
         
      } while (orig_elem != NULL);
   }  
  
   return elem_head;
}






// Create a new element to the mxArray
static mxArray* add_mxArray(mxArray* elemS, const char *fieldname, enum enum_d_type d_type, int rows, int cols, void *data) 
{
   int err = 0;
   mxArray *elem_head = elemS;

   // check if elemS is not empty
   if (elemS) {
      // check for duplicate fieldnames
      do {
         int len = strlen(fieldname);
         if (len > 64)
            len = 64;
         if (strncmp(elemS->fieldname,(char *)fieldname,len) == 0) {
            err = -1; break;
         }
         if (elemS->next != NULL)
            elemS = elemS->next;
         else
            break;   
         
      } while (elemS != NULL);
   }  
   
   if (err == 0) {
      mxArray* elem = create_mxArray(fieldname, d_type, rows, cols, data);
      elemS->next = elem;
   }

   return elem_head;
}


// Create a new element to the mxArray and put it in second position
static mxArray* add_front_mxArray(mxArray* elemS, const char *fieldname, enum enum_d_type d_type, int rows, int cols, void *data) 
{
   mxArray *elem_head = elemS;

   // check if elemS is not empty
   if ((elemS) && (strncmp(elemS->fieldname,(char *)fieldname,64) != 0)) {
      mxArray* elem = create_mxArray(fieldname, d_type, rows, cols, data);
      elem->next = elemS->next;
      elemS->next = elem;
   }
   
   return elem_head;
}




static mxArray* concatenate_mxArray(mxArray *pm1, mxArray *pm2)
{
   mxArray *pm_head = pm1;
   // go through the list
   while (pm1->next != NULL) {
      pm1 = pm1->next;
   }
   pm1->next = pm2;

   return pm_head;

} 


static mxArray* get_field_mxArray(mxArray *pm, char *fieldname)
{

   mxArray *pmIter = pm;

   if (pm == NULL)
   	return NULL;


   while (pmIter != NULL) {
      int len = strlen(fieldname);
      if (len > 64)
         len = 64;
      if (strncmp((char *)pmIter->fieldname,fieldname,len) == 0) {
         return pmIter;
      }
      else
         pmIter = pmIter->next;   
   }
   return NULL;
   
}


static mxArray* get_last_mxArray(mxArray *pm)
{

   mxArray *pmIter = pm;

   if (pm == NULL)
   	return NULL;

   while (pmIter != NULL) {
   	 if (pmIter->next == NULL)
   	 	return pmIter;
         pmIter = pmIter->next;   
   }
   return NULL;
   
}



static mxArray* get_field_by_index_mxArray(mxArray *pm, int index)
{
   mxArray *pmIter = pm;

   int cnt = 0;
   while (pmIter != NULL) {
      if (cnt == index)
      	 return pmIter;
      pmIter = pmIter->next;   
   }
   return NULL;
}


static int num_fields_mxArray(mxArray *pm)
{
   int num_fields = 1;

   if (pm == NULL)
      return 0;
   
   // go through the list
   while (pm->next != NULL) {
      pm = pm->next;
      num_fields++;
   }

   return num_fields;

} 


static void print_type_mxArray(enum enum_d_type d_type)
{
   switch(d_type) {
      case(AT_INT8):   printf("INT8");  break;
      case(AT_UINT8):  printf("UINT8"); break;
      case(AT_INT16):  printf("INT16"); break;
      case(AT_UINT16): printf("UINT16");break;
      case(AT_INT32):  printf("INT32"); break;
      case(AT_UINT32): printf("UINT32");break;
      case(AT_INT64):  printf("INT64"); break;
      case(AT_UINT64): printf("UINT64");break;
      case(AT_FLOAT):  printf("FLOAT"); break;
      case(AT_DOUBLE): printf("DOUBLE");break;
      case(AT_CHAR):   printf("CHAR");break;
      case(AT_DOUBLE_STR): printf("DOUBLE");break;
      case(AT_FUNCTION): printf("FUNCTION");break;
      case(AT_POINTER): printf("POINTER");break;
      case(AT_POINTER_F): printf("POINTER_F");break;
      case(AT_POINTER_NODE): printf("POINTER_NODE");break;      
  }
}


static void print_strout_type_mxArray(enum enum_d_type d_type, char *strout)
{
   switch(d_type) {
      case(AT_INT8):   sprintf(strout,"%sINT8",strout);  break;
      case(AT_UINT8):  sprintf(strout,"%sUINT8",strout); break;
      case(AT_INT16):  sprintf(strout,"%sINT16",strout); break;
      case(AT_UINT16): sprintf(strout,"%sUINT16",strout);break;
      case(AT_INT32):  sprintf(strout,"%sINT32",strout); break;
      case(AT_UINT32): sprintf(strout,"%sUINT32",strout);break;
      case(AT_INT64):  sprintf(strout,"%sINT64",strout); break;
      case(AT_UINT64): sprintf(strout,"%sUINT64",strout);break;
      case(AT_FLOAT):  sprintf(strout,"%sFLOAT",strout); break;
      case(AT_DOUBLE): sprintf(strout,"%sDOUBLE",strout);break;
      case(AT_CHAR):   sprintf(strout,"%sCHAR",strout);break;
      case(AT_DOUBLE_STR): sprintf(strout,"%sDOUBLE",strout);break;
      case(AT_FUNCTION): sprintf(strout,"%sFUNCTION",strout);break;
      case(AT_POINTER): sprintf(strout,"%sPOINTER",strout);break;
      case(AT_POINTER_F): sprintf(strout,"%sPOINTER_F",strout);break;
      case(AT_POINTER_NODE): sprintf(strout,"%sPOINTER_NODE",strout);break;      
  }
}



static void print_data_elem_mxArray(void *value, enum enum_d_type d_type)
{
   switch(d_type) {
      case(AT_INT8):   printf("%hhi",*(int8_t *)  value); break;
      case(AT_UINT8):  printf("%hhu",*(uint8_t *) value); break;
      case(AT_INT16):  printf("%hi", *(int16_t *) value); break;
      case(AT_UINT16): printf("%hu", *(uint16_t *)value); break;
      case(AT_INT32):  printf("%i",  *(int32_t *) value); break;
      case(AT_UINT32): printf("%u",  *(uint32_t *)value); break;
      case(AT_INT64):  printf("%lli",(long long int) *(int64_t *) value); break;
      case(AT_UINT64): printf("%llu",(long long unsigned int) *(uint64_t *)value); break;
      case(AT_FLOAT):  printf("%f",  *(float *)   value); break;
      case(AT_DOUBLE): printf("%.15e",  *(double *)  value); break;
      case(AT_CHAR):   printf("%s",  (char *)  value); break;
      case(AT_DOUBLE_STR): printf("%.15e",  *(double *)  value); break;  
      case(AT_FUNCTION): printf("0x%llx",  (long long unsigned int)value); break;   
      case(AT_POINTER): printf("0x%llx",  (long long unsigned int)value); break;
      case(AT_POINTER_F): printf("0x%llx",  (long long unsigned int)value); break;
      case(AT_POINTER_NODE): printf("0x%llx",  (long long unsigned int)value); break;      
  }
}


static void print_strout_data_elem_mxArray(void *value, enum enum_d_type d_type, char *strout)
{
   switch(d_type) {
      case(AT_INT8):   sprintf(strout,"%s%hhi",strout,*(int8_t *)  value); break;
      case(AT_UINT8):  sprintf(strout,"%s%hhu",strout,*(uint8_t *) value); break;
      case(AT_INT16):  sprintf(strout,"%s%hi",strout, *(int16_t *) value); break;
      case(AT_UINT16): sprintf(strout,"%s%hu",strout, *(uint16_t *)value); break;
      case(AT_INT32):  sprintf(strout,"%s%i",strout,  *(int32_t *) value); break;
      case(AT_UINT32): sprintf(strout,"%s%u",strout,  *(uint32_t *)value); break;
      case(AT_INT64):  sprintf(strout,"%s%lli",strout,(long long int) *(int64_t *) value); break;
      case(AT_UINT64): sprintf(strout,"%s%llu",strout,(long long unsigned int) *(uint64_t *)value); break;
      case(AT_FLOAT):  sprintf(strout,"%s%f",strout,  *(float *)   value); break;
      case(AT_DOUBLE): sprintf(strout,"%s%.15e",strout,  *(double *)  value); break;
      case(AT_CHAR):   sprintf(strout,"%s%s",strout,  (char *)  value); break;
      case(AT_DOUBLE_STR): sprintf(strout,"%s%.15e",strout,  *(double *)  value); break;  
      case(AT_FUNCTION): sprintf(strout,"%s0x%llx",strout,  (long long unsigned int)value); break;   
      case(AT_POINTER): sprintf(strout,"%s0x%llx",strout,  (long long unsigned int)value); break;
      case(AT_POINTER_F): sprintf(strout,"%s0x%llx", strout, (long long unsigned int)value); break;
      case(AT_POINTER_NODE): sprintf(strout,"%s0x%llx", strout, (long long unsigned int)value); break;      
  }
}


static int data_size_elem_mxArray(enum enum_d_type d_type)
{
   switch(d_type) {
      case(AT_CHAR):   return sizeof(int8_t); break;
      case(AT_INT8):   return sizeof(int8_t); break;
      case(AT_UINT8):  return sizeof(uint8_t); break;
      case(AT_INT16):  return sizeof(int16_t); break;
      case(AT_UINT16): return sizeof(uint16_t); break;
      case(AT_INT32):  return sizeof(int32_t); break; 
      case(AT_UINT32): return sizeof(uint32_t); break;
      case(AT_INT64):  return sizeof(int64_t); break;
      case(AT_UINT64): return sizeof(uint64_t); break;
      case(AT_FLOAT):  return sizeof(float); break;
      case(AT_DOUBLE): return sizeof(double); break;
      case(AT_DOUBLE_STR): return sizeof(double); break; 
      case(AT_FUNCTION): return sizeof(int*); break;    
      case(AT_POINTER): return sizeof(int*); break; 
      case(AT_POINTER_F): return sizeof(int*); break;
      case(AT_POINTER_NODE): return sizeof(int*); break;      
  }
  return 0;
}


static void print_all_fields_mxArray(mxArray *pm)
{

   while (pm != NULL) {
      printf(" field %s rows:%d cols:%d\n",pm->fieldname, pm->rows, pm->cols);
      pm = pm->next;   
   }

} 


static mxArray* destroy_field_mxArray(mxArray *pm, char *fieldname)
{
   mxArray *pre_pm = NULL, *head_pm = pm, *post_pm = NULL;
 

   if (pm != NULL)
      post_pm = pm->next;

   mxArray* elemIter = pm;
   do {
      // if field found
      int len = strlen(fieldname);
      if (len > 64)
         len = 64;
      if (strncmp((char *)elemIter->fieldname,fieldname,len) == 0) {
         // first element
         if (pre_pm == NULL) {
            head_pm = elemIter->next;
            free_mxArray(elemIter);
            break;
         }
         // last element
         else if (elemIter->next == NULL) {
            pre_pm->next = NULL;
            free_mxArray(elemIter);
            break;        
         }
         else {
           pre_pm->next = post_pm;
           free_mxArray(elemIter);
           break;              
         }
      }
      else {
         pre_pm = elemIter;
         elemIter = elemIter->next; 
         if (elemIter != NULL)
            post_pm = elemIter->next;
         else
            post_pm = NULL;
      }
   } while (elemIter != NULL);

   return head_pm;

}


static void free_mxArray(mxArray *pm)
{

   if (pm == NULL)
      return;

   if ((pm->d_type != AT_POINTER) && (pm->d_type != AT_FUNCTION) && (pm->d_type != AT_POINTER_NODE)) {    
      if (pm->data) {
         free(pm->data);   	   
      }
   }   	   
   if (pm) {
      free(pm); 
   
   }
}


static void destroy_all_mxArray(mxArray *pm)
{

   if (pm == NULL)
      return;

   mxArray* elemIter = pm;
   do {
       mxArray* pm_del = elemIter;
       elemIter = elemIter->next; 
       free_mxArray(pm_del);
   } while (elemIter != NULL);
  
}


static void print_mxArray(mxArray *pm, char *fieldname)
{
   // go through the list
   mxArray* elem = get_field_mxArray(pm, fieldname);

   if (elem != NULL) {
      printf("\n fieldname: %s\n",fieldname);
      printf(" data type: "); print_type_mxArray(elem->d_type); printf("\n");
      printf("  rows: %i\n", elem->rows);
      printf("  cols: %i\n", elem->cols);
      printf("  value:\n");
      int d_size = data_size_elem_mxArray(elem->d_type);

      for (int i = 0; i < elem->rows; i++) {
         printf("\t");
         if (elem->d_type != AT_CHAR) {
	    for (int j = 0; j < elem->cols; j++) {
               print_data_elem_mxArray((char *)elem->data + j * d_size + i * elem->cols * d_size, elem->d_type);printf("\t");
            }
         }
         else {
            print_data_elem_mxArray(elem->data, elem->d_type);printf("\t");
	 }
	 printf("\n");		 
      }
      printf("\n");
   }

} 


static void print_strout_mxArray(mxArray *pm, char *fieldname, char *strout)
{
   // go through the list
   mxArray* elem = get_field_mxArray(pm, fieldname);

   if (elem != NULL) {
      sprintf(strout,"%s\n fieldname: %s\n",strout,fieldname);
      sprintf(strout,"%s data type: ",strout); print_strout_type_mxArray(elem->d_type,strout); sprintf(strout,"%s\n",strout);
      sprintf(strout,"%s  rows: %i\n",strout, elem->rows);
      sprintf(strout,"%s  cols: %i\n",strout, elem->cols);
      sprintf(strout,"%s  value:\n",strout);
      int d_size = data_size_elem_mxArray(elem->d_type);

      for (int i = 0; i < elem->rows; i++) {
         sprintf(strout,"%s\t",strout);
         if (elem->d_type != AT_CHAR) {
	    for (int j = 0; j < elem->cols; j++) {
               print_strout_data_elem_mxArray((char *)elem->data + j * d_size + i * elem->cols * d_size, elem->d_type,strout);sprintf(strout,"%s\t",strout);
            }
         }
         else {
            print_strout_data_elem_mxArray(elem->data, elem->d_type,strout);sprintf(strout,"%s\t",strout);
	 }
	 sprintf(strout,"%s\n",strout);		 
      }
      sprintf(strout,"%s\n",strout);
   }

} 


static void print_all_mxArray(mxArray *pm)
{
   mxArray* elemIter = pm;
   while (elemIter != NULL) {
      print_mxArray(elemIter, elemIter->fieldname);
      elemIter = elemIter->next;   
   }

}


static void print_strout_all_mxArray(mxArray *pm, char *strout)
{
   mxArray* elemIter = pm;
   while (elemIter != NULL) {
      print_strout_mxArray(elemIter, elemIter->fieldname, strout);
      elemIter = elemIter->next;   
   }

}


static void* atGetPointer(mxArray *ElemData, const char *fieldname)
{
    mxArray *field = NULL;
    
    field = get_field_mxArray(ElemData,(char *)fieldname);
    if (!field) {
    	//printf("%s: error AT:WrongArg, the required attribute %s is missing.\n", __func__, fieldname);
    	return NULL;
    }
    else {	
    	return field->data;
    }	
}

static long atGetLong(mxArray *ElemData, const char *fieldname)
{
    mxArray *field=get_field_mxArray(ElemData,(char *)fieldname);
    if (!field) printf("%s: error AT:WrongArg, the required attribute %s is missing.\n", __func__, fieldname);
    
    // all the strings are saved as double in mxArray
    if ((field->d_type == AT_DOUBLE_STR) || (field->d_type == AT_DOUBLE))
    	return (int32_t) *(double *)field->data;
    else if (field->d_type == AT_FLOAT)
    	return (int32_t) *(float *)field->data;
    
    return *(long *)field->data;
}

static double atGetDouble(mxArray *ElemData, const char *fieldname)
{
    mxArray *field=get_field_mxArray(ElemData,(char*)fieldname);
    if (!field) {
	printf("%s: error AT:WrongArg, the required attribute %s is missing.\n", __func__, fieldname);
	return NAN;
    }
    return *(double *)field->data;
}

static double* atGetDoubleArraySz(mxArray *ElemData, char *fieldname, int *msz, int *nsz)
{
    mxArray *field=get_field_mxArray(ElemData,fieldname);
    if (!field) printf("%s: error AT:WrongArg, the required attribute %s is missing.\n", __func__, fieldname);
    *msz = field->rows;  /*Number of rows in the 2-D array*/
    *nsz = field->cols;  /*Number of columns in the 2-D array.*/
    return (double *)field->data;
}

static double* atGetDoubleArray(mxArray *ElemData, char *fieldname)
{
    int msz, nsz;
    return atGetDoubleArraySz(ElemData, fieldname, &msz, &nsz);
}

static long atGetOptionalLong(mxArray *ElemData, char *fieldname, long default_value)
{
    mxArray *field=get_field_mxArray(ElemData,fieldname);

   // all the strings are saved as double in mxArray
   if (field) {
	   if ((field->d_type == AT_DOUBLE_STR) || (field->d_type == AT_DOUBLE))
	    	return (field) ? (int32_t) *(double *)field->data : default_value;
	    else if (field->d_type == AT_FLOAT)
	    	return (field) ? (int32_t) *(float *)field->data : default_value;    
   }
    
   return (field) ? *(long *)field->data : default_value;
}

static double atGetOptionalDouble(mxArray *ElemData, char *fieldname, double default_value)
{
    mxArray *field=get_field_mxArray(ElemData,fieldname);
    return (field) ? *(double *)field->data : default_value;
}

static double* atGetOptionalDoubleArraySz(mxArray *ElemData, char *fieldname, int *msz, int *nsz)
{
    double *ptr = NULL;
    mxArray *field=get_field_mxArray(ElemData,fieldname);
    if (field) {
        *msz = field->rows;
        *nsz = field->cols;
        ptr = (double *)field->data;
    }
    return ptr;
}

static double* atGetOptionalDoubleArray(mxArray *ElemData, char *fieldname)
{
    int msz, nsz;
    return atGetOptionalDoubleArraySz(ElemData, fieldname, &msz, &nsz);
}

 
