#ifndef ATELEM_C_H
#define ATELEM_C_H

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h> // otherwise strtok does not work

#include <attypes.h>

enum enum_d_type {AT_INT8, AT_UINT8, AT_INT16, AT_UINT16, AT_INT32, AT_UINT32, AT_INT64, AT_UINT64, AT_FLOAT, AT_DOUBLE, AT_DOUBLE_STR, AT_CHAR, AT_FUNCTION, AT_POINTER, AT_POINTER_F, AT_POINTER_NODE}; 


typedef struct mxArray {
	char fieldname[64];
        enum enum_d_type d_type;
        int rows; // number of rows
        int cols; // number of cols
        int malloc_data_size;
        struct mxArray *next;
	void *data;
} mxArray;


typedef mxArray atElem;

#define check_error()
#define atError(...) 
#define atWarning(...) 

typedef void * (*track_function_ptr)(mxArray *ElemData, struct elem *Elem,
			     double *r_in, int num_particles, struct parameters *Param);  // Define a type for the function pointer

// Create a new mxArray
static mxArray* create_mxArray(const char *fieldname, enum enum_d_type d_type, int rows, int cols, void *data);
// Create a new element to the mxArray
static mxArray* add_mxArray(mxArray* elemS, const char *fieldname, enum enum_d_type d_type, int rows, int cols, void *data);
// Create a new element and set in second order
static mxArray* add_front_mxArray(mxArray* elemS, const char *fieldname, enum enum_d_type d_type, int rows, int cols, void *data);

// concatenate two atElements
static mxArray* concatenate_mxArray(mxArray *pm1, mxArray *pm2);
// 

static mxArray* get_field_by_index_mxArray(mxArray *pm, int index);

static mxArray* get_field_mxArray(mxArray *pm, char *fieldname);
//
static int num_fields_mxArray(mxArray *pm);

static void print_type_mxArray(enum enum_d_type d_type);

static void print_strout_type_mxArray(enum enum_d_type d_type, char *strout);

static void print_data_elem_mxArray(void *value, enum enum_d_type d_type);

static void print_strout_data_elem_mxArray(void *value, enum enum_d_type d_type, char *strout);

static int data_size_elem_mxArray(enum enum_d_type d_type);

static mxArray* destroy_field_mxArray(mxArray *pm, char *fieldname);

static void destroy_all_mxArray(mxArray *pm);

static void free_mxArray(mxArray *pm);

static mxArray* get_last_mxArray(mxArray *pm);

static void print_all_fields_mxArray(mxArray *pm);

static void print_mxArray(mxArray *pm, char *fieldname); 

static void print_strout_mxArray(mxArray *pm, char *fieldname, char *strout); 

static void print_all_mxArray(mxArray *pm);

static void print_strout_all_mxArray(mxArray *pm, char *strout);

static void* atGetPointer(mxArray *ElemData, const char *fieldname);

static long atGetLong(mxArray *ElemData, const char *fieldname);

static double atGetDouble(mxArray *ElemData, const char *fieldname);

static double* atGetDoubleArraySz(mxArray *ElemData, char *fieldname, int *msz, int *nsz);

static double* atGetDoubleArray(mxArray *ElemData, char *fieldname);

static long atGetOptionalLong(mxArray *ElemData, char *fieldname, long default_value);

static double atGetOptionalDouble(mxArray *ElemData, char *fieldname, double default_value);

static double* atGetOptionalDoubleArraySz(mxArray *ElemData, char *fieldname, int *msz, int *nsz);

static double* atGetOptionalDoubleArray(mxArray *ElemData, char *fieldname);





#endif






