#include "amat.h"


void amat_alloc(int size, gsl_matrix **A, gsl_vector_complex **lambda)
{
	(*A) = gsl_matrix_alloc(size,size);
	(*lambda) = gsl_vector_complex_alloc(size / 2);
}


void amat_free(gsl_matrix **A, gsl_vector_complex **lambda)
{
	gsl_matrix_free((*A));
	gsl_vector_complex_free((*lambda));
}


int amat(double *M66i, double *Ai_test, int N, gsl_matrix *A_out, gsl_vector_complex *lambda_out)
{
	int dms = N / 2;
	int err = 0;

	gsl_matrix_complex *M66p = gsl_matrix_complex_alloc_p(M66i, N, N); 

	gsl_matrix_complex *Vxyz = jmat_complex_alloc(dms);

	gsl_matrix_complex *S = kmat_complex_alloc(dms);

	// [V,lambda]=eig(transmat);
	gsl_vector_complex *lambda = gsl_vector_complex_alloc (N);
	gsl_matrix_complex *V = gsl_matrix_complex_alloc (N, N);


	eig(M66i, lambda, V, N, 0, 0); 
	//print_matrix_complex(M66p);
	

	// the sign of the eigenvector is arbitrary
	// multiply by -1 to be compliant to esrf matlab output
	// GSL has inverted the sign of V in respect to matlab

	// GG zio poldin
	// gsl_matrix_complex_rescale_real(V,  -1, V); //valore originale

	//gsl_matrix_complex_rescale_real(V,  1, V); // valore futizzato



	// VT = V'
	gsl_matrix_complex *Vt = gsl_matrix_complex_alloc (N, N);
	gsl_matrix_complex_conjtranspose_memcpy(V,Vt);  

	// Vp = V' * s  
	gsl_matrix_complex *Vp = gsl_matrix_complex_alloc (N, N);
	gsl_matrix_complex_mul(Vt, S, Vp);

	
	// VpNc = Vp.' // transposed not coniugated
	gsl_matrix_complex *VpNc = gsl_matrix_complex_alloc (N, N);
	gsl_matrix_complex_transpose2_memcpy(Vp,VpNc);  


	// (Vp.'.*V);
	gsl_matrix_complex *VpV = gsl_matrix_complex_alloc (N, N);
	gsl_matrix_complex_mul_one2one(VpNc,  V, VpV);

	//print_matrix_complex(VpV); qua OK

	// sum(Vp.'.*V);
	gsl_vector_complex *n = gsl_vector_complex_alloc (N);
	gsl_matrix_complex_sum_col(VpV, n);

	// n= -0.5i*sum(Vp.'.*V);
	gsl_complex scale; GSL_SET_COMPLEX(&scale, 0, -0.5);
	gsl_vector_complex_rescale_complex(n, scale, n);

	

	gsl_vector *nr = gsl_vector_alloc(N);
	gsl_vector_complex_real_at(n, nr);


	// abs(-0.5i*sum(Vp.'.*V);  
	gsl_vector *absn = gsl_vector_alloc(N);
	gsl_vector_complex_abs_real(n, absn);

	// if any(abs(n) < 1.0E-12)
	if (gsl_vector_min(absn) < 1e-12) {
		//printf("'AT:Unstable,  unstable machine\n");
		gsl_matrix_complex_free(M66p); 
		gsl_matrix_complex_free(Vxyz);
		gsl_matrix_complex_free(S);
		gsl_vector_complex_free(lambda);
		gsl_matrix_complex_free(V);
		gsl_matrix_complex_free(Vt);
		gsl_matrix_complex_free(Vp);
		gsl_matrix_complex_free(VpNc);
		gsl_matrix_complex_free(VpV);
		gsl_vector_complex_free(n);
		gsl_vector_free(nr);
		gsl_vector_free(absn);
		return -1;
	}

	int *order = malloc(N * sizeof(int));
	int *select = malloc(dms * sizeof(int));

	calculate_select_order(dms, select, order, nr);

	/*
	V=V(:,order);
	n=n(order);
	lambda=lambda(order);  
	*/
	gsl_matrix_complex *Vo = gsl_matrix_complex_alloc(N, N);
	gsl_matrix_complex_order_columns(V,  order, Vo);

	gsl_vector_complex *no = gsl_vector_complex_alloc(N);

	gsl_vector_complex_order(n,  order, no);

	gsl_vector_complex *lambdao = gsl_vector_complex_alloc(N);

	gsl_vector_complex_order(lambda,  order, lambdao); 

	gsl_vector *nro = gsl_vector_alloc(N), *absnro = gsl_vector_alloc(N);
	//n=n(order);
	gsl_vector_order(nr,  order, nro);

	for (unsigned int i = 0; i < absnro->size; i++)
		gsl_vector_set(absnro, i , fabs(gsl_vector_get(nro, i)));

	gsl_vector *inv_sqrt_absnro = gsl_vector_alloc(N);
	// 1/sqrt(abs(n)
	for (unsigned int i = 0; i < absnro->size; i++)
		gsl_vector_set(inv_sqrt_absnro, i , pow(1 / gsl_vector_get(absnro, i),0.5));	
		
        // V is ordered
	// Vn=V./sqrt(abs(n));
	gsl_matrix_complex *Vn = gsl_matrix_complex_alloc(N, N);
	gsl_matrix_complex_rescale_vector_real(Vo,  inv_sqrt_absnro, Vn);

	// S*Vxyz
	gsl_matrix_complex *SVxyz = gsl_matrix_complex_alloc(N, N); 
	gsl_matrix_complex_mul(S, Vxyz, SVxyz); 

	// Vn'
	gsl_matrix_complex *Vnt = gsl_matrix_complex_alloc (N, N);
	gsl_matrix_complex_conjtranspose_memcpy(Vn, Vnt); 

	// Vn'*S*Vxyz
	gsl_matrix_complex *VntSVxyz = gsl_matrix_complex_alloc(N, N); 
	gsl_matrix_complex_mul(Vnt, SVxyz, VntSVxyz);

	//-1i*Vn'*S*Vxyz 
	gsl_complex scaleI; GSL_SET_COMPLEX(&scaleI, 0, -1);
	gsl_matrix_complex_rescale_complex(VntSVxyz, scaleI, VntSVxyz);  

	// nn=0.5*abs(sqrt(-1i*Vn'*S*Vxyz));
	gsl_matrix_complex_sqrt(VntSVxyz,VntSVxyz);

	gsl_matrix_complex_abs(VntSVxyz,VntSVxyz);

	gsl_matrix_complex_rescale_real(VntSVxyz,0.5,VntSVxyz);

	int *project_xyz_order = malloc(dms * sizeof(int));

	// find the vecs that project most onto x,y,z, and reorder
	// nn will have structure
	// n1x n1y n1z
	// n2x n2y n2z
	// n3x n3y n3z

	find_xyz_projection(VntSVxyz, select, project_xyz_order);
	
	// V_ordered=Vn(:,order);
	gsl_matrix_complex *Vordered = gsl_matrix_complex_alloc(N, N/2);  
	gsl_matrix_complex_copy_columns(Vn, project_xyz_order, Vordered);

	// lambda=lambda(order);
	gsl_vector_complex_copy_elem(lambdao, project_xyz_order, lambda_out);

	// build the A matrix   
	// a=reshape([real(V_ordered);imag(V_ordered)],nv,nv);
	build_a_matrix(Vordered, A_out);

	if (Ai_test != NULL) {
		gsl_matrix *A_test = gsl_matrix_alloc_p(Ai_test, N, N);

		double err = gsl_matrix_abs_diff(A_test, A_out);
		printf("err = %.8e\n",err);
		if (err > 0.01) {
			printf("TEST FAILED!!!!!!!!!\n");
		}
		gsl_matrix_free(A_test);
	}   

	if (project_xyz_order)
		free(project_xyz_order); 
	if (order)
		free(order);
	if (select)	
		free(select);
	if (absn)	
		gsl_vector_free(absn);
	if (absnro)
		gsl_vector_free(absnro);
	if (inv_sqrt_absnro)
		gsl_vector_free(inv_sqrt_absnro);  
	if (nr)
		gsl_vector_free(nr);  
	if (lambda)
		gsl_vector_complex_free(lambda);
	if (lambdao)
		gsl_vector_complex_free(lambdao); 
	if (Vordered)
		gsl_matrix_complex_free(Vordered);
	if (Vt)
		gsl_matrix_complex_free(Vt);  
	if (Vo)
		gsl_matrix_complex_free(Vo); 
	if (Vnt)
		gsl_matrix_complex_free(Vnt);  
	if (Vn)
		gsl_matrix_complex_free(Vn);    
	if (Vp)
		gsl_matrix_complex_free(Vp); 
	if (no)
		gsl_vector_complex_free(no);  
	if (n)
		gsl_vector_complex_free(n);
	if (VpV)
		gsl_matrix_complex_free(VpV);   
	if (VntSVxyz)
		gsl_matrix_complex_free(VntSVxyz); 
	if (SVxyz)
		gsl_matrix_complex_free(SVxyz); 
	if (VpNc)
		gsl_matrix_complex_free(VpNc);   
	if (V)
		gsl_matrix_complex_free(V);
	if (S)
		gsl_matrix_complex_free(S);
	if (Vxyz)
		gsl_matrix_complex_free(Vxyz);
	if (M66p)
		gsl_matrix_complex_free(M66p);
		
	return err;	

}

