
#include "at_math.h"






void print_complex(gsl_complex z)
{
	printf("%.8e + %.8ei", GSL_REAL(z), GSL_IMAG(z));
} 


void print_matrix(gsl_matrix *A) 
{
	double sum=0;
	for (unsigned int i = 0; i < A->size1; i++) {
		for (unsigned int j = 0; j < A->size2; j++) {
			sum += gsl_matrix_get(A, i, j);
			printf("%.13e\t",gsl_matrix_get(A, i, j));
		}
		printf("\n");
	}
	printf("abssum=%.8e\n",sum);
}


void print_vector(gsl_vector *A) 
{
	for (unsigned int i = 0; i < A->size; i++) {
		printf("%.13e\t", gsl_vector_get(A, i));
	}
	printf("\n\n");
}



void print_matrix_complex(gsl_matrix_complex *A) 
{
	for (unsigned int i = 0; i < A->size1; i++) {
		for (unsigned int j = 0; j < A->size2; j++) {
			 gsl_complex z = gsl_matrix_complex_get(A, i, j);
			 printf("%.13e + %.13ei\t", GSL_REAL(z), GSL_IMAG(z));
		}
		printf("\n");
	}
	printf("\n");
}


void print_vector_complex(gsl_vector_complex *A) 
{
	for (int i = 0; i < (int)A->size; i++) {
		gsl_complex z = gsl_vector_complex_get(A, i);
		printf("%.13e + %.13ei\t", GSL_REAL(z), GSL_IMAG(z));
	}
	printf("\n\n");
}




gsl_matrix *gsl_matrix_alloc_p(double *vector, const int N, const int M) 
{
	gsl_matrix *A = gsl_matrix_alloc(N, M);

	for (int i = 0; i < N; i++) {
		for (int j = 0; j < M; j++) {
			gsl_matrix_set(A, i, j, vector[i*M+j]);
		}
	}
	return A;
}


gsl_matrix *gsl_matrix_alloc_p_trasp(double *vector, const int N, const int M) 
{
	gsl_matrix *A = gsl_matrix_alloc(N, M);

	for (int i = 0; i < N; i++) {
		for (int j = 0; j < M; j++) {
			gsl_matrix_set(A, j, i, vector[i*M+j]);
		}
	}
	return A;
}



gsl_matrix_complex *gsl_matrix_complex_alloc_p(double *vector, const int N, const int M) 
{
	gsl_matrix_complex *A = gsl_matrix_complex_alloc(N, M);

	for (int i = 0; i < N; i++) {
		for (int j = 0; j < M; j++) {
		 	gsl_complex z;
			GSL_SET_COMPLEX(&z, vector[i*M+j], 0);
			gsl_matrix_complex_set(A, i, j, z);
		}
	}
	return A;
}


gsl_matrix_complex *gsl_matrix_complex_alloc_p_trasp(double *vector, const int N, const int M) 
{
	gsl_matrix_complex *A = gsl_matrix_complex_alloc(N, M);

	for (int i = 0; i < N; i++) {
		for (int j = 0; j < M; j++) {
		 	gsl_complex z;
			GSL_SET_COMPLEX(&z, vector[i*M+j], 0);
			gsl_matrix_complex_set(A, i, j, z);
		}
	}
	return A;
}


gsl_matrix_complex *jmat_complex_alloc(int dms)
{

	// create the xyz complex matrix
	gsl_matrix_complex *xyz = gsl_matrix_complex_alloc(2, 2);
	gsl_complex z;
	GSL_SET_COMPLEX(&z, 0, -1);
	gsl_matrix_complex_set(xyz, 0, 0, z);
	GSL_SET_COMPLEX(&z, -1, 0);
	gsl_matrix_complex_set(xyz, 0, 1, z); 
	GSL_SET_COMPLEX(&z, 1, 0);
	gsl_matrix_complex_set(xyz, 1, 0, z); 
	GSL_SET_COMPLEX(&z, 0, 1);
	gsl_matrix_complex_set(xyz, 1, 1, z);     

	gsl_matrix_complex *Vxyz = gsl_matrix_complex_alloc(2*dms, 2*dms);
	for (int i = 0; i < 2*dms; i++) {
		for (int j = 0; j < 2*dms; j++) {
			gsl_complex q;	GSL_SET_COMPLEX(&q, 0, 0);
			gsl_matrix_complex_set(Vxyz,i,j,q);
		}
	}

	// block diagonal
	for (int k = 0; k < dms; k++) {
		for (int i = 0; i < 2; i++) {
			for (int j = 0; j < 2; j++) {
				gsl_complex q =  gsl_matrix_complex_get(xyz, i, j);
				gsl_matrix_complex_set(Vxyz, i + k*2 ,j + k*2, q);
			}
		}
	}	

	gsl_matrix_complex_free(xyz);

	return Vxyz;
}


gsl_matrix *jmat_alloc(int dms)
{

	// create the xyz complex matrix
	gsl_matrix *xyz = gsl_matrix_alloc(2, 2);

	gsl_matrix_set(xyz, 0, 0, 0);
	gsl_matrix_set(xyz, 0, 1, 1); 
	gsl_matrix_set(xyz, 1, 0, -1); 
	gsl_matrix_set(xyz, 1, 1, 0);     

	gsl_matrix *Vxyz = gsl_matrix_alloc(2*dms, 2*dms);
	for (int i = 0; i < 2*dms; i++) {
		for (int j = 0; j < 2*dms; j++) {	
			gsl_matrix_set(Vxyz,i,j,0);
		}
	}

	// block diagonal
	for (int k = 0; k < dms; k++) {
		for (int i = 0; i < 2; i++) {
			for (int j = 0; j < 2; j++) {
				double q =  gsl_matrix_get(xyz, i, j);
				gsl_matrix_set(Vxyz, i + k*2 ,j + k*2, q);
			}
		}
	}	

	gsl_matrix_free(xyz);

	return Vxyz;
}


gsl_matrix_complex *kmat_complex_alloc(int dms)
{

	// create the xyz complex matrix
	gsl_matrix_complex *s = gsl_matrix_complex_alloc(2, 2);
	gsl_complex z;
	GSL_SET_COMPLEX(&z, 0, 0);
	gsl_matrix_complex_set(s, 0, 0, z);
	GSL_SET_COMPLEX(&z, 1, 0);
	gsl_matrix_complex_set(s, 0, 1, z); 
	GSL_SET_COMPLEX(&z, -1, 0);
	gsl_matrix_complex_set(s, 1, 0, z); 
	GSL_SET_COMPLEX(&z, 0, 0);
	gsl_matrix_complex_set(s, 1, 1, z);     

	gsl_matrix_complex *S = gsl_matrix_complex_alloc(2*dms, 2*dms);
	for (int i = 0; i < 2*dms; i++) {
		for (int j = 0; j < 2*dms; j++) {
			gsl_complex q;	GSL_SET_COMPLEX(&q, 0, 0);
			gsl_matrix_complex_set(S,i,j,q);
		}
	}

	// block diagonal
	for (int k = 0; k < dms; k++) {
		for (int i = 0; i < 2; i++) {
			for (int j = 0; j < 2; j++) {
				gsl_complex q =  gsl_matrix_complex_get(s, i, j);
				gsl_matrix_complex_set(S, i + k*2 ,j + k*2, q);
			}
		}
	}	

	// transpose the miniblock
	if (dms == 3) {
		gsl_complex tmp = gsl_matrix_complex_get(S, 4, 5);
		gsl_matrix_complex_set(S, 4, 5, gsl_matrix_complex_get(S, 5, 4));
		gsl_matrix_complex_set(S, 5, 4, tmp);
	}

	gsl_matrix_complex_free(s);

	return S;

}


gsl_matrix *kmat_alloc(int dms)
{
	// create the xyz complex matrix
	gsl_matrix *s = gsl_matrix_alloc(2, 2);
	// 0  1
	//-1  0
	gsl_matrix_set(s, 0, 0, 0);
	gsl_matrix_set(s, 0, 1, 1); 
	gsl_matrix_set(s, 1, 0, -1); 
	gsl_matrix_set(s, 1, 1, 0);     

	gsl_matrix *S = gsl_matrix_alloc(2*dms, 2*dms);
	for (int i = 0; i < 2*dms; i++) {
		for (int j = 0; j < 2*dms; j++) {	
			gsl_matrix_set(S,i,j,0);
		}
	}			

	// block diagonal
	for (int k = 0; k < dms; k++) {
		for (int i = 0; i < 2; i++) {
			for (int j = 0; j < 2; j++) {
				double q =  gsl_matrix_get(s, i, j);
				gsl_matrix_set(S, i + k*2 ,j + k*2, q);
			}
		}
	}	

	// transpose the miniblock
	if (dms == 3) {
		double tmp = gsl_matrix_get(S, 4, 5);
		gsl_matrix_set(S, 4, 5, gsl_matrix_get(S, 5, 4));		
		gsl_matrix_set(S, 5, 4, tmp);
	}

	gsl_matrix_free(s);

	return S;

}


/*double gsl_complex_abs(gsl_complex z)
{
	return pow(GSL_REAL(z)*GSL_REAL(z)+GSL_IMAG(z)*GSL_IMAG(z),0.5);
}*/	


void gsl_sqrt_complex2(gsl_complex in, gsl_complex *out)
{
	double complex c1 = GSL_REAL(in) + GSL_IMAG(in) * I;
	double complex sqrt_c1 = csqrt(c1);

	GSL_SET_COMPLEX(out, creal(sqrt_c1), cimag(sqrt_c1));
}


void eig(double *data, gsl_vector_complex *eval, gsl_matrix_complex *evec, int dim, int sort, int print) 
{
	gsl_matrix_view m
		= gsl_matrix_view_array (data, dim, dim);

	gsl_eigen_nonsymmv_workspace * w =
		gsl_eigen_nonsymmv_alloc (dim);

	gsl_eigen_nonsymmv (&m.matrix, eval, evec, w);

	gsl_eigen_nonsymmv_free (w);

	if (sort)
		gsl_eigen_nonsymmv_sort (eval, evec, GSL_EIGEN_SORT_ABS_DESC);

	if (print) {
		int i, j;
		for (i = 0; i < dim; i++) {
			gsl_complex eval_i
			   = gsl_vector_complex_get (eval, i);
			gsl_vector_complex_view evec_i
			   = gsl_matrix_complex_column (evec, i);

			printf ("eigenvalue = %g + %gi\n",
				GSL_REAL(eval_i), GSL_IMAG(eval_i));
			printf ("eigenvector = \n");
			for (j = 0; j < dim; ++j) {
				gsl_complex z =
				gsl_vector_complex_get(&evec_i.vector, j);
				printf("%.9e + %.9ei\n", GSL_REAL(z), GSL_IMAG(z));
			}
		}
	}
}



void gsl_matrix_complex_conjtranspose_memcpy(gsl_matrix_complex *V, gsl_matrix_complex *Vt)
{
	for (unsigned int i = 0; i < V->size1; i++) {
  		for (unsigned int j = 0; j < V->size2; j++) {
			gsl_complex z = gsl_matrix_complex_get(V,i,j);
			gsl_complex zc;
     		        GSL_SET_COMPLEX(&zc, GSL_REAL(z), -GSL_IMAG(z));
		 	gsl_matrix_complex_set(Vt, j, i, zc);		
		}
	}	
}


void gsl_complex_mul2(gsl_complex a, gsl_complex b, gsl_complex *c)
{
	GSL_SET_COMPLEX(c, GSL_REAL(a)*GSL_REAL(b)-GSL_IMAG(a)*GSL_IMAG(b), 
		GSL_REAL(a)*GSL_IMAG(b) +  GSL_IMAG(a)*GSL_REAL(b));
}


void gsl_matrix_complex_transpose2_memcpy(gsl_matrix_complex *V, gsl_matrix_complex *Vt)
{
	for (unsigned int i = 0; i < V->size1; i++) {
  		for (unsigned int j = 0; j < V->size2; j++) {
			gsl_complex z = gsl_matrix_complex_get(V,i,j);
			gsl_complex zc;
     		        GSL_SET_COMPLEX(&zc, GSL_REAL(z), GSL_IMAG(z));
		 	gsl_matrix_complex_set(Vt, j, i, zc);		
		}
	}	
}


void gsl_matrix_transpose2_memcpy(gsl_matrix *V, gsl_matrix *Vt)
{
	for (unsigned int i = 0; i < V->size1; i++) {
  		for (unsigned int j = 0; j < V->size2; j++) {
			double d = gsl_matrix_get(V,i,j);
		 	gsl_matrix_set(Vt, j, i, d);		
		}
	}	
}


void gsl_matrix_complex_rescale_complex(gsl_matrix_complex *V,  gsl_complex rescale, gsl_matrix_complex *Vout)
{
	for (unsigned int i = 0; i < V->size1; i++) {
  		for (unsigned int j = 0; j < V->size2; j++) {
			gsl_complex z = gsl_matrix_complex_get(V,i,j);
			gsl_complex zc;
			gsl_complex_mul2(z,rescale,&zc);     		       
		 	gsl_matrix_complex_set(Vout, i, j, zc);		
		}
	}	
}


void gsl_matrix_complex_rescale_real(gsl_matrix_complex *V,  double rescale, gsl_matrix_complex *Vout)
{
	for (unsigned int i = 0; i < V->size1; i++) {
  		for (unsigned int j = 0; j < V->size2; j++) {
			gsl_complex z = gsl_matrix_complex_get(V,i,j);
			gsl_complex zc;
     		        GSL_SET_COMPLEX(&zc, GSL_REAL(z) * rescale, GSL_IMAG(z) * rescale);
		 	gsl_matrix_complex_set(Vout, i, j, zc);		
		}
	}	
}


void gsl_matrix_complex_rescale_vector_real(gsl_matrix_complex *m,  gsl_vector *rescale, gsl_matrix_complex *mout)
{
	for (unsigned int i = 0; i < m->size1; i++) {
		for (unsigned int j = 0; j < m->size2; j++) {
			gsl_complex z = gsl_matrix_complex_get(m,i,j);
			gsl_complex zc;
			GSL_SET_COMPLEX(&zc, GSL_REAL(z) * gsl_vector_get(rescale,j), GSL_IMAG(z) * gsl_vector_get(rescale,j));
			gsl_matrix_complex_set(mout, i, j, zc);		
		}
	}	
}


void gsl_vector_complex_rescale_complex(gsl_vector_complex *V,  gsl_complex rescale, gsl_vector_complex *Vout)
{
	for (unsigned int i = 0; i < V->size; i++) {
		gsl_complex mz = gsl_vector_complex_get(V,i);
		gsl_complex zc;
		gsl_complex_mul2(mz, rescale, &zc);
		GSL_SET_COMPLEX(&zc, GSL_REAL(zc), GSL_IMAG(zc));
	 	gsl_vector_complex_set(Vout, i, zc);		
	}	
}


void gsl_matrix_complex_mul(gsl_matrix_complex *m1,  gsl_matrix_complex *m2, gsl_matrix_complex *mout)
{
	for (unsigned int i = 0; i < mout->size1; i++) {
		for (unsigned int j = 0; j < mout->size2; j++) {
			// set element i,j to 0
			gsl_complex z_sum; GSL_SET_COMPLEX(&z_sum, 0, 0);
			for (size_t k = 0; k < m1->size1; k++) {
			gsl_complex z_mul;

			gsl_complex_mul2(gsl_matrix_complex_get(m1,i,k), gsl_matrix_complex_get(m2,k,j), &z_mul);
			GSL_SET_COMPLEX(&z_sum, GSL_REAL(z_sum)+GSL_REAL(z_mul), GSL_IMAG(z_sum)+GSL_IMAG(z_mul));
			}
			gsl_matrix_complex_set(mout, i, j, z_sum);
		}
	}
}


void gsl_matrix_mul(gsl_matrix *m1,  gsl_matrix *m2, gsl_matrix *mout)
{
	for (unsigned int i = 0; i < mout->size1; i++) {
		for (unsigned int j = 0; j < mout->size2; j++) {
			// set element i,j to 0
			double sum = 0;
			for (size_t k = 0; k < m2->size1; k++) {
				double mul = gsl_matrix_get(m1,i,k) * gsl_matrix_get(m2,k,j);
				sum += mul;
			}
			gsl_matrix_set(mout, i, j, sum);
		}
	}
}



void gsl_matrix_complex_mul_one2one(gsl_matrix_complex *m1,  gsl_matrix_complex *m2, gsl_matrix_complex *mout)
{
	for (unsigned int i = 0; i < m1->size1; i++) {
		for (unsigned int j = 0; j < m1->size2; j++) {
			gsl_complex z_mul;
			gsl_complex_mul2(gsl_matrix_complex_get(m1,i,j), gsl_matrix_complex_get(m2,i,j), &z_mul);

			gsl_matrix_complex_set(mout, i, j, z_mul);

		}
	}
}


void gsl_matrix_complex_sum_col(gsl_matrix_complex *m, gsl_vector_complex *vout)
{
	for (unsigned int i = 0; i < m->size1; i++) {
		gsl_complex z_sum; GSL_SET_COMPLEX(&z_sum, 0, 0);
		for (unsigned int j = 0; j < m->size2; j++) {
		    	gsl_complex z_col = gsl_matrix_complex_get(m,j,i);
		    	GSL_SET_COMPLEX(&z_sum, GSL_REAL(z_sum)+GSL_REAL(z_col), GSL_IMAG(z_sum)+GSL_IMAG(z_col));
		}
		gsl_vector_complex_set(vout, i, z_sum);
	}
}


void gsl_matrix_complex_abs(gsl_matrix_complex *m, gsl_matrix_complex *mout)
{
	for (unsigned int i = 0; i < m->size1; i++) {
		for (unsigned int j = 0; j < m->size2; j++) {
		    	gsl_complex mz = gsl_matrix_complex_get(m,i,j);
			gsl_complex zout;
			GSL_SET_COMPLEX(&zout, gsl_complex_abs(mz), 0);
			gsl_matrix_complex_set(mout, i, j, zout);
		}
	}
}


void gsl_vector_complex_abs(gsl_vector_complex *m, gsl_vector_complex *mout)
{
	for (unsigned int i = 0; i < m->size; i++) {
		gsl_complex mz = gsl_vector_complex_get(m,i);
		gsl_complex zout;
		GSL_SET_COMPLEX(&zout, gsl_complex_abs(mz), 0);
		gsl_vector_complex_set(mout, i, zout);
	}
}


void gsl_vector_complex_abs_real(gsl_vector_complex *m, gsl_vector *mout)
{
	for (unsigned int i = 0; i < m->size; i++) {
		gsl_complex mz = gsl_vector_complex_get(m,i);
		gsl_vector_set(mout, i, gsl_complex_abs(mz));
	}
}


/*
 * Copy the real part of of a complex matrix into a new real matrix
 * gsl_matrix_complex_real is declared in GSL but uses "view" vectors
 */
void gsl_matrix_complex_sqrt(gsl_matrix_complex *m, gsl_matrix_complex *mout)
{
	for (unsigned int i = 0; i < m->size1; i++) {
		for (unsigned int j = 0; j < m->size2; j++) {
			gsl_complex zin = gsl_matrix_complex_get(m,i,j);
			gsl_complex zout;
			gsl_sqrt_complex2(zin,&zout);
			gsl_matrix_complex_set(mout, i, j, zout);
		}
	}
}


/*
 * Copy the real part of of a complex vector into a new real vector
 * gsl_vector_complex_real is declared in GSL but uses "view" vectors
 */
void gsl_vector_complex_real_at(gsl_vector_complex *m, gsl_vector *mout)
{
	for (unsigned int i = 0; i < m->size; i++) {
		gsl_complex mz = gsl_vector_complex_get(m,i);
		gsl_vector_set(mout, i, GSL_REAL(mz));
	}

}


void calculate_select_order(int dms, int *select, int *order, gsl_vector *nr)
{
  //put positive modes before negative modes (swap columns if first n in pair
  // is negative)
	for (int i = 0; i < dms; i++) {
		select[i] = i*2;
		if (gsl_vector_get(nr,i*2) > gsl_vector_get(nr,i*2+1)) {
			order[i*2] = i*2;
			order[i*2+1] = i*2+1; 
		}
		else {
			order[i*2] = i*2+1;
			order[i*2+1] = i*2;  	
		}
	}
}


void gsl_matrix_complex_order_columns(gsl_matrix_complex *m, int *order, gsl_matrix_complex *mout)
{
	// columns
	for (unsigned int i = 0; i < m->size2; i++) {
		// rows
		for (unsigned int j = 0; j < m->size1; j++) {
			gsl_complex mz = gsl_matrix_complex_get(m,i,j);
			gsl_matrix_complex_set(mout, i, order[j], mz);			
		}
	}		
}


void gsl_matrix_order_columns(gsl_matrix *m, int *order, gsl_matrix *mout)
{
	// columns
	for (unsigned int i = 0; i < m->size2; i++) {
		// rows
		for (unsigned int j = 0; j < m->size1; j++) {
			double mz = gsl_matrix_get(m,i,j);
			gsl_matrix_set(mout, i, order[j], mz);			
		}
	}		
}


void gsl_vector_complex_order(gsl_vector_complex *m, int *order, gsl_vector_complex *mout)
{
	// columns
	for (unsigned int i = 0; i < m->size; i++) {
		gsl_complex mz = gsl_vector_complex_get(m,i);
		gsl_vector_complex_set(mout, order[i], mz);			
	}		
}


void gsl_vector_order(gsl_vector *m, int *order, gsl_vector *mout)
{
	for (unsigned int i = 0; i < m->size; i++) {
		gsl_vector_set(mout, order[i], gsl_vector_get(m,i));			
	}		
}


void find_xyz_projection(gsl_matrix_complex *m, int *select, int *order)
{
	// do a copy 
	int *select_col = (int *)malloc(sizeof(int) * m->size1/2);
	int *select_row = (int *)malloc(sizeof(int) * m->size1/2);

	for (unsigned int i = 0; i < m->size1/2; i++) {
		select_row[i] = select[i];
		select_col[i] = select[i];
	}

	// select column (usually 0,2,4)
	for (unsigned int j = 0; j < m->size1/2; j++) {

		int col = select_col[j];

		double max_val = -1e100; 
		int max_row = 0;
		int max_idx_row = 0;
		for (unsigned int i = 0; i < m->size1/2; i++) {
			int row = select_row[i];
			if (row >= 0) {
				gsl_complex z = gsl_matrix_complex_get(m, row, col);
				double absz = gsl_complex_abs(z);					
				if (absz > max_val) {
					max_val = absz;
					max_row = row;
					max_idx_row = i;
				}
			}
		}	
		order[j] = max_row;
		select_row[max_idx_row] = -1; // mark as used
		
	}	
	
	
	free(select_row);
	free(select_col);

}


void gsl_matrix_complex_copy_columns(gsl_matrix_complex *m, int *select, gsl_matrix_complex *mout)
{
	// cols
	for (unsigned int i = 0; i < mout->size2; i++) {
		// rows
		for (unsigned int j = 0; j < mout->size1; j++) {
			gsl_complex mz = gsl_matrix_complex_get(m,j,select[i]);
			gsl_matrix_complex_set(mout, j, i, mz);			
		}
	}	

}


void gsl_matrix_complex_copyall_columns(gsl_matrix_complex *m, int *select, gsl_matrix_complex *mout)
{
	// cols
	for (unsigned int i = 0; i < mout->size2; i++) {
		// rows
		for (unsigned int j = 0; j < mout->size1; j++) {
			gsl_complex mz = gsl_matrix_complex_get(m,j,i);
			gsl_matrix_complex_set(mout, j, select[i], mz);			
		}
	}	

}


void gsl_matrix_copy_columns(gsl_matrix *m, int *select, gsl_matrix *mout)
{
	// cols
	for (unsigned int i = 0; i < mout->size2; i++) {
		// rows
		for (unsigned int j = 0; j < mout->size1; j++) {
			double mz = gsl_matrix_get(m,j,select[i]);
			gsl_matrix_set(mout, j, i, mz);			
		}
	}	

}


void gsl_matrix_copy_rows(gsl_matrix *m, int *select, gsl_matrix *mout)
{
	// rows
	for (unsigned int i = 0; i < mout->size1; i++) {
		// cols
		for (unsigned int j = 0; j < mout->size2; j++) {
			double mz = gsl_matrix_get(m,select[i],j);
			gsl_matrix_set(mout, i, j, mz);			
		}
	}	

}


void gsl_matrix_copy_columns_to_vector(gsl_matrix *m, int col, gsl_vector *vout)
{
	// rows
	for (unsigned int j = 0; j < m->size1; j++) {
		double mz = gsl_matrix_get(m,j,col);
		gsl_vector_set(vout, j, mz);			
	}

}

void gsl_vector_copy_matrix_columns(gsl_vector *v, int col, gsl_matrix *mout)
{
	// rows
	for (unsigned int j = 0; j < mout->size1; j++) {
		double mz = gsl_vector_get(v,j);
		gsl_matrix_set(mout, j, col, mz);			
	}

}



void gsl_matrix_copyall_columns(gsl_matrix *m, int *select, gsl_matrix *mout)
{
	// cols
	for (unsigned int i = 0; i < m->size2; i++) {
		// rows
		for (unsigned int j = 0; j < m->size1; j++) {
			double mz = gsl_matrix_get(m,j,i);
			gsl_matrix_set(mout, j, select[i], mz);			
		}
	}	
}


void gsl_matrix_copy(gsl_matrix *m,  gsl_matrix *mout)
{
	// cols
	for (unsigned int i = 0; i < m->size1; i++) {
		// rows
		for (unsigned int j = 0; j < m->size2; j++) {
			double mz = gsl_matrix_get(m,i,j);
			gsl_matrix_set(mout, i, j, mz);			
		}
	}	
}


void gsl_vector_complex_copy_elem(gsl_vector_complex *m, int *select, gsl_vector_complex *mout)
{
	for (unsigned int i = 0; i < mout->size; i++) {
		gsl_complex mz = gsl_vector_complex_get(m,select[i]);
		gsl_vector_complex_set(mout, i, mz);			
	}	

}


void gsl_vector_copy_elem(gsl_vector *m, int *select, gsl_vector *mout)
{
	for (unsigned int i = 0; i < mout->size; i++) {
		double mz = gsl_vector_get(m,select[i]);
		gsl_vector_set(mout, i, mz);			
	}	

}



double gsl_matrix_abs_diff(gsl_matrix *m1, gsl_matrix *m2)
{
	double sum = 0;
	for (unsigned int i = 0; i < m1->size1; i++) {
		for (unsigned int j = 0; j < m1->size2; j++) {
		        double m1_d = gsl_matrix_get(m1,i,j); 
		        double m2_d = gsl_matrix_get(m2,i,j);
		        // the sign of the eigenvector is arbitray so the error
		        // depends on the difference between the abs's instead
		        // the abs of the differences
			sum += fabs(m1_d) - fabs(m2_d);
		}		
	}	
	return sum;
}


//build the a-matrix
//a=reshape([real(V_ordered);imag(V_ordered)],nv,nv);
void build_a_matrix(gsl_matrix_complex *m, gsl_matrix *a)
{
	for (unsigned int i = 0; i < m->size1; i++) {
		for (unsigned int j = 0; j < m->size2; j++) {
			gsl_complex mz = gsl_matrix_complex_get(m,i,j);
			gsl_matrix_set(a, i, j*2, GSL_REAL(mz));
			gsl_matrix_set(a, i, j*2+1, GSL_IMAG(mz));
		}				
	}	
}


void gsl_linear_solve(gsl_matrix *Ain, gsl_vector *bin, gsl_vector *xg) 
{

	gsl_matrix *Ain_bk = gsl_matrix_alloc(Ain->size1,Ain->size2);
	
	// there is a funcking bug that corrupt A matrix after the execution of
	// gsl_linalg_LU_solve
	gsl_matrix_copy(Ain, Ain_bk);

	// Allocate workspace for the solver
	gsl_permutation *p = gsl_permutation_alloc(Ain->size2);

	// Solve the system
	int s;
	gsl_linalg_LU_decomp(Ain, p, &s);
	gsl_linalg_LU_solve(Ain, p, bin, xg);

	gsl_matrix_copy(Ain_bk, Ain);
	
	gsl_matrix_free(Ain_bk);
	
	// Clean up
	gsl_permutation_free(p);

}


void gsl_linear_solve_matrix(gsl_matrix *Ain, gsl_matrix *bin, gsl_matrix *xg) 
{

	gsl_vector *bin_v = gsl_vector_alloc(bin->size1);
	gsl_vector *xg_v = gsl_vector_alloc(xg->size1);
	
	for (unsigned int i = 0; i < bin->size2; i++) {
		gsl_matrix_copy_columns_to_vector(bin, i, bin_v);
		gsl_linear_solve(Ain, bin_v, xg_v);
		gsl_vector_copy_matrix_columns(xg_v, i, xg);
	}
	
	gsl_vector_free(bin_v);
	gsl_vector_free(xg_v);

}


void gsl_matrix_square_submatrix(gsl_matrix *a, int *indexes, gsl_matrix *a22)
{
	for (int i = indexes[0]; i <= indexes[1]; i++) {
		for (int j = indexes[0]; j <= indexes[1]; j++) {	
			gsl_matrix_set(a22, i-indexes[0] , j - indexes[0], gsl_matrix_get(a, i, j));
		}	
	}
}


void gsl_matrix_selection_mul(gsl_matrix *m1, gsl_matrix *m2, int *slc, int slc_size, gsl_matrix *mout)
{
	gsl_matrix *m1_red = gsl_matrix_alloc(m1->size1, slc_size);
	gsl_matrix *m2_red = gsl_matrix_alloc(slc_size, m2->size2);
		
	
	gsl_matrix_copy_columns(m1, slc, m1_red);
        gsl_matrix_copy_rows(m2, slc, m2_red);

	gsl_matrix_mul(m1_red, m2_red, mout);
	
	gsl_matrix_free(m1_red);
	gsl_matrix_free(m2_red);
}


void gsl_matrix_identity(gsl_matrix *mi)
{
	int size;
	if (mi->size1 <= mi->size2)
		size = mi->size1;
	else
		size = mi->size2;		

	// first set all values to 0
	for (unsigned int i = 0; i < mi->size1; i++) {
		for (unsigned int j = 0; j < mi->size2; j++) {
			gsl_matrix_set(mi, i, j, 0);
		}
	}				

	for (int i = 0; i < size; i++) {
		gsl_matrix_set(mi, i, i, 1);
	}	

}


void gsl_convert_vector_complex_rect_to_polar(gsl_vector_complex *in, gsl_vector *r, gsl_vector *theta)
{
	for (unsigned int i = 0; i < in->size; i++) {
		gsl_complex z = gsl_vector_complex_get(in, i);
		double r_d, theta_d;
		gsl_convert_complex_rect_to_polar(z, &r_d, &theta_d);
		gsl_vector_set(r, i, r_d);
		gsl_vector_set(theta, i, theta_d);
	}
} 



void gsl_convert_complex_rect_to_polar(gsl_complex in, double *r, double *theta)
{
	double a = GSL_REAL(in), b = GSL_IMAG(in);
	*r = pow(a * a + b * b, 0.5);
	*theta = atan2(b,a);
} 

/*double gsl_complex_abs3(gsl_complex z)
{
	return pow(GSL_REAL(z)*GSL_REAL(z)+GSL_IMAG(z)*GSL_IMAG(z),0.5);
}*/	

void gsl_vector_complex_logabs(gsl_vector_complex *in, gsl_vector *out)
{
	for (unsigned int i = 0; i < in->size; i++) {
		gsl_complex z = gsl_vector_complex_get(in, i);
		gsl_vector_set(out, i, log(gsl_complex_abs(z)));

	}
} 

void gsl_matrix_inv(gsl_matrix *in, gsl_matrix *out)
{
    gsl_permutation *p = gsl_permutation_alloc(in->size1); // matrix must be square
    int s;

    gsl_matrix *in_bk = gsl_matrix_alloc(in->size1,in->size2);
    gsl_matrix_copy(in, in_bk);	

    // Compute the LU decomposition of this matrix
    gsl_linalg_LU_decomp(in, p, &s);

    // Compute the  inverse of the LU decomposition
    gsl_linalg_LU_invert(in, p, out);
    
    gsl_matrix_copy(in_bk, in);
    gsl_matrix_free(in_bk);

    gsl_permutation_free(p);

}


void matrix_mul(double *m1,  double *m2, int N, double *mout)
{
	for (int i = 0; i < N; i++) {
		for (int j = 0; j < N; j++) {
			// set element i,j to 0
			double sum = 0;
			for (int k = 0; k < N; k++) {
				//double mul = gsl_matrix_get(m1,i,k) * gsl_matrix_get(m2,k,j);
				double mul = m1[i*N+k] * m2[k*N+j];
				sum += mul;
			}
			mout[i*N+j] = sum;
		}
	}
}





void matrix_mul_t1(double *m1_t,  double *m2, int N, double *mout)
{
	for (int i = 0; i < N; i++) {
		for (int j = 0; j < N; j++) {
			// set element i,j to 0
			double sum = 0;
			for (int k = 0; k < N; k++) {
				//double mul = gsl_matrix_get(m1,i,k) * gsl_matrix_get(m2,k,j);
				double mul = m1_t[k*N+i] * m2[k*N+j];
				sum += mul;
			}
			mout[i*N+j] = sum;
		}
	}
}


void matrix_mul_t1_tout(double *m1_t,  double *m2, int N, double *mout)
{
	for (int i = 0; i < N; i++) {
		for (int j = 0; j < N; j++) {
			// set element i,j to 0
			double sum = 0;
			for (int k = 0; k < N; k++) {
				//double mul = gsl_matrix_get(m1,i,k) * gsl_matrix_get(m2,k,j);
				double mul = m1_t[k*N+i] * m2[k*N+j];
				sum += mul;
			}
			mout[j*N+i] = sum;
		}
	}
}


void matrix_mul_t2(double *m1,  double *m2_t, int N, double *mout)
{
	for (int i = 0; i < N; i++) {
		for (int j = 0; j < N; j++) {
			// set element i,j to 0
			double sum = 0;
			for (int k = 0; k < N; k++) {
				//double mul = gsl_matrix_get(m1,i,k) * gsl_matrix_get(m2,k,j);
				double mul = m1[i*N+k] * m2_t[j*N+k];
				sum += mul;
			}
			mout[i*N+j] = sum;
		}
	}
}


void matrix_mul_t2_tout(double *m1,  double *m2_t, int N, double *mout)
{
	for (int i = 0; i < N; i++) {
		for (int j = 0; j < N; j++) {
			// set element i,j to 0
			double sum = 0;
			for (int k = 0; k < N; k++) {
				//double mul = gsl_matrix_get(m1,i,k) * gsl_matrix_get(m2,k,j);
				double mul = m1[i*N+k] * m2_t[j*N+k];
				sum += mul;
			}
			mout[j*N+i] = sum;
		}
	}
}


void matrix_mul_t1_t2(double *m1_t,  double *m2_t, int N, double *mout)
{
	for (int i = 0; i < N; i++) {
		for (int j = 0; j < N; j++) {
			// set element i,j to 0
			double sum = 0;
			for (int k = 0; k < N; k++) {
				//double mul = gsl_matrix_get(m1,i,k) * gsl_matrix_get(m2,k,j);
				double mul = m1_t[k*N+i] * m2_t[j*N+k];
				sum += mul;
			}
			mout[i*N+j] = sum;
		}
	}
}


void matrix_mul_tout(double *m1,  double *m2, int N, double *mout)
{
	for (int i = 0; i < N; i++) {
		for (int j = 0; j < N; j++) {
			// set element i,j to 0
			double sum = 0;
			for (int k = 0; k < N; k++) {
				//double mul = gsl_matrix_get(m1,i,k) * gsl_matrix_get(m2,k,j);
				double mul = m1[i*N+k] * m2[k*N+j];
				sum += mul;
			}
			mout[j*N+i] = sum;
		}
	}
}


void lapack_schur_factor(double *A, double *U, double *T, int N, int order)
{
    int n = N, lda = N, info;
    memcpy(T, A, N * N * sizeof(double));
    double wr[N], wi[N];
    int sdim;
    int lvds = N; 

    
    // Call dgees to perform Schur factorization
    info = LAPACKE_dgees(order, 'V', 'N', 
                         NULL, n, T, 
                         lda, &sdim, wr, 
                         wi, U, lvds);
}


double *lapack_solve_sylvester(double *A, double *B, double *C, int n, int order)
{

	double *QA = malloc(n * n * sizeof(double));
	double *TA = malloc(n * n * sizeof(double));
	double *QB = malloc(n * n * sizeof(double));
	double *TB = malloc(n * n * sizeof(double));
	double *QAC = malloc(n * n * sizeof(double));
	double *QACQB = malloc(n * n * sizeof(double));
	double *QAX = malloc(n * n * sizeof(double));
	double *QAXQB = malloc(n * n * sizeof(double));
	double scale;
	int isgn = 1;

	// check sylvester.m Matlab source file
	lapack_schur_factor(A, QA, TA, n, order);
	lapack_schur_factor(B, QB, TB, n, order);

	matrix_mul_t1(QA, C, n, QAC);
        matrix_mul(QAC, QB, n, QACQB);

	LAPACKE_dtrsyl(order, 'N', 'N', isgn, n , n, TA, n, TB, n, QACQB, n, &scale);  

	matrix_mul(QA, QACQB, n, QAX);
	matrix_mul_t2(QAX, QB, n, QAXQB);

	free(QA);
	free(TA);
	free(QB);
	free(TB);
	free(QAC);
	free(QACQB);
	free(QAX);

	return QAXQB;

}










