#ifndef AT_MATH_H
#define AT_MATH_H

#include <stdio.h>
#include <string.h> 

#include <gsl/gsl_matrix.h>
#include <gsl/gsl_complex.h>
#include <gsl/gsl_complex_math.h>
#include <gsl/gsl_linalg.h>
#include <gsl/gsl_eigen.h>

#include <lapacke_utils.h>

#include <complex.h>
#include <math.h>

void print_complex(gsl_complex z);
void print_matrix(gsl_matrix *A);
void print_vector(gsl_vector *A);
void print_matrix_complex(gsl_matrix_complex *A);
void print_vector_complex(gsl_vector_complex *A);
gsl_matrix *gsl_matrix_alloc_p(double *vector, const int N, const int M);
gsl_matrix *gsl_matrix_alloc_p_trasp(double *vector, const int N, const int M);
gsl_matrix_complex *gsl_matrix_complex_alloc_p(double *vector, const int N, const int M);
gsl_matrix_complex *gsl_matrix_complex_alloc_p_trasp(double *vector, const int N, const int M);
gsl_matrix_complex *jmat_complex_alloc(int dms);
gsl_matrix *jmat_alloc(int dms);
gsl_matrix_complex *kmat_complex_alloc(int dms);
gsl_matrix *kmat_alloc(int dms);
//double abs_complex(gsl_complex z);
void gsl_sqrt_complex2(gsl_complex in, gsl_complex *out);
void eig(double *data, gsl_vector_complex *eval, gsl_matrix_complex *evec, int dim, int sort, int print);
void gsl_matrix_complex_conjtranspose_memcpy(gsl_matrix_complex *V, gsl_matrix_complex *Vt);
void gsl_complex_mul2(gsl_complex a, gsl_complex b, gsl_complex *c);
void gsl_matrix_complex_transpose2_memcpy(gsl_matrix_complex *V, gsl_matrix_complex *Vt);
void gsl_matrix_transpose2_memcpy(gsl_matrix *V, gsl_matrix *Vt);
void gsl_matrix_complex_rescale_complex(gsl_matrix_complex *V,  gsl_complex rescale, gsl_matrix_complex *Vout);
void gsl_matrix_complex_rescale_real(gsl_matrix_complex *V,  double rescale, gsl_matrix_complex *Vout);
void gsl_matrix_complex_rescale_vector_real(gsl_matrix_complex *m,  gsl_vector *rescale, gsl_matrix_complex *mout);
void gsl_vector_complex_rescale_complex(gsl_vector_complex *V,  gsl_complex rescale, gsl_vector_complex *Vout);
void gsl_matrix_complex_mul(gsl_matrix_complex *m1,  gsl_matrix_complex *m2, gsl_matrix_complex *mout);
void gsl_matrix_mul(gsl_matrix *m1,  gsl_matrix *m2, gsl_matrix *mout);
void gsl_matrix_complex_mul_one2one(gsl_matrix_complex *m1,  gsl_matrix_complex *m2, gsl_matrix_complex *mout);
void gsl_matrix_complex_sum_col(gsl_matrix_complex *m, gsl_vector_complex *vout);
void gsl_matrix_complex_abs(gsl_matrix_complex *m, gsl_matrix_complex *mout);
void gsl_vector_complex_abs(gsl_vector_complex *m, gsl_vector_complex *mout);
void gsl_vector_complex_abs_real(gsl_vector_complex *m, gsl_vector *mout);
void gsl_matrix_complex_sqrt(gsl_matrix_complex *m, gsl_matrix_complex *mout);
void gsl_vector_complex_real_at(gsl_vector_complex *m, gsl_vector *mout);
void calculate_select_order(int dms, int *select, int *order, gsl_vector *nr);
void gsl_matrix_complex_order_columns(gsl_matrix_complex *m, int *order, gsl_matrix_complex *mout);
void gsl_matrix_order_columns(gsl_matrix *m, int *order, gsl_matrix *mout);
void gsl_vector_complex_order(gsl_vector_complex *m, int *order, gsl_vector_complex *mout);
void gsl_vector_order(gsl_vector *m, int *order, gsl_vector *mout);
void find_xyz_projection(gsl_matrix_complex *m, int *select, int *order);
void gsl_matrix_complex_copy_columns(gsl_matrix_complex *m, int *select, gsl_matrix_complex *mout);
void gsl_matrix_complex_copyall_columns(gsl_matrix_complex *m, int *select, gsl_matrix_complex *mout);
void gsl_matrix_copy_columns(gsl_matrix *m, int *select, gsl_matrix *mout);
void gsl_matrix_copy_rows(gsl_matrix *m, int *select, gsl_matrix *mout);
void gsl_matrix_copy_columns_to_vector(gsl_matrix *m, int col, gsl_vector *vout);
void gsl_vector_copy_matrix_columns(gsl_vector *v, int col, gsl_matrix *mout);
void gsl_matrix_copyall_columns(gsl_matrix *m, int *select, gsl_matrix *mout);
void gsl_matrix_copy(gsl_matrix *m,  gsl_matrix *mout);
void gsl_vector_complex_copy_elem(gsl_vector_complex *m, int *select, gsl_vector_complex *mout);
void gsl_vector_copy_elem(gsl_vector *m, int *select, gsl_vector *mout);
double gsl_matrix_abs_diff(gsl_matrix *m1, gsl_matrix *m2);
void build_a_matrix(gsl_matrix_complex *m, gsl_matrix *a);
void gsl_linear_solve(gsl_matrix *Ain, gsl_vector *bin, gsl_vector *xg);
void gsl_linear_solve_matrix(gsl_matrix *Ain, gsl_matrix *bin, gsl_matrix *xg);
void gsl_matrix_square_submatrix(gsl_matrix *a, int *indexes, gsl_matrix *a22);
void gsl_matrix_selection_mul(gsl_matrix *m1, gsl_matrix *m2, int *slc, int slc_size, gsl_matrix *mout);
void gsl_matrix_identity(gsl_matrix *mi);
void gsl_convert_complex_rect_to_polar(gsl_complex in, double *r, double *theta);
void gsl_convert_vector_complex_rect_to_polar(gsl_vector_complex *in, gsl_vector *r, gsl_vector *theta);
void gsl_vector_complex_logabs(gsl_vector_complex *in, gsl_vector *out);
/*double gsl_complex_abs3(gsl_complex z);*/
void gsl_matrix_inv(gsl_matrix *in, gsl_matrix *out);


void lapack_schur_factor(double *A, double *U, double *T, int N, int order);
double *lapack_solve_sylvester(double *A, double *B, double *C, int n, int order);
void matrix_mul(double *m1,  double *m2, int N, double *mout);
void matrix_mul_t1(double *m1_t,  double *m2, int N, double *mout);
void matrix_mul_t2(double *m1,  double *m2_t, int N, double *mout);
void matrix_mul_t1_t2(double *m1_t,  double *m2_t, int N, double *mout);
void matrix_mul_t1_tout(double *m1_t,  double *m2, int N, double *mout_t);
void matrix_mul_t2_tout(double *m1,  double *m2_t, int N, double *mout_t);
void matrix_mul_tout(double *m1,  double *m2_2, int N, double *mout);


#endif

