#include "cat.h"


char pass_method_list[AT_NUM_PASS_METHODS][40] = {
"DriftPass",
"BndMPoleSymplectic4Pass",
"StrMPoleSymplectic4Pass",
"BndStrMPoleSymplectic4Pass",
"CorrectorPass",
"IdentityPass",
"CavityPass",
"RFCavityPass",
"ThinMPolePass",
"EAperturePass",
"BndMPoleSymplectic4RadPass",
"StrMPoleSymplectic4RadPass",
"BndStrMPoleSymplectic4RadPass",
"GWigSymplecticPass",
"GWigSymplecticRadPass",
"QuadLinearPass",
"BendLinearPass"
};


char elem_class_list[AT_NUM_ELEM_CLASS][40] = {
"Corrector",
"Marker",
"Monitor",
"Sextupole",
"Quadrupole",
"Atsbend",
"Atrbend",
"Thinmpole",
"Drift",
"Rfcavity",
"Wiggler"
};



char *trim_comma(char *text)
{
    int count = 0;
 
    for (int i = 0; text[i] != '\0'; i++)
    {
    	// removed space and single comma chars from the configuration string
        if ((text[i] != '?') && (text[i] != '\'')) 
        {
            text[count] = text[i];
            count++;
        }    
    }
    
    text[count] = '\0';
    return text;	
}


void unwrap_array(double *in, double *out, int len) {
    out[0] = in[0];
    for (int i = 1; i < len; i++) {
        float d = in[i] - in[i-1];
        d = d > M_PI ? d - 2 * M_PI : (d < -M_PI ? d + 2 * M_PI : d);
        out[i] = out[i-1] + d;
    }
}


void create_poly_array(char *str, int num_zeros, double last_val) 
{
	sprintf(str,"0 ");
	for (int i = 0; i < num_zeros-1; i++) {
		sprintf(str,"0 %s",str);	
	}

	if (!isnan(last_val)) {
		str[strlen(str)-1] = '\0';
		sprintf(str,"%s %.18e",str,last_val);
	} 
	else {
		str[strlen(str)-1] = '\0';
	}

}


void create_poly_idx_array(char *str, int size_pol, int idx, double last_val) 
{
	if (size_pol == 1) {
		sprintf(str,"%.18e",last_val);
		return;
	}
	else {
		sprintf(str,"0 ");
	}

	for (int i = 1; i < size_pol; i++) {
		if (idx == i)
			sprintf(str,"%s %.18e",str,last_val);
		else
			sprintf(str,"%s 0",str);
	}

}


int substr_match(const char *str, const char *sub) 
{
    int len = strlen(sub);
    if (len == 0) {
        return 0;
    }
    while (*str != '\0') {
        if (strncmp(str, sub, len) == 0) {
            return 1;
        }
        str++;
    }
    return 0;
}


char *replace_str(char *str, char *orig, char *rep)
{
  static char buffer[4096];
  char *p;

  if(!(p = strstr(str, orig)))  // Is 'orig' even in 'str'?
    return str;

  strncpy(buffer, str, p-str); // Copy characters from 'str' start to 'orig' st$
  buffer[p-str] = '\0';

  sprintf(buffer+(p-str), "%s%s", rep, p+strlen(orig));

  return buffer;
}


char *text_atField(char *matrix, int *rows, int *cols)
{
	int num_elem = 0;
	*rows = 1;
	*cols = 1;
	int i = 0, n_digits = 0;

	while (matrix[i] != 0) {
		// first remove the brackets
		if ((matrix[i] == '[') || (matrix[i] == ']')) {
			matrix[i] = ' ';
		}		
		else if (matrix[i] == ';') {
			(*rows)++;
			matrix[i] = ' ';
		}
		else {
			n_digits++;
		}	
		i++;
	}

	i = 0;
	while (matrix[i] != 0) {
		if ((matrix[i+1] != 0)  && (matrix[i] != ' ') && (matrix[i+1] == ' '))
			num_elem ++;
		i++;
	}

	if ((num_elem  == 0) && (n_digits > 0))
		num_elem  = 1;
	
	*cols = num_elem  / *rows;
	
	return matrix;
	
} 
   

char **parse_atConfigElem(char *configuration, int *num_elem) 
{ 
	char **string = (char **)malloc((strlen(configuration)+1) * sizeof (int*));
	char delimit[]="(),"; 
	unsigned int i = 0;
	
	configuration = trim_comma(configuration);
	//configuration = replace_str(configuration,", ",",");
	//configuration = replace_str(configuration," ,",",");

	i = 0;
	string[i]=strtok(configuration, delimit); 
	
	while (string[i] != 0) {
		i++;
		if (i > (strlen(configuration)+1) * sizeof (int*))
			break;
		string[i] = strtok(NULL,delimit);
		string[i] = (string[i]);
	}
	
	*num_elem = i;
	
	return string;
}


int is_atFieldName(char *fieldname) 
{ 
	
	return isalpha(fieldname[0]); 		     
}



int is_atPassMethodField(char *fieldname) 
{ 

	return 1 - strcmp("PassMethod", fieldname); 		     
	
}



int is_atPassMethodName(char *pass_method_name) 
{ 
	for (int i = 0; i < AT_NUM_PASS_METHODS; i++) {
		if (strcmp(pass_method_list[i], pass_method_name) == 0)
			return 1;
	}	
	return 0;		     
	
}


int is_atElemClassName(char *elem_class_name) 
{ 
	for (int i = 0; i < AT_NUM_ELEM_CLASS; i++) {
		if (strcmp(elem_class_list[i], elem_class_name) == 0)
			return 1;
	}	
	return 0;		     
	
}


char *find_PassMethod(char **strings, int num_elem)
{
	for (int i = 0; i < num_elem; i++) {
		for (int j = 0; j < AT_NUM_PASS_METHODS; j++) {
			if (strcmp(strings[i],pass_method_list[j]) == 0)
				return strings[i];
		}		
	}
	return NULL;
}


char *get_atFieldValue(char **strings, int num_elem, const char *fieldname)
{
	for (int i = 0; i < num_elem; i++) {
		if (strcmp(strings[i],(char *)fieldname) == 0) {
			return strings[i+1];
		}		
	}
	return NULL;
}



void *get_atPassPtr(const char *pass_method_path, const char *pass_method_name) 
{ 
	char lib_pass_path[100];
	
	snprintf(lib_pass_path, 100, "%s/lib%s.so", pass_method_path, pass_method_name);

	void *handle = dlopen(lib_pass_path, RTLD_LAZY);
	if (!handle) {
		fprintf(stderr, "Failed to open library: %s\n", dlerror());
		return NULL;
	}
	
	void (*pass_track) (const mxArray *ElemData, struct elem *Elem, 
			     double *r_in, int num_particles, struct parameters *Param) = dlsym(handle, "trackFunction");
			     
			     
			     
	return (void *)pass_track;			     
	
}


mxArray *init_pass_atElem(mxArray *atProp, mxArray *atElem, const char *pass_method_name) 
{
	struct parameters *Param = (struct parameters *) malloc(sizeof(struct parameters));
	memset(Param,0,sizeof(struct parameters));
		
	track_function_ptr pass_track = (track_function_ptr) get_atPassPtr((const char *)atGetPointer(atProp,"PassMethodPath"),(const char *)pass_method_name);

	if (pass_track) {
		atElem = add_mxArray(atElem, "PassMethod", AT_CHAR, 1, 1, (void*)pass_method_name);
		atElem = add_mxArray(atElem, "TrackFunction", AT_FUNCTION, 1, 1, (void*)pass_track);

		// Now initialize the wrapper of the track function to get back the pointer of the data struct containing
		// the values passed to the "real" track function
		track_function_ptr fptr = (track_function_ptr) atGetPointer(atElem,"TrackFunction");
		double r_in[6] = {0,0,0,0,0,0};

	 	// execute for the first time the pass method to get back the elem structure pointer
	 	// and store the elem structure pointer into atElem
	 	// the malloc is performed in the pass track function
		atElem = add_mxArray(atElem, "Elem", AT_POINTER_F, 1, 1, fptr(atElem, NULL, r_in, 0, Param));
	}

	if (substr_match(pass_method_name,"Rad")) {
		uint32_t *NumRadElem = atGetPointer(atProp, "NumRadElem");
		(*NumRadElem) += 1;
	}
	else if ((substr_match(pass_method_name,"RFCavityPass")) || (substr_match(pass_method_name,"CavityPass"))) {
		uint32_t *NumRFElem = atGetPointer(atProp, "NumRFElem");
		(*NumRFElem) += 1;
	}	
	
	

	free(Param);
	
	return atElem;

}


mxArray *init_elem_atElem(mxArray *atElem) 
{
	struct parameters *Param = (struct parameters *) malloc(sizeof(struct parameters));
	memset(Param,0,sizeof(struct parameters));
			
	// Now initialize the wrapper of the track function to get back the pointer of the data struct containing
	// the values passed to the "real" track function
	track_function_ptr fptr = (track_function_ptr) atGetPointer(atElem,"TrackFunction");
	double r_in[6] = {0,0,0,0,0,0};
 	// execute for the first time the pass method to get back the elem structure pointer
 	// and store the elem structure pointer into atElem
 	// the malloc is performed in the pass track function
	destroy_field_mxArray(atElem, "Elem");
	atElem = add_mxArray(atElem, "Elem", AT_POINTER_F, 1, 1, fptr(atElem, NULL, r_in, 0, Param));
	free(Param);
	
	return atElem;

}


// SOLEIL: atmarker('SECT1');";
mxArray *create_marker_atElem(mxArray *atProp, char *configuration) 
{
	int num_fields = 0, cnt = 0, rows, cols;

	// these are the fields not passed through configuration file for the corrector
	char **field_ptr = parse_atConfigElem(configuration, &num_fields);

	// mandatory fields common to any element
	mxArray *atElem = create_mxArray("Class", AT_CHAR, 1, 1, (void*)"Marker");cnt++;
	atElem = add_mxArray(atElem, "Fname", AT_CHAR, 1, 1,field_ptr[1]);cnt++;
	
	// open the shared library and get the pass method function symbol
	// optional parameters of the element
	for (int j = cnt; j < num_fields; j++) {
		// do not pickup here the pass method
		if (!is_atPassMethodName(field_ptr[j]) && is_atFieldName(field_ptr[j]) && (substr_match(field_ptr[j], "PassMethod") == 0)) {		
			if (is_atFieldName(field_ptr[j+1]) == 0) 
				atElem = add_mxArray(atElem, field_ptr[j], AT_DOUBLE_STR, rows, cols, text_atField(field_ptr[j+1],&rows, &cols));
			else 
				atElem = add_mxArray(atElem, field_ptr[j], AT_CHAR, 1, 1, field_ptr[j+1]);
			j++;
		}
	}

	// add length if not defined
	double Length = 0;
	atElem = add_mxArray(atElem, "Length", AT_DOUBLE, 1, 1, &Length);

	// initialize pass method, scan all configuration string to find the pass method
	if (!find_PassMethod(field_ptr, num_fields))
		atElem = init_pass_atElem(atProp, atElem, "IdentityPass");
	else	
		atElem = init_pass_atElem(atProp ,atElem, find_PassMethod(field_ptr, num_fields));

	free(field_ptr);

	return atElem;

}


// ESRF: atmonitor('BPM');
mxArray *create_monitor_atElem(mxArray *atProp, char *configuration) 
{
	int num_fields = 0, cnt = 0, rows, cols;

	// these are the fields not passed through configuration file for the corrector
	char **field_ptr = parse_atConfigElem(configuration, &num_fields);

	// mandatory fields common to any element
	mxArray *atElem = create_mxArray("Class", AT_CHAR, 1, 1, (void*)"Monitor");cnt++;
	atElem = add_mxArray(atElem, "Fname", AT_CHAR, 1, 1,field_ptr[1]);cnt++;
	
	// open the shared library and get the pass method function symbol
	// optional parameters of the element
	for (int j = cnt; j < num_fields; j++) {
		// do not pickup here the pass method
		if (!is_atPassMethodName(field_ptr[j]) && is_atFieldName(field_ptr[j]) && (substr_match(field_ptr[j], "PassMethod") == 0)) {		
			if (is_atFieldName(field_ptr[j+1]) == 0) 
				atElem = add_mxArray(atElem, field_ptr[j], AT_DOUBLE_STR, rows, cols, text_atField(field_ptr[j+1],&rows, &cols));
			else 
				atElem = add_mxArray(atElem, field_ptr[j], AT_CHAR, 1, 1, field_ptr[j+1]);
			j++;
		}
	}

	// add length if not defined
	double Length = 0;
	atElem = add_mxArray(atElem, "Length", AT_DOUBLE, 1, 1, &Length);

	// initialize pass method, scan all configuration string to find the pass method
	if (!find_PassMethod(field_ptr, num_fields))
		atElem = init_pass_atElem(atProp, atElem, "IdentityPass");
	else	
		atElem = init_pass_atElem(atProp, atElem, find_PassMethod(field_ptr, num_fields));

	free(field_ptr);

	return atElem;

}


// SOLEIL: atcorrector('FCOR',0,[0 0],'PolynomA',0,'PolynomB',0,'NumIntSteps',1,'MaxOrder',1,'Energy',2739100000);
mxArray *create_corrector_atElem(mxArray *atProp, char *configuration) 
{
	int num_fields = 0, cnt = 0, rows, cols;

	// these are the fields not passed through configuration file for the corrector
	char **field_ptr = parse_atConfigElem(configuration, &num_fields);


	// mandatory fields common to any element
	mxArray *atElem = create_mxArray("Class", AT_CHAR, 1, 1, (void*)"Corrector");cnt++;
	atElem = add_mxArray(atElem, "Fname", AT_CHAR, 1, 1,field_ptr[1]);cnt++;
	
	// fields that are mandatory in the configuration file
	if (is_atFieldName(field_ptr[2]) == 0) {
		atElem = add_mxArray(atElem, "Length", AT_DOUBLE_STR, rows, cols, text_atField(field_ptr[2],&rows, &cols));cnt++;
	}
	
	// fields that are mandatory in the configuration file
	if (is_atFieldName(field_ptr[3]) == 0) {
		atElem = add_mxArray(atElem, "KickAngle", AT_DOUBLE_STR, rows, cols, text_atField(field_ptr[3],&rows, &cols));cnt++;
	}

	if (get_atFieldValue(field_ptr, num_fields, "MaxOrder")) {
		atElem = add_mxArray(atElem, "MaxOrder", AT_DOUBLE_STR, 1, 1, get_atFieldValue(field_ptr, num_fields, "MaxOrder"));
		uint32_t MaxOrder = atGetLong(atElem, "MaxOrder");
		char polb[80], pola[80];
		create_poly_array(polb, MaxOrder+1, NAN); // NAN: do not add any value at the end of the array
		create_poly_array(pola, MaxOrder+1, NAN);
		atElem = add_mxArray(atElem, "PolynomB", AT_DOUBLE_STR, 1, MaxOrder+1, polb);
		atElem = add_mxArray(atElem, "PolynomA", AT_DOUBLE_STR, 1, MaxOrder+1, pola);
	}
	else {
		if (!get_atFieldValue(field_ptr, num_fields, "PolynomB") && !get_atFieldValue(field_ptr, num_fields, "PolynomA")) {
			char pol[] = "0 0"; 
			atElem = add_mxArray(atElem, "PolynomB", AT_DOUBLE_STR, 1, 2, (void*)pol);
			atElem = add_mxArray(atElem, "PolynomA", AT_DOUBLE_STR, 1, 2, (void*)pol);
			uint32_t MaxOrder = 1;
			atElem = add_mxArray(atElem, "MaxOrder", AT_UINT32, 1, 1, &MaxOrder);	
		}	
	}	
	
	uint32_t MaxOrder = 1;
	for (int j = cnt; j < num_fields; j+=2) {			
		// do not pickup here the pass method
		if (!is_atPassMethodName(field_ptr[j]) && is_atFieldName(field_ptr[j]) && 
			(substr_match(field_ptr[j], "PassMethod") == 0)) {
			if (isalpha(field_ptr[j+1][0]) == 0) // if field value is a number convert it into double	
				atElem = add_mxArray(atElem, field_ptr[j], AT_DOUBLE_STR, rows, cols, text_atField(field_ptr[j+1],&rows, &cols));
			else // otherwise save it as text
				atElem = add_mxArray(atElem, field_ptr[j], AT_CHAR, 1, 1, (void*)field_ptr[j+1]);	
			// MaxOrder
			if (substr_match(field_ptr[j], "Polynom")) {
				if (rows > MaxOrder)
					MaxOrder = rows;
				if (cols > MaxOrder)
					MaxOrder = cols;						
			}
		}
	}



	// if NumIntStep is not defined, set default value to 10
	if (get_atFieldValue(field_ptr, num_fields, "NumIntSteps") == NULL) { 
		uint32_t NumIntSteps = 1;
		atElem = add_mxArray(atElem, "NumIntSteps", AT_UINT32, 1, 1, &NumIntSteps);
	}
	if (get_atFieldValue(field_ptr, num_fields, "MaxOrder") == NULL) { 
		atElem = add_mxArray(atElem, "MaxOrder", AT_UINT32, 1, 1, &MaxOrder);	
	}
	// add energy field to all magnets
	if (get_atFieldValue(field_ptr, num_fields, "Energy") == NULL) { 
		double Energy = 0;
		atElem = add_mxArray(atElem, "Energy", AT_DOUBLE, 1, 1, &Energy);
	}

	// initialize pass method, scan all configuration string to find the pass method
	if (!find_PassMethod(field_ptr, num_fields))
		atElem = init_pass_atElem(atProp, atElem, "CorrectorPass");
	else	
		atElem = init_pass_atElem(atProp, atElem, find_PassMethod(field_ptr, num_fields));

	free(field_ptr);


	return atElem;

}


// SOLEIL: atsextupole('S7',1e-08,-496000000,'Energy',2739100000);
// ESRF:   atsextupole('S20',0.4,-11.0599090880946,'NumIntSteps',20,'T1',[0;0;0;0;0;0],'T2',[0;0;0;0;0;0]);
// FODO:   atsextupole('SF',0.1,1);
mxArray *create_sextupole_atElem(mxArray *atProp, char *configuration) 
{
	int num_fields = 0, cnt = 0, rows, cols;

	// these are the fields not passed through configuration file for the corrector
	char **field_ptr = parse_atConfigElem(configuration, &num_fields);

	

	// mandatory fields common to any element
	mxArray *atElem = create_mxArray("Class", AT_CHAR, 1, 1, (void*)"Sextupole");cnt++;
	atElem = add_mxArray(atElem, "Fname", AT_CHAR, 1, 1,field_ptr[1]);cnt++;
	
	// fields that are mandatory in the configuration file
	if (is_atFieldName(field_ptr[2]) == 0) {
		atElem = add_mxArray(atElem, "Length", AT_DOUBLE_STR, rows, cols, text_atField(field_ptr[2],&rows, &cols));cnt++;
	}

	if (is_atFieldName(field_ptr[3]) == 0) {
		// if polynomials not defined, insert field_ptr[3] as the offset factor of PolynomB
		if (!get_atFieldValue(field_ptr, num_fields, "PolynomB") && !get_atFieldValue(field_ptr, num_fields, "PolynomA")) {
			char polb[80], pola[80];
			sprintf(polb,"0 0 %s",field_ptr[3]);
			sprintf(pola,"0 0 0");
			atElem = add_mxArray(atElem, "PolynomB", AT_DOUBLE_STR, 1, 3, polb);
			atElem = add_mxArray(atElem, "PolynomA", AT_DOUBLE_STR, 1, 3, pola);
			uint32_t MaxOrder = 2;
			atElem = add_mxArray(atElem, "MaxOrder", AT_UINT32, 1, 1, &MaxOrder);
		}
		cnt++;	
	}
	
	// open the shared library and get the pass method function symbol
	// optional parameters of the element
	for (int j = cnt; j < num_fields; j++) {
		// do not pickup here the pass method
		if (!is_atPassMethodName(field_ptr[j]) && is_atFieldName(field_ptr[j]) && (substr_match(field_ptr[j], "PassMethod") == 0)) {		
			if (is_atFieldName(field_ptr[j+1]) == 0) 
				atElem = add_mxArray(atElem, field_ptr[j], AT_DOUBLE_STR, rows, cols, text_atField(field_ptr[j+1],&rows, &cols));
			else 
				atElem = add_mxArray(atElem, field_ptr[j], AT_CHAR, 1, 1, field_ptr[j+1]);
			j++;
		}
	}

	// if NumIntStep is not defined, set default value to 10
	if (get_atFieldValue(field_ptr, num_fields, "NumIntSteps") == NULL) { 
		uint32_t NumIntSteps = 10;
		atElem = add_mxArray(atElem, "NumIntSteps", AT_UINT32, 1, 1, &NumIntSteps);
	}
	if (get_atFieldValue(field_ptr, num_fields, "MaxOrder") == NULL) { 
		uint32_t MaxOrder = 1;
		atElem = add_mxArray(atElem, "MaxOrder", AT_UINT32, 1, 1, &MaxOrder);	
	}
	// add energy field to all magnets
	if (get_atFieldValue(field_ptr, num_fields, "Energy") == NULL) { 
		double Energy = 0;
		atElem = add_mxArray(atElem, "Energy", AT_DOUBLE, 1, 1, &Energy);
	}

	// initialize pass method, scan all configuration string to find the pass method
	if (!find_PassMethod(field_ptr, num_fields))
		atElem = init_pass_atElem(atProp, atElem, "StrMPoleSymplectic4Pass");
	else	
		atElem = init_pass_atElem(atProp, atElem, find_PassMethod(field_ptr, num_fields));

	free(field_ptr);

	return atElem;

}

// SOLEIL: atquadrupole('Q1',0.3602,-1.148605,'StrMPoleSymplectic4Pass','Energy',2739100000)
// ESRF:   atquadrupole('QD3',0.53367,-0.60355343,'StrMPoleSymplectic4Pass','NumIntSteps',20,'FringeQuadEntrance',1,'FringeQuadExit',1)
// FODO:   atquadrupole('QD',0.5,-1.2,'PassMethod','StrMPoleSymplectic4Pass','Energy',3e9)
mxArray *create_quadrupole_atElem(mxArray *atProp, char *configuration) 
{
	int num_fields = 0, cnt = 0, rows, cols;
	
	// these are the fields not passed through configuration file for the corrector
	char **field_ptr = parse_atConfigElem(configuration, &num_fields);

	// mandatory fields common to any element
	mxArray *atElem = create_mxArray("Class", AT_CHAR, 1, 1, (void*)"Quadrupole");cnt++;
	atElem = add_mxArray(atElem, "Fname", AT_CHAR, 1, 1,field_ptr[1]);cnt++;

	// fields that are mandatory in the configuration file
	if (is_atFieldName(field_ptr[2]) == 0) {
		atElem = add_mxArray(atElem, "Length", AT_DOUBLE_STR, rows, cols, text_atField(field_ptr[2],&rows, &cols));cnt++;
	}

	// fields that are mandatory in the configuration file
	if (is_atFieldName(field_ptr[3]) == 0) {
		atElem = add_mxArray(atElem, "K", AT_DOUBLE_STR, rows, cols, text_atField(field_ptr[3],&rows, &cols));cnt++;
	}

	// if polynomials not defined, insert field_ptr[3] as the offset factor of PolynomB
	if (!get_atFieldValue(field_ptr, num_fields, "PolynomB") && !get_atFieldValue(field_ptr, num_fields, "PolynomA")) {
		char polb[80], pola[80];
		sprintf(polb,"0 %s",field_ptr[3]);
		sprintf(pola,"0 0");
		atElem = add_mxArray(atElem, "PolynomB", AT_DOUBLE_STR, 1, 2, polb);
		atElem = add_mxArray(atElem, "PolynomA", AT_DOUBLE_STR, 1, 2, pola);
		uint32_t MaxOrder = 1;
		atElem = add_mxArray(atElem, "MaxOrder", AT_UINT32, 1, 1, &MaxOrder);
	}

	// open the shared library and get the pass method function symbol
	// optional parameters of the element
	for (int j = cnt; j < num_fields; j++) {
		// do not pickup here the pass method
		if (!is_atPassMethodName(field_ptr[j]) && is_atFieldName(field_ptr[j]) && (substr_match(field_ptr[j], "PassMethod") == 0)) {		
			if (is_atFieldName(field_ptr[j+1]) == 0) 
				atElem = add_mxArray(atElem, field_ptr[j], AT_DOUBLE_STR, rows, cols, text_atField(field_ptr[j+1],&rows, &cols));
			else 
				atElem = add_mxArray(atElem, field_ptr[j], AT_CHAR, 1, 1, field_ptr[j+1]);
			j++;
		}
	}

	// if NumIntStep is not defined, se default value to 10
	if (get_atFieldValue(field_ptr, num_fields, "NumIntSteps") == NULL) { 
		uint32_t NumIntSteps = 10;
		atElem = add_mxArray(atElem, "NumIntSteps", AT_UINT32, 1, 1, &NumIntSteps);
	}
	if (get_atFieldValue(field_ptr, num_fields, "MaxOrder") == NULL) { 
		uint32_t MaxOrder = 1;
		atElem = add_mxArray(atElem, "MaxOrder", AT_UINT32, 1, 1, &MaxOrder);	
	}
	// add energy field to all magnets
	if (get_atFieldValue(field_ptr, num_fields, "Energy") == NULL) { 
		double Energy = 0;
		atElem = add_mxArray(atElem, "Energy", AT_DOUBLE, 1, 1, &Energy);
	}

	// initialize pass method, scan all configuration string to find the pass method
	if (!find_PassMethod(field_ptr, num_fields))
		atElem = init_pass_atElem(atProp, atElem, "StrMPoleSymplectic4Pass");
	else	
		atElem = init_pass_atElem(atProp, atElem, find_PassMethod(field_ptr, num_fields));

	free(field_ptr);

	return atElem;

}


// SOLEIL: atsbend('BEND',1.05243,0.196349540849362,0.00204,'BndMPoleSymplectic4Pass','EntranceAngle',0.097574770424681,'ExitAngle',0.099074770424681,'FringeBendEntrance',2,'FringeBendExit',2,'FringeQuadEntrance',0,'FringeQuadExit',0,'FullGap',0.053576,'FringeInt1',0.5,'FringeInt2',0.5,'Energy',2739100000)";
// ESRF:   atsbend('B1S',0.2927106,0.00585,0,'BndMPoleSymplectic4Pass','EntranceAngle',-0.0432374,'ExitAngle',0.0490874,'NumIntSteps',20,'FullGap',0,'FringeQuadEntrance',1,'FringeQuadExit',1)";
// FODO:   atsbend('Bend',1,0.15707963,'PassMethod','BndMPoleSymplectic4Pass','Energy',3e9";
// THOMX:   atsbend('BEND',0.27646,0.785398,0,'MaxOrder',3,'ByError',0,'FullGap',0.02784,'FringeInt1',0.5,'FringeInt2',0.5,'EdgeEffect1',1,'EdgeEffect2',1,'R1',[1 0 0 0 0 0;0 1 0 0 0 0;0 0 1 0 0 0;0 0 0 1 0 0;0 0 0 0 1 0;0 0 0 0 0 1],'R2',[1 0 0 0 0 0;0 1 0 0 0 0;0 0 1 0 0 0;0 0 0 1 0 0;0 0 0 0 1 0;0 0 0 0 0 1],'T1',[0 0 0 0 0 0],'T2',[0 0 0 0 0 0],'Energy',50000000)


mxArray *create_atsbend_atElem(mxArray *atProp, char *configuration) 
{
	int num_fields = 0, cnt = 0, rows, cols;

	// these are the fields not passed through configuration file for the corrector
	char **field_ptr = parse_atConfigElem(configuration, &num_fields);

	// mandatory fields common to any element
	mxArray *atElem = create_mxArray("Class", AT_CHAR, 1, 1, (void*)"Bend");cnt++;
	atElem = add_mxArray(atElem, "Fname", AT_CHAR, 1, 1,field_ptr[1]);cnt++;

	// fields that are mandatory in the configuration file (Length)
	if (is_atFieldName(field_ptr[2]) == 0) {
		atElem = add_mxArray(atElem, "Length", AT_DOUBLE_STR, rows, cols, text_atField(field_ptr[2],&rows, &cols));cnt++;
	}

	// fields that are mandatory in the configuration file (Bending Angle)
	if (is_atFieldName(field_ptr[3]) == 0) {
		atElem = add_mxArray(atElem, "BendingAngle", AT_DOUBLE_STR, rows, cols, text_atField(field_ptr[3],&rows, &cols));cnt++;
	}

	// fields that optional in the configuration file (K) but must be initialized in the at element 
	double K;
	if (is_atFieldName(field_ptr[4]) == 0)
		K = atof(field_ptr[4]);
	else
		K = 0;
	atElem = add_mxArray(atElem, "K", AT_DOUBLE, 1, 1, &K);cnt++;	

	// if polynomials not defined, insert field_ptr[3] as the offset factor of PolynomB
	if (!get_atFieldValue(field_ptr, num_fields, "PolynomB") && !get_atFieldValue(field_ptr, num_fields, "PolynomA")) {
		char polb[80], pola[80];
		sprintf(polb,"0 %.18e",K);
		sprintf(pola,"0 0");
		atElem = add_mxArray(atElem, "PolynomB", AT_DOUBLE_STR, 1, 2, polb);
		atElem = add_mxArray(atElem, "PolynomA", AT_DOUBLE_STR, 1, 2, pola);
		uint32_t MaxOrder = 1;
		atElem = add_mxArray(atElem, "MaxOrder", AT_UINT32, 1, 1, &MaxOrder);
	}

	// open the shared library and get the pass method function symbol
	// optional parameters of the element
	for (int j = cnt; j < num_fields; j++) {
		// do not pickup here the pass method
		if (!is_atPassMethodName(field_ptr[j]) && is_atFieldName(field_ptr[j]) && (substr_match(field_ptr[j], "PassMethod") == 0)) {		
			if (is_atFieldName(field_ptr[j+1]) == 0) 
				atElem = add_mxArray(atElem, field_ptr[j], AT_DOUBLE_STR, rows, cols, text_atField(field_ptr[j+1],&rows, &cols));
			else 
				atElem = add_mxArray(atElem, field_ptr[j], AT_CHAR, 1, 1, field_ptr[j+1]);
			j++;
		}
	}

	// if NumIntStep is not defined, se default value to 10
	if (get_atFieldValue(field_ptr, num_fields, "NumIntSteps") == NULL) { 
		uint32_t NumIntSteps = 10;
		atElem = add_mxArray(atElem, "NumIntSteps", AT_UINT32, 1, 1, &NumIntSteps);
	}
	if (get_atFieldValue(field_ptr, num_fields, "EntranceAngle") == NULL) { 
		double EntranceAngle = 0;
		atElem = add_mxArray(atElem, "EntranceAngle", AT_DOUBLE, 1, 1, &EntranceAngle);
	}
	if (get_atFieldValue(field_ptr, num_fields, "ExitAngle") == NULL) { 
		double ExitAngle = 0;
		atElem = add_mxArray(atElem, "ExitAngle", AT_DOUBLE, 1, 1, &ExitAngle);
	}	
	if (get_atFieldValue(field_ptr, num_fields, "MaxOrder") == NULL) { 
		uint32_t MaxOrder = 1;
		atElem = add_mxArray(atElem, "MaxOrder", AT_UINT32, 1, 1, &MaxOrder);	
	}

	// add energy field to all magnets
	if (get_atFieldValue(field_ptr, num_fields, "Energy") == NULL) { 
		double Energy = 0;
		atElem = add_mxArray(atElem, "Energy", AT_DOUBLE, 1, 1, &Energy);
	}

	// initialize pass method, scan all configuration string to find the pass method
	if (!find_PassMethod(field_ptr, num_fields))
		atElem = init_pass_atElem(atProp, atElem, "BndMPoleSymplectic4Pass");
	else	
		atElem = init_pass_atElem(atProp, atElem, find_PassMethod(field_ptr, num_fields));

	free(field_ptr);

	return atElem;

}


// SOLEIL:  atrbend('SkewQuad',1e-08,0,0.2,'StrMPoleSymplectic4Pass','MaxOrder',3,'R1',[1 0 0 0 0 0;0 1 0 0 0 0;0 0 1 0 0 0;0 0 0 1 0 0;0 0 0 0 1 0;0 0 0 0 0 1],'R2',[1 0 0 0 0 0;0 1 0 0 0 0;0 0 1 0 0 0;0 0 0 1 0 0;0 0 0 0 1 0;0 0 0 0 0 1],'T1',[0 0 0 0 0 0],'T2',[0 0 0 0 0 0],'Energy',2739100000);;
mxArray *create_atrbend_atElem(mxArray *atProp, char *configuration) 
{
	int num_fields = 0, cnt = 0, rows, cols;

	// these are the fields not passed through configuration file for the corrector
	char **field_ptr = parse_atConfigElem(configuration, &num_fields);

	// mandatory fields common to any element
	mxArray *atElem = create_mxArray("Class", AT_CHAR, 1, 1, (void*)"Bend");cnt++;
	atElem = add_mxArray(atElem, "Fname", AT_CHAR, 1, 1,field_ptr[1]);cnt++;

	// fields that are mandatory in the configuration file (Length)
	if (is_atFieldName(field_ptr[2]) == 0) {
		atElem = add_mxArray(atElem, "Length", AT_DOUBLE_STR, rows, cols, text_atField(field_ptr[2],&rows, &cols));cnt++;
	}

	// fields that are mandatory in the configuration file (Bending Angle)
	if (is_atFieldName(field_ptr[3]) == 0) {
		atElem = add_mxArray(atElem, "BendingAngle", AT_DOUBLE_STR, rows, cols, text_atField(field_ptr[3],&rows, &cols));cnt++;
	}

	// fields that optional in the configuration file (K) but must be initialized in the at element 
	double K;
	if (is_atFieldName(field_ptr[4]) == 0)
		K = strtold(field_ptr[4],NULL);
	else
		K = 0;
	double angle = strtold(text_atField(field_ptr[3],&rows, &cols),NULL);

	//printf("%s bendingangle=%s %1.15e K=%1.16e \n",field_ptr[1],text_atField(field_ptr[3],&rows, &cols),angle,K);

	atElem = add_mxArray(atElem, "K", AT_DOUBLE, 1, 1, &K);cnt++;	

	uint32_t MaxOrder = 4;
	if (get_atFieldValue(field_ptr, num_fields, "MaxOrder")) {
		atElem = add_mxArray(atElem, "MaxOrder", AT_DOUBLE_STR, 1, 1, get_atFieldValue(field_ptr, num_fields, "MaxOrder"));
		MaxOrder = atGetLong(atElem, "MaxOrder");
	}
	
	char polb[80], pola[80];
	//create_poly_array(polb, MaxOrder-1, K); // NAN: do not add any value at the end of the array

	create_poly_idx_array(polb, MaxOrder+1, 1, K); // [0 K ...], size = max_order - 1
	create_poly_array(pola, MaxOrder+1, NAN);

	atElem = add_mxArray(atElem, "PolynomB", AT_DOUBLE_STR, 1, MaxOrder+1, polb);
	atElem = add_mxArray(atElem, "PolynomA", AT_DOUBLE_STR, 1, MaxOrder+1, pola);

	// open the shared library and get the pass method function symbol
	// optional parameters of the element
	for (int j = cnt; j < num_fields; j++) {
		// do not pickup here the pass method
		if (!is_atPassMethodName(field_ptr[j]) && is_atFieldName(field_ptr[j]) && (substr_match(field_ptr[j], "PassMethod") == 0)) {	
			if (is_atFieldName(field_ptr[j+1]) == 0) 
				atElem = add_mxArray(atElem, field_ptr[j], AT_DOUBLE_STR, rows, cols, text_atField(field_ptr[j+1],&rows, &cols));
			else 
				atElem = add_mxArray(atElem, field_ptr[j], AT_CHAR, 1, 1, field_ptr[j+1]);
			j++;
		}
	}


	// if NumIntStep is not defined, se default value to 10
	if (get_atFieldValue(field_ptr, num_fields, "NumIntSteps") == NULL) { 
		uint32_t NumIntSteps = 10;
		atElem = add_mxArray(atElem, "NumIntSteps", AT_UINT32, 1, 1, &NumIntSteps);
	}
	if (get_atFieldValue(field_ptr, num_fields, "EntranceAngle") == NULL) { 
		double EntranceAngle = atGetDouble(atElem, "BendingAngle") / 2;
		atElem = add_mxArray(atElem, "EntranceAngle", AT_DOUBLE, 1, 1, &EntranceAngle);
	}
	if (get_atFieldValue(field_ptr, num_fields, "ExitAngle") == NULL) { 
		double ExitAngle = atGetDouble(atElem, "BendingAngle") / 2;
		atElem = add_mxArray(atElem, "ExitAngle", AT_DOUBLE, 1, 1, &ExitAngle);
	}	
	if (get_atFieldValue(field_ptr, num_fields, "MaxOrder") == NULL) { 
		uint32_t MaxOrder = 1;
		atElem = add_mxArray(atElem, "MaxOrder", AT_UINT32, 1, 1, &MaxOrder);	
	}

	// add energy field to all magnets
	if (get_atFieldValue(field_ptr, num_fields, "Energy") == NULL) { 
		double Energy = 0;
		atElem = add_mxArray(atElem, "Energy", AT_DOUBLE, 1, 1, &Energy);
	}

	// initialize pass method, scan all configuration string to find the pass method
	// initialize pass method, scan all configuration string to find the pass method
	if (!find_PassMethod(field_ptr, num_fields))
		atElem = init_pass_atElem(atProp, atElem, "BndMPoleSymplectic4Pass");
	else	
		atElem = init_pass_atElem(atProp, atElem, find_PassMethod(field_ptr, num_fields));

	free(field_ptr);

	return atElem;

}


mxArray *create_multipole_atElem(mxArray *atProp, char *configuration) 
{
	int num_fields = 0, cnt = 1, // Class defined in the configuration string
	rows, cols;

	// these are the fields not passed through configuration file for the corrector
	char **field_ptr = parse_atConfigElem(configuration, &num_fields);

	// mandatory fields common to any element
	mxArray *atElem = create_mxArray("Fname", AT_CHAR, 1, 1, (void*)field_ptr[1]);cnt++;

	// fields that are mandatory in the configuration file (Length)
	atElem = add_mxArray(atElem, "Length", AT_DOUBLE_STR, rows, cols, text_atField(field_ptr[2],&rows, &cols));cnt++;

	uint32_t MaxOrder = 1;

	// polynomA
	atElem = add_mxArray(atElem, "PolynomA", AT_DOUBLE_STR, rows, cols, text_atField(field_ptr[3],&rows, &cols));cnt++;
	// polynomB
	atElem = add_mxArray(atElem, "PolynomB", AT_DOUBLE_STR, rows, cols, text_atField(field_ptr[4],&rows, &cols));cnt++;

	// skip the pass method if it is defined without tag after PolynomB, it will be configured at the end of the function
	// so all parameters of the pass method will be initialized correctly 
	if (is_atPassMethodName(field_ptr[cnt])) {
		cnt++;
	}
	
	// open the shared library and get the pass method function symbol
	// optional parameters of the element
	for (int j = cnt; j < num_fields; j+=2) {			
		// do not pickup here the pass method
		if (!is_atPassMethodName(field_ptr[j]) && is_atFieldName(field_ptr[j]) && 
			(substr_match(field_ptr[j], "PassMethod") == 0)) {
			if (isalpha(field_ptr[j+1][0]) == 0) { // if field value is a number convert it into double	
				atElem = add_mxArray(atElem, field_ptr[j], AT_DOUBLE_STR, rows, cols, text_atField(field_ptr[j+1],&rows, &cols)); 
			}
			else // otherwise save it as text
				atElem = add_mxArray(atElem, field_ptr[j], AT_CHAR, 1, 1, (void*)field_ptr[j+1]);	
			// MaxOrder
			if (substr_match(field_ptr[j], "Polynom")) {
				if (rows > MaxOrder)
					MaxOrder = rows;
				if (cols > MaxOrder)
					MaxOrder = cols;						
			}
		}
	}

	if (get_atFieldValue(field_ptr, num_fields, "MaxOrder") == NULL) { 
		atElem = add_mxArray(atElem, "MaxOrder", AT_UINT32, 1, 1, &MaxOrder);
	}

	if (get_atFieldValue(field_ptr, num_fields, "Class") == NULL) { 
		atElem = add_mxArray(atElem, "Class", AT_CHAR, 1, 1, (void*)"Thinmpole");
	}

	// if NumIntStep is not defined, se default value to 10
	if (get_atFieldValue(field_ptr, num_fields, "NumIntSteps") == NULL) { 
		uint32_t NumIntSteps = 10;
		atElem = add_mxArray(atElem, "NumIntSteps", AT_UINT32, 1, 1, &NumIntSteps);
	}
	
	// add energy field to all magnets
	if (get_atFieldValue(field_ptr, num_fields, "Energy") == NULL) { 
		double Energy = 0;
		atElem = add_mxArray(atElem, "Energy", AT_DOUBLE, 1, 1, &Energy);
	}	

	if (!find_PassMethod(field_ptr, num_fields))
		atElem = init_pass_atElem(atProp, atElem, "StrMPoleSymplectic4Pass");
	else	
		atElem = init_pass_atElem(atProp, atElem, find_PassMethod(field_ptr, num_fields));

	free(field_ptr);

	return atElem;

}


mxArray *create_thinmpole_atElem(mxArray *atProp, char *configuration) 
{
	int num_fields = 0, cnt = 0, rows, cols;

	// these are the fields not passed through configuration file for the corrector
	char **field_ptr = parse_atConfigElem(configuration, &num_fields);

	// mandatory fields common to any element
	mxArray *atElem = create_mxArray("Class", AT_CHAR, 1, 1, (void*)"ThinMultipole");cnt++;
	atElem = add_mxArray(atElem, "Fname", AT_CHAR, 1, 1,field_ptr[1]);cnt++;

	double len = 0; rows = 1; cols = 1
;	atElem = add_mxArray(atElem, "Length", AT_DOUBLE_STR, rows, cols, &len);cnt++;
	
	// polynomA
	atElem = add_mxArray(atElem, "PolynomA", AT_DOUBLE_STR, rows, cols, text_atField(field_ptr[2],&rows, &cols));cnt++;
	// polynomB
	atElem = add_mxArray(atElem, "PolynomB", AT_DOUBLE_STR, rows, cols, text_atField(field_ptr[3],&rows, &cols));cnt++;

	// skip the pass method if it is defined without tag after PolynomB, it will be configured at the end of the function
	// so all parameters of the pass method will be initialized correctly 
	if (is_atPassMethodName(field_ptr[cnt])) {
		cnt++;
	}

	// open the shared library and get the pass method function symbol
	// optional parameters of the element
	for (int j = cnt; j < num_fields; j++) {
		// do not pickup here the pass method
		if (!is_atPassMethodName(field_ptr[j]) && is_atFieldName(field_ptr[j]) && (substr_match(field_ptr[j], "PassMethod") == 0)) {	
			if (is_atFieldName(field_ptr[j+1]) == 0) 
				atElem = add_mxArray(atElem, field_ptr[j], AT_DOUBLE_STR, rows, cols, text_atField(field_ptr[j+1],&rows, &cols));
			else 
				atElem = add_mxArray(atElem, field_ptr[j], AT_CHAR, 1, 1, field_ptr[j+1]);
			j++;
		}
	}

	// max order is mandatory
	uint32_t MaxOrder = 0; // this is the default value in Matlab AT
	if (get_atFieldValue(field_ptr, num_fields, "MaxOrder") == NULL) {
		atElem = add_mxArray(atElem, "MaxOrder", AT_DOUBLE_STR, 1, 1, get_atFieldValue(field_ptr, num_fields, "MaxOrder"));
		MaxOrder = atGetLong(atElem, "MaxOrder");
	}

	// poly are mandatory
	char polb[80], pola[80];
	create_poly_array(polb, MaxOrder+1, NAN); // NAN: do not add any value at the end of the array
	create_poly_array(pola, MaxOrder+1, NAN);
	atElem = add_mxArray(atElem, "PolynomB", AT_DOUBLE_STR, 1, MaxOrder+1, polb);
	atElem = add_mxArray(atElem, "PolynomA", AT_DOUBLE_STR, 1, MaxOrder+1, pola);

	// add energy field to all magnets
	if (get_atFieldValue(field_ptr, num_fields, "Energy") == NULL) { 
		double Energy = 0;
		atElem = add_mxArray(atElem, "Energy", AT_DOUBLE, 1, 1, &Energy);
	}	

	if (!find_PassMethod(field_ptr, num_fields))
		atElem = init_pass_atElem(atProp, atElem, "ThinMPolePass");
	else	
		atElem = init_pass_atElem(atProp, atElem, find_PassMethod(field_ptr, num_fields));

	free(field_ptr);

	return atElem;

}


// srElettra2_low_beta: atmarker('oct3','StrMPoleSymplectic4RadPass','Length',0.12, 'Class','Octupole','PolynomA',[0 0 0 0 0],'PolynomB',[0 -0.257181620708876 0 -34.6941 0],'T1',[0;0;0;0;0;0],'T2',[0;0;0;0;0;0],'MaxOrder',4,'EApertures',[0.0135 0.0085],'NumIntSteps',10)
mxArray *create_octupole_atElem(mxArray *atProp, char *configuration) 
{
	int num_fields = 0, cnt = 0, rows, cols;

	// these are the fields not passed through configuration file for the corrector
	char **field_ptr = parse_atConfigElem(configuration, &num_fields);

	

	// mandatory fields common to any element
	mxArray *atElem = create_mxArray("Class", AT_CHAR, 1, 1, (void*)"Octupole");cnt++;
	atElem = add_mxArray(atElem, "Fname", AT_CHAR, 1, 1,field_ptr[1]);cnt++;

	// jump over pass method 
	cnt++;
	// then all field are defined without any shortcut, each value is declared after its label

	
	// open the shared library and get the pass method function symbol
	// optional parameters of the element 
	for (int j = cnt; j < num_fields; j++) {
		// do not pickup here the pass method
		// the Octupole is define in the atmarker object so there is a field 'Class' that specifies 'Octupole' value
		if (!is_atPassMethodName(field_ptr[j]) && is_atFieldName(field_ptr[j]) && (substr_match(field_ptr[j], "PassMethod") == 0)) {
			if (is_atFieldName(field_ptr[j+1]) == 0) 
				atElem = add_mxArray(atElem, field_ptr[j], AT_DOUBLE_STR, rows, cols, text_atField(field_ptr[j+1],&rows, &cols));
			else 
				atElem = add_mxArray(atElem, field_ptr[j], AT_CHAR, 1, 1, field_ptr[j+1]);
			j++;
		}
	}

	// add energy field to all magnets
	if (get_atFieldValue(field_ptr, num_fields, "Energy") == NULL) { 
		double Energy = 0;
		atElem = add_mxArray(atElem, "Energy", AT_DOUBLE, 1, 1, &Energy);
	}

	// initialize pass method, scan all configuration string to find the pass method
	if (!find_PassMethod(field_ptr, num_fields))
		atElem = init_pass_atElem(atProp, atElem, "StrMPoleSymplectic4Pass");
	else	
		atElem = init_pass_atElem(atProp, atElem, find_PassMethod(field_ptr, num_fields));

	free(field_ptr);

	return atElem;

}



// SCW E1: atwiggler('SCW',1.568,'GWigSymplecticPass','Class','Wiggler','Lw',0.0245,'Bmax',-3.5,'Energy',2000000000,
//                 'Nstep',5,'Nmeth',4,'Bx',[],'By',[1;1;0;1;1;0],'NHharm',1,'NVharm',0);
mxArray *create_wiggler_atElem(mxArray *atProp, char *configuration) 
{
	int num_fields = 0, cnt = 0, rows, cols;

	// these are the fields not passed through configuration file for the corrector
	char **field_ptr = parse_atConfigElem(configuration, &num_fields);
	mxArray *atElem;


	atElem = create_mxArray("Class", AT_CHAR, 1, 1, (void*)"Wiggler");cnt++;
	// element 1: name
	atElem = add_mxArray(atElem, "Fname", AT_CHAR, 1, 1,field_ptr[1]);cnt++;
	// element 2: length (no chars in the field)
	if (is_atFieldName(field_ptr[2]) == 0) {
		atElem = add_mxArray(atElem, "Length", AT_DOUBLE_STR, rows, cols, text_atField(field_ptr[2],&rows, &cols));cnt++;
	}
	// open the shared library and get the pass method function symbol
	// optional parameters of the element
	for (int j = cnt; j < num_fields; j++) {
		// do not pickup here the pass method
		if (!is_atPassMethodName(field_ptr[j]) && is_atFieldName(field_ptr[j]) && (substr_match(field_ptr[j], "PassMethod") == 0)) {		
			if (is_atFieldName(field_ptr[j+1]) == 0) 
				atElem = add_mxArray(atElem, field_ptr[j], AT_DOUBLE_STR, rows, cols, text_atField(field_ptr[j+1],&rows, &cols));
			else 
				atElem = add_mxArray(atElem, field_ptr[j], AT_CHAR, 1, 1, field_ptr[j+1]);
			j++;
		}
	}

	// initialize pass method, scan all configuration string to find the pass method
	if (!find_PassMethod(field_ptr, num_fields))
		atElem = init_pass_atElem(atProp, atElem, "GWigSymplecticPass");
	else	
		atElem = init_pass_atElem(atProp, atElem, find_PassMethod(field_ptr, num_fields));

	free(field_ptr);

	return atElem;


}




// SOLEIL: atdrift('SDB5',0.205,'Energy',2739100000)";
// ESRF:   atdrift('SD3U',0.133165)";
// FODO:   atdrift('Dr',0.2)";
mxArray *create_drift_atElem(mxArray *atProp, char *configuration) 
{
	int num_fields = 0, cnt = 0, rows, cols;

	// these are the fields not passed through configuration file for the corrector
	char **field_ptr = parse_atConfigElem(configuration, &num_fields);

	// mandatory fields common to any element
	mxArray *atElem = create_mxArray("Class", AT_CHAR, 1, 1, (void*)"Drift");cnt++;
	atElem = add_mxArray(atElem, "Fname", AT_CHAR, 1, 1,field_ptr[1]);cnt++;

	// fields that are mandatory in the configuration file (Length)
	if (is_atFieldName(field_ptr[2]) == 0) {
		atElem = add_mxArray(atElem, "Length", AT_DOUBLE_STR, rows, cols, text_atField(field_ptr[2],&rows, &cols));cnt++;
	}

	// open the shared library and get the pass method function symbol
	// optional parameters of the element
	for (int j = cnt; j < num_fields; j++) {
		// do not pickup here the pass method
		if (!is_atPassMethodName(field_ptr[j]) && is_atFieldName(field_ptr[j]) && (substr_match(field_ptr[j], "PassMethod") == 0)) {		
			if (is_atFieldName(field_ptr[j+1]) == 0) 
				atElem = add_mxArray(atElem, field_ptr[j], AT_DOUBLE_STR, rows, cols, text_atField(field_ptr[j+1],&rows, &cols));
			else 
				atElem = add_mxArray(atElem, field_ptr[j], AT_CHAR, 1, 1, field_ptr[j+1]);
			j++;
		}
	}

	// initialize pass method, scan all configuration string to find the pass method
	if (!find_PassMethod(field_ptr, num_fields))
		atElem = init_pass_atElem(atProp, atElem, "DriftPass");
	else	
		atElem = init_pass_atElem(atProp, atElem, find_PassMethod(field_ptr, num_fields));

	free(field_ptr);

	return atElem;

}


// SOLEIL: atrfcavity('RF',0,2472000,352201954.08583,416,2739100000)";
// ESRF:   atrfcavity('CA5',0,2000000,352199664.076085,992,6040000000,'RFCavityPass')";
// FODO:   atrfcavity('RFCav')";
mxArray *create_rfcavity_atElem(mxArray *atProp, char *configuration) 
{
	int num_fields = 0, cnt = 0, rows, cols;

	// these are the fields not passed through configuration file for the corrector
	char **field_ptr = parse_atConfigElem(configuration, &num_fields);

	// mandatory fields common to any element
	mxArray *atElem = create_mxArray("Class", AT_CHAR, 1, 1, (void *)"RFCavity");cnt++;
	atElem = add_mxArray(atElem, "Fname", AT_CHAR, 1, 1,field_ptr[1]);cnt++;

	// fields that are mandatory in the configuration file (Length)
	if ((num_fields >= 3) && (is_atFieldName(field_ptr[2]) == 0)) {
		atElem = add_mxArray(atElem, "Length", AT_DOUBLE_STR, rows, cols, text_atField(field_ptr[2],&rows, &cols));cnt++;
	}	
	else {
		atElem = add_mxArray(atElem, "Length", AT_DOUBLE_STR, rows, cols, (void *)"0");
	}

	// fields that are mandatory in the configuration file (Voltage)
	if ((num_fields >= 4) && (is_atFieldName(field_ptr[3]) == 0)) {
		atElem = add_mxArray(atElem, "Voltage", AT_DOUBLE_STR, rows, cols, text_atField(field_ptr[3],&rows, &cols));cnt++;
	}	
	else {
		atElem = add_mxArray(atElem, "Voltage", AT_DOUBLE_STR, rows, cols, (void *)"0");
	}

	// fields that are mandatory in the configuration file (Frequency)
	if ((num_fields >= 5) && (is_atFieldName(field_ptr[4]) == 0)) {
		atElem = add_mxArray(atElem, "Frequency", AT_DOUBLE_STR, rows, cols, text_atField(field_ptr[4],&rows, &cols));cnt++;
	}
	else {
		atElem = add_mxArray(atElem, "Frequency", AT_DOUBLE_STR, rows, cols, (void *)"0");
	}		

	if ((num_fields >= 6) && (is_atFieldName(field_ptr[5]) == 0)) {
		atElem = add_mxArray(atElem, "HarmNumber", AT_DOUBLE_STR, rows, cols, text_atField(field_ptr[5],&rows, &cols));cnt++;
	}
	else {
		atElem = add_mxArray(atElem, "HarmNumber", AT_DOUBLE_STR, rows, cols, (void *)"0");
	}		

	if ((num_fields >= 6) && (is_atFieldName(field_ptr[6]) == 0)) {
		atElem = add_mxArray(atElem, "Energy", AT_DOUBLE_STR, rows, cols, text_atField(field_ptr[6],&rows, &cols));cnt++;
	}
	else {
		atElem = add_mxArray(atElem, "Energy", AT_DOUBLE_STR, rows, cols, (void *)"0");
	}

	// open the shared library and get the pass method function symbol
	// optional parameters of the element
	for (int j = cnt; j < num_fields; j++) {
		// do not pickup here the pass method
		if (!is_atPassMethodName(field_ptr[j]) && is_atFieldName(field_ptr[j]) && (substr_match(field_ptr[j], "PassMethod") == 0)) {	
			if (is_atFieldName(field_ptr[j+1]) == 0) 
				atElem = add_mxArray(atElem, field_ptr[j], AT_DOUBLE_STR, rows, cols, text_atField(field_ptr[j+1],&rows, &cols));
			else 
				atElem = add_mxArray(atElem, field_ptr[j], AT_CHAR, 1, 1, field_ptr[j+1]);
			j++;
		}
	}

	// define it if not defined
	if (get_atFieldValue(field_ptr, num_fields, "TimeLag") == NULL) { 
		double TimeLag = 0;
		atElem = add_mxArray(atElem, "TimeLag", AT_DOUBLE, 1, 1, &TimeLag);
	}
	// define it if not defined
	if (get_atFieldValue(field_ptr, num_fields, "Energy") == NULL) { 
		double Energy = 0;
		atElem = add_mxArray(atElem, "Energy", AT_DOUBLE, 1, 1, &Energy);
	}
	

	// initialize pass method, scan all configuration string to find the pass method
	if (!find_PassMethod(field_ptr, num_fields)) {
		atElem = init_pass_atElem(atProp, atElem, "RFCavityPass");
	}	
	else {
		atElem = init_pass_atElem(atProp, atElem, find_PassMethod(field_ptr, num_fields));
	}	
	
	free(field_ptr);

	return atElem;

}


mxArray *create_atElem(mxArray *atProp, char *configuration) 
{

	// octupole is defined in the configuration file as:
	// atmarker('oct3','StrMPoleSymplectic4RadPass','Length',0.12, 'Class','Octupole',....
	if (strstr(configuration,"Class") && strstr(configuration,"Octupole")) { 
		return create_octupole_atElem(atProp, configuration);
	}
	if (strstr(configuration,"Class") && strstr(configuration,"Wiggler")) { 
		return create_wiggler_atElem(atProp, configuration);
	}
	else if (strstr(configuration,"atdrift")) {
		return create_drift_atElem(atProp, configuration);
	}
	else if (strstr(configuration,"atmultipole")) {
		return create_multipole_atElem(atProp, configuration);
	}	
	else if (strstr(configuration,"atquadrupole")) {
		return create_quadrupole_atElem(atProp, configuration);
	}
	else if (strstr(configuration,"atcorrector")) {
		return create_corrector_atElem(atProp, configuration);
	}		 
	else if (strstr(configuration,"atsextupole")) {
		return create_sextupole_atElem(atProp, configuration);
	}
	else if (strstr(configuration,"atmonitor")) {
		return create_monitor_atElem(atProp, configuration);
	}
	else if (strstr(configuration,"atsbend")) {
		return create_atsbend_atElem(atProp, configuration);
	}	
	else if (strstr(configuration,"atrbend")) {
		return create_atrbend_atElem(atProp, configuration);
	}
	else if (strstr(configuration,"atmarker")) {
		return create_marker_atElem(atProp, configuration);
	}
	else if (strstr(configuration,"atrfcavity")) {
		return create_rfcavity_atElem(atProp, configuration);
	}
	else if (strstr(configuration,"atthinmultipole")) {
		return create_thinmpole_atElem(atProp, configuration);
	}		
	else {
		printf("Configuration file corrupted (error in %s)!!!!\n",configuration);
		return NULL;
	}
	

}


void print_all_ring(atSuperRing_t *SR)
{
	mxArray *atRingIter;
	atRingIter = SR->atRing;
	
	int cnt = 0;
	// load the ring
	do {
		char row_desc[20];
		// the fieldname of each pointer to atElem object is an incrementing number in text format
		// atFile->data contains one row of the configuration file
		sprintf(row_desc,"%d",cnt);
		
		mxArray *atElem = (mxArray *) atGetPointer(atRingIter, row_desc);
		
		print_all_mxArray(atElem);
		
		cnt++;
		atRingIter = atRingIter->next;

	} while (atRingIter != NULL);
}


mxArray *read_file(const char *filename)
{

    if (filename == NULL) {
        printf("read_file(): error, undefined lattice file\n");
        return NULL;
    }

    FILE* file = fopen(filename, "r"); /* should check the result */
    char line[4096];

    if (file == NULL)
	return NULL;

    mxArray *rows_file; 

    rows_file = NULL;

    int cnt = 0;
    while (fgets(line, sizeof(line), file)) {
	char row_desc[20];
	sprintf(row_desc,"%d",cnt);

	// remove the chars after )
	char *ptr = strchr(line, ')');
	if (ptr != NULL) {
		*(ptr+1) = '\0';
	}
	// remove spaces at the start of the string
	int offset = 0;
	while (isspace(line[offset]))
        	offset++;

	// skip header and footer of the m-file lattice generated bt the atwritem function		
	if (!(strstr(line + offset,"function ring=") || strstr(line + offset,"ring={") || strlen(line + offset) < 6)) { 
		if (cnt == 0)
			rows_file = create_mxArray(row_desc, AT_CHAR, 1, 1, line + offset);
		else
			rows_file = add_mxArray(rows_file, row_desc, AT_CHAR, 1, 1, line + offset);	
		cnt++;	   
	}

    }
    /* may check feof here to make a difference between eof and io failure -- network
       timeout for instance */

    fclose(file);
   
    return rows_file;

}


mxArray *read_configuration(const char **configuration, int num_rows)
{

    if (configuration == NULL)
	return NULL;

    mxArray *rows_file; 

    rows_file = NULL;

  

    int cnt = 0;
    for (int i = 0; i < num_rows; i++) {
	char row_desc[20];
	sprintf(row_desc,"%d",cnt);

	// remove spaces at the start of the string
	int offset = 0;
	while (1) {
		char *a = (configuration[i]+offset);
		if (isspace(*a) == 0)
			break;
        	offset++;
	}

	// skip header and footer of the m-file lattice generated bt the atwritem function		
	if (!(strstr(configuration[i] + offset,"function ring=") || strstr(configuration[i] + offset,"ring={") || strlen(configuration[i] + offset) < 6)) { 
		if (cnt == 0)
			rows_file = create_mxArray(row_desc, AT_CHAR, 1, 1, configuration[i] + offset);
		else
			rows_file = add_mxArray(rows_file, row_desc, AT_CHAR, 1, 1, configuration[i] + offset);	
		cnt++;	   
	}

	

    }

    return rows_file;

}


mxArray *create_atRing(mxArray *atProp) 
{
	mxArray *atFile;
	
	// store configuration information into an array

	if (atGetPointer(atProp,"LatticeFile")) 		
		atFile = read_file(atGetPointer(atProp,"LatticeFile"));
	else 
		atFile= read_configuration(atGetPointer(atProp,"Configuration"), atGetLong(atProp,"ConfigurationSize"));
	

	mxArray *atFileIter = atFile;
	mxArray *atRing;
	int cnt = 0;
	uint32_t val0 = 0;

	if (atFile == NULL)
		return NULL;

	atRing = NULL;
	if (atProp == NULL) 
		atProp = create_mxArray("NumRadElem", AT_UINT32, 1, 1, &val0);
	else
		atProp = add_mxArray(atProp, "NumRadElem", AT_UINT32, 1, 1, &val0);
		
	atProp = add_mxArray(atProp, "NumRFElem", AT_UINT32, 1, 1, &val0);

	// load the ring

	do {
		char row_desc[20];
		sprintf(row_desc,"%d",cnt);
		// atFileIter->data is the pointer of to a one row of configuration file

		mxArray *atElem = create_atElem(atProp, atFileIter->data);


		if (atElem) {
			if (cnt == 0) 
				atRing = create_mxArray(row_desc, AT_POINTER_NODE, 1, 1, atElem);	
			else
				atRing = add_mxArray(atRing, row_desc, AT_POINTER_NODE, 1, 1, atElem);		
			cnt++;
		}
		

		atFileIter = atFileIter->next;
	} while (atFileIter != NULL);
	
	destroy_all_mxArray(atFile);

	return atRing;
}


void destroy_atRing(mxArray *atRing) 
{
	mxArray *atRingElemIter = atRing;

	// atRingElemIter has two main pointers:
	// ->next: next element of the lattice
	// ->data: pointer to the list of AT fields that define the lattice element

	// load the ring
	while (1) {
		if (atRingElemIter == NULL)
			break;
		// atElem is the pointer to the first field of a list that defines the
		// lattice element
		mxArray *atElem = atRingElemIter->data;
		destroy_all_mxArray(atElem);
		if (atRingElemIter->next != NULL)
			atRingElemIter = atRingElemIter->next;
		else 
			break;
	} 

	destroy_all_mxArray(atRing);
	
}


#ifdef CICCIO
void destroy_atRing(mxArray *atRing) 
{
	mxArray *atRingIter;

	atRingIter = atRing;
	
	int cnt = 0;
	// load the ring
	do {
		char row_desc[20];
		// the fieldname of each pointer to atElem object is an incrementing number in text format
		// atFile->data contains one row of the configuration file
		sprintf(row_desc,"%d",cnt);
		mxArray *atElem = (mxArray *) atGetPointer(atRingIter, row_desc);
		destroy_all_mxArray(atElem);
		cnt++;
		if (atRingIter != NULL)
			atRingIter = atRingIter->next;

	} while (atRingIter != NULL);

	destroy_all_mxArray(atRing);
	
}
#endif


void alloc_data_atSuperRing(atSuperRing_t *SR) 
{
	int32_t num_elem = atGetLong(SR->atProp,"NumElem");

	SR->elemdata.mu_x = (double *)malloc(num_elem * sizeof(double));
	SR->elemdata.mu_y = (double *)malloc(num_elem * sizeof(double));	
	SR->elemdata.alpha_x = (double *)malloc(num_elem * sizeof(double));
	SR->elemdata.alpha_y = (double *)malloc(num_elem * sizeof(double));	
	SR->elemdata.beta_x = (double *)malloc(num_elem * sizeof(double));
	SR->elemdata.beta_y = (double *)malloc(num_elem * sizeof(double));	
	SR->elemdata.eta_x = (double *)malloc(num_elem * sizeof(double));
	SR->elemdata.eta_xp = (double *)malloc(num_elem * sizeof(double));	
	SR->elemdata.eta_y = (double *)malloc(num_elem * sizeof(double));
	SR->elemdata.eta_yp = (double *)malloc(num_elem * sizeof(double));
	SR->elemdata.x = (double *)malloc(num_elem * sizeof(double));
	SR->elemdata.xp = (double *)malloc(num_elem * sizeof(double));
	SR->elemdata.y = (double *)malloc(num_elem * sizeof(double));
	SR->elemdata.yp = (double *)malloc(num_elem * sizeof(double));
	SR->elemdata.dpd = (double *)malloc(num_elem * sizeof(double));
	SR->elemdata.ct = (double *)malloc(num_elem * sizeof(double));	
	SR->elemdata.spos = (double *)malloc(num_elem * sizeof(double));
	SR->elemdata.tilt = (double *)malloc(num_elem * sizeof(double));	
	SR->elemdata.sigma_x = (double *)malloc(num_elem * sizeof(double));
	SR->elemdata.sigma_y = (double *)malloc(num_elem * sizeof(double));		
	SR->elemdata.num_elem = num_elem;
	
	SR->ringdata.tunes = (double *)malloc(3 * sizeof(double));
	SR->ringdata.int_tunes = (double *)malloc(3 * sizeof(double));
	SR->ringdata.damping_times = (double *)malloc(3 * sizeof(double));
	SR->ringdata.damping_rates = (double *)malloc(3 * sizeof(double));		
	SR->ringdata.chromaticity = (double *)malloc(3 * sizeof(double));

}


void free_data_atSuperRing(atSuperRing_t *SR) 
{
	if (SR->elemdata.mu_x) free(SR->elemdata.mu_x);
	if (SR->elemdata.mu_y) free(SR->elemdata.mu_y);	
	if (SR->elemdata.alpha_x) free(SR->elemdata.alpha_x);
	if (SR->elemdata.alpha_y) free(SR->elemdata.alpha_y);	
	if (SR->elemdata.beta_x) free(SR->elemdata.beta_x);
	if (SR->elemdata.beta_y) free(SR->elemdata.beta_y);	
	if (SR->elemdata.eta_x) free(SR->elemdata.eta_x);
	if (SR->elemdata.eta_xp) free(SR->elemdata.eta_xp);	
	if (SR->elemdata.eta_y) free(SR->elemdata.eta_y);
	if (SR->elemdata.eta_yp) free(SR->elemdata.eta_yp);
	if (SR->elemdata.x) free(SR->elemdata.x);
	if (SR->elemdata.xp) free(SR->elemdata.xp);
	if (SR->elemdata.y) free(SR->elemdata.y);
	if (SR->elemdata.yp) free(SR->elemdata.yp);
	if (SR->elemdata.dpd) free(SR->elemdata.dpd);
	if (SR->elemdata.ct) free(SR->elemdata.ct);	
	if (SR->elemdata.spos) free(SR->elemdata.spos);
	if (SR->elemdata.tilt) free(SR->elemdata.tilt);
	if (SR->elemdata.sigma_x) free(SR->elemdata.sigma_x);
	if (SR->elemdata.sigma_y) free(SR->elemdata.sigma_y);
	
	if (SR->ringdata.tunes) free(SR->ringdata.tunes);
	if (SR->ringdata.int_tunes) free(SR->ringdata.int_tunes);	
	if (SR->ringdata.damping_times) free(SR->ringdata.damping_times);	
	if (SR->ringdata.damping_rates) free(SR->ringdata.damping_rates);		
	if (SR->ringdata.chromaticity) free(SR->ringdata.chromaticity);
}


void init_data_atSuperRing(atSuperRing_t *SR) 
{
	SR->elemdata.mu_x = NULL;
	SR->elemdata.mu_y = NULL;	
	SR->elemdata.alpha_x = NULL;
	SR->elemdata.alpha_y = NULL;	
	SR->elemdata.beta_x = NULL;
	SR->elemdata.beta_y = NULL;	
	SR->elemdata.eta_x = NULL;
	SR->elemdata.eta_xp = NULL;	
	SR->elemdata.eta_y = NULL;
	SR->elemdata.eta_yp = NULL;
	SR->elemdata.x = NULL;
	SR->elemdata.xp = NULL;
	SR->elemdata.y = NULL;
	SR->elemdata.yp = NULL;
	SR->elemdata.dpd = NULL;
	SR->elemdata.ct = NULL;	
	SR->elemdata.spos = NULL;
	SR->elemdata.tilt = NULL;
	SR->elemdata.sigma_x = NULL;
	SR->elemdata.sigma_y = NULL;
	
	SR->ringdata.tunes = NULL;
	SR->ringdata.int_tunes = NULL;	
	SR->ringdata.damping_times = NULL;	
	SR->ringdata.damping_rates = NULL;		
	SR->ringdata.chromaticity = NULL;
}


void memcpy_data_atSuperRing(atSuperRing_t *SR_dest, atSuperRing_t *SR_orig) 
{

	int32_t num_elem = atGetLong(SR_orig->atProp,"NumElem");

	memcpy(SR_dest->elemdata.mu_x, SR_orig->elemdata.mu_x, num_elem * sizeof(double));
	memcpy(SR_dest->elemdata.mu_y, SR_orig->elemdata.mu_y, num_elem * sizeof(double));	
	memcpy(SR_dest->elemdata.alpha_x, SR_orig->elemdata.alpha_x, num_elem * sizeof(double));
	memcpy(SR_dest->elemdata.alpha_y, SR_orig->elemdata.alpha_y, num_elem * sizeof(double));	
	memcpy(SR_dest->elemdata.beta_x, SR_orig->elemdata.beta_x, num_elem * sizeof(double));
	memcpy(SR_dest->elemdata.beta_y, SR_orig->elemdata.beta_y, num_elem * sizeof(double));	
	memcpy(SR_dest->elemdata.eta_x, SR_orig->elemdata.eta_x, num_elem * sizeof(double));
	memcpy(SR_dest->elemdata.eta_xp, SR_orig->elemdata.eta_xp, num_elem * sizeof(double));	
	memcpy(SR_dest->elemdata.eta_y, SR_orig->elemdata.eta_y, num_elem * sizeof(double));
	memcpy(SR_dest->elemdata.eta_yp, SR_orig->elemdata.eta_yp, num_elem * sizeof(double));
	memcpy(SR_dest->elemdata.x, SR_orig->elemdata.x, num_elem * sizeof(double));
	memcpy(SR_dest->elemdata.xp, SR_orig->elemdata.xp, num_elem * sizeof(double));
	memcpy(SR_dest->elemdata.y, SR_orig->elemdata.y, num_elem * sizeof(double));
	memcpy(SR_dest->elemdata.yp, SR_orig->elemdata.yp, num_elem * sizeof(double));
	memcpy(SR_dest->elemdata.dpd, SR_orig->elemdata.dpd, num_elem * sizeof(double));
	memcpy(SR_dest->elemdata.ct, SR_orig->elemdata.ct, num_elem * sizeof(double));	
	memcpy(SR_dest->elemdata.spos, SR_orig->elemdata.spos, num_elem * sizeof(double));
	memcpy(SR_dest->elemdata.tilt, SR_orig->elemdata.tilt, num_elem * sizeof(double));
	memcpy(SR_dest->elemdata.sigma_x, SR_orig->elemdata.sigma_x, num_elem * sizeof(double));
	memcpy(SR_dest->elemdata.sigma_y, SR_orig->elemdata.sigma_y, num_elem * sizeof(double));
	
	memcpy(SR_dest->ringdata.tunes, SR_orig->ringdata.tunes, 3 * sizeof(double));
	memcpy(SR_dest->ringdata.int_tunes, SR_orig->ringdata.int_tunes, 3 * sizeof(double));	
	memcpy(SR_dest->ringdata.damping_times, SR_orig->ringdata.damping_times, 3 * sizeof(double));	
	memcpy(SR_dest->ringdata.damping_rates, SR_orig->ringdata.damping_rates, 3 * sizeof(double));		
	memcpy(SR_dest->ringdata.chromaticity, SR_orig->ringdata.chromaticity, 3 * sizeof(double));
	
}


atSuperRing_t *create_atSuperRingFile(const char *passmethod_path, const char *filename, const char *ringname, const char *ringclass) 
{
	atSuperRing_t *SR = (atSuperRing_t *)malloc(sizeof(struct atSuperRing_t));

	SR->atProp = create_mxArray("Fname", AT_CHAR, 1, 1, (void*)ringname);
	SR->atProp = add_mxArray(SR->atProp, "Class", AT_CHAR, 1, 1, (void*)ringclass);	
	SR->atProp = add_mxArray(SR->atProp, "PassMethodPath", AT_CHAR, 1, 1, (void*)passmethod_path);
	SR->atProp = add_mxArray(SR->atProp, "LatticeFile", AT_CHAR, 1, 1, (void*)filename);

	// remove dynamic configuration variables
	if (atGetPointer(SR->atProp, "Configuration")) 
		destroy_field_mxArray(SR->atProp, "Configuration");
	if (atGetPointer(SR->atProp, "ConfigurationSize")) 	
		destroy_field_mxArray(SR->atProp, "ConfigurationSize");

	SR->atRing = create_atRing(SR->atProp); 

	if (SR->atRing == NULL)
		return NULL;

	find_atRingLength(SR); 
	
	alloc_data_atSuperRing(SR);	

	map_atSuperRing(SR);

	return SR;	

}


atSuperRing_t *create_atSuperRingConfiguration(const char *passmethod_path, const char **configuration, int num_rows, const char *ringname, const char *ringclass) 
{
	atSuperRing_t *SR = (atSuperRing_t *)malloc(sizeof(struct atSuperRing_t));

	SR->atProp = create_mxArray("Fname", AT_CHAR, 1, 1, (void*)ringname);
	SR->atProp = add_mxArray(SR->atProp, "Class", AT_CHAR, 1, 1, (void*)ringclass);	
	SR->atProp = add_mxArray(SR->atProp, "PassMethodPath", AT_CHAR, 1, 1, (void*)passmethod_path);

	// in case the configuration is passed on the fly through a C-array of strings (C **) then
	// save in the atProp the starting pointer of the array and its size
	if (atGetPointer(SR->atProp, "Configuration")) 
		destroy_field_mxArray(SR->atProp, "Configuration");
	SR->atProp = add_mxArray(SR->atProp, "Configuration", AT_POINTER, 1, 1, (void*)configuration);

	if (atGetPointer(SR->atProp, "ConfigurationSize")) 	
		destroy_field_mxArray(SR->atProp, "ConfigurationSize");
	SR->atProp = add_mxArray(SR->atProp, "ConfigurationSize", AT_INT32, 1, 1, &num_rows);

	SR->atRing = create_atRing(SR->atProp); 

	if (SR->atRing == NULL)
		return NULL;

	find_atRingLength(SR); 
	
	alloc_data_atSuperRing(SR);	

	map_atSuperRing(SR);

	return SR;	

}


atSuperRing_t *clone_atSuperRing(atSuperRing_t *SR) 
{
	struct parameters *Param = getSRParam(SR);
	atSuperRing_t *SR_new = malloc(sizeof(struct atSuperRing_t));

	SR_new->atRing = clone_elem_mxArray(SR->atRing);
	SR_new->atProp = clone_elem_mxArray(SR->atProp);
	alloc_data_atSuperRing(SR_new);
	
	mxArray *new_atRingElemIter = SR_new->atRing; 
	mxArray *atRingElemIter = SR->atRing;
		
	int cnt = 0;
	double r_in[6] = {0,0,0,0,0,0};	
	while (1) {

		if (atRingElemIter == NULL)
			break;

		mxArray *atElem = (mxArray *) atRingElemIter->data;
		// clone lattice element, this is the CORE!!!
		// when element is cloned the pass function of the integrator is allocated and new values are passed to the internal integrator structure
		// this cloning feature is used to update the values in the tango server where each of the computing threads (optics ohmienvelope chromaticity)
		// clones the master ring at each iteration. So any updates of the elements is automatically passed to the integrator internal structure
		// by this cloning action 
		if (atElem != NULL) {
			mxArray *atElem_new = clone_elem_mxArray(atElem);
			// this code executes the update of the values passed to the AT pass functions
			if (atElem_new != NULL) {
				destroy_field_mxArray(atElem_new, "Elem"); // destroy the data structure passed to the atpass function
				track_function_ptr fptr = (track_function_ptr) atGetPointer(atElem_new,"TrackFunction");
				// fptr(atElem_new, NULL, r_in, 0, Param) -> this is the new structure passed to the atpass function
				atElem_new = add_mxArray(atElem_new, "Elem", AT_POINTER_F, 1, 1, fptr(atElem_new, NULL, r_in, 0, Param));	
				new_atRingElemIter->data = atElem_new;
			}	
		}	
		else {
		   new_atRingElemIter->data = NULL;
		}
		
		if (new_atRingElemIter != NULL) {
		        new_atRingElemIter = new_atRingElemIter->next;
		}        
		if (atRingElemIter != NULL)        
			atRingElemIter = atRingElemIter->next;
		else
			break;
		 
	} 

	free(Param);

	map_atSuperRing(SR_new);

	return SR_new;
	
}


#ifdef CICCIO
atSuperRing_t *clone_atSuperRing(atSuperRing_t *SR) 
{
	struct parameters *Param = getSRParam(SR);
	atSuperRing_t *SR_new = malloc(sizeof(struct atSuperRing_t));

	SR_new->atRing = clone_elem_mxArray(SR->atRing);
	SR_new->atProp = clone_elem_mxArray(SR->atProp);
	alloc_data_atSuperRing(SR_new);
	
	mxArray *new_atRingIter = SR_new->atRing, 
	*atRingIter = SR->atRing;
		
	int cnt = 0;
	double r_in[6] = {0,0,0,0,0,0};	
	do {
		char row_desc[20];
		sprintf(row_desc,"%d",cnt);
		mxArray *atElem = (mxArray *) atGetPointer(atRingIter, row_desc);
		// clone lattice element, this is the CORE!!!
		// when element is cloned the pass function of the integrator is allocated and new values are passed to the internal integrator structure
		// Rhis cloning feature is used to update the values in the tango server where each of the computing threads (optics ohmienvelope chromaticity)
		// clones the master ring at each iteration. So any updates of the elements is automatically passed to the integrator internal structure
		// by this cloning action 
		if (atElem != NULL) {
			mxArray *atElem_new = clone_elem_mxArray(atElem);
			// this code executes the update of the values passed to the AT pass functions
			if (atElem_new != NULL) {
				destroy_field_mxArray(atElem_new, "Elem"); // destroy the data structure passed to the atpass function
				track_function_ptr fptr = (track_function_ptr) atGetPointer(atElem_new,"TrackFunction");
				// fptr(atElem_new, NULL, r_in, 0, Param) -> this is the new structure passed to the atpass function
				atElem_new = add_mxArray(atElem_new, "Elem", AT_POINTER_F, 1, 1, fptr(atElem_new, NULL, r_in, 0, Param));	
				new_atRingIter->data = atElem_new;
			}	
		}	
		else {
		   new_atRingIter->data = NULL;
		}
		
		if (new_atRingIter != NULL) {
		        new_atRingIter = new_atRingIter->next;
		}        
		if (atRingIter != NULL)        
			atRingIter = atRingIter->next;
		
		cnt++; 
	} while (atRingIter != NULL);

	free(Param);

	map_atSuperRing(SR_new);

	return SR_new;
	
}
#endif

void map_atSuperRing(atSuperRing_t *SR) 
{
	int cnt = 0;

	SR->elem_map = map_create(16);

	mxArray *atRingElemIter = SR->atRing; // first element of the ring

	while (1) {
		// go through the ring
		// get one element by element
		//char row_desc[20];
		//sprintf(row_desc,"%d",cnt);
		//mxArray *atRingElemIter = (mxArray *) atGetPointer(SR->atRing, row_desc);
		if (atRingElemIter == NULL)
			break;

		//mxArray *atElemIter = atRingElemIter
		mxArray *atElemIter = atRingElemIter->data; // define the iterator inside the element

		// search for Fname fields inside the RingElem[0...1000] --> Fname->Length->PolynomA->FnameA0->MaxOrder->....
		while (1) {
			mxArray *atElem = get_field_mxArray(atElemIter, "Fname");
			if (atElem == NULL)
				break;
			// map element name with the first element of the list that identifies the element
			// and then pass to the next element of the list because other fnames could be defined 
			// inside the ring element iterator	
			if (map_put(SR->elem_map, (char *)atElem->data , (void *)  atRingElemIter->data) == MAP_FAILURE) {
				//printf("Error configuring map; element[%d], duplicated Fname %s\n", cnt,
				//	(char *)atElemIter->data);
			}

			// search if there are other Fnames inside the ring element (FnameA0...FnameB0...FnameA1...)
			if (atElemIter->next != NULL) 
				atElemIter = atElemIter->next;
			else
				break;

		}

		if (atRingElemIter->next != NULL) 
			atRingElemIter = atRingElemIter->next;
		else
			break;

		cnt++;
	} 

}


void unmap_atSuperRing(atSuperRing_t *SR) 
{
	map_destroy(SR->elem_map);
}



mxArray *get_atElemSuperRing(atSuperRing_t *SR, char *elem_fname) 
{
	return (mxArray *) map_get(SR->elem_map, elem_fname);
}



void destroy_atSuperRing(atSuperRing_t *SR) 
{
	unmap_atSuperRing(SR);
	free_data_atSuperRing(SR);
	destroy_atRing(SR->atRing);
	destroy_all_mxArray(SR->atProp);
	free(SR);	

}


void find_atRFCavIndexes(atSuperRing_t *SR) 
{
	mxArray *atRingIter = SR->atRing;
	int cnt = 0, cav_cnt = 0;
	double Voltage = 0;
	double cav_idx_array[100];

	struct parameters *Param = getSRParam(SR);

	if (atRingIter == NULL) {
		printf("Ring is empty\n");
		return;
	} 

	do {
	
		char idx[20];sprintf(idx,"%d",cnt);
		mxArray *atElem = (mxArray *)atGetPointer(atRingIter,idx);

		if (strcmp("RFCavity", (char *)atGetPointer(atElem, "Class")) == 0) {
			
			cav_idx_array[cav_cnt] = cnt;
			if (atGetPointer(atElem, "Energy")) {
				double Energy = atGetDouble(atElem, "Energy");
				SR->atProp = add_mxArray(SR->atProp,"Energy", AT_DOUBLE, 1, 1, &Energy);
			}	
			if (atGetPointer(atElem, "Frequency")) {
				double Frequency = atGetDouble(atElem, "Frequency");
				SR->atProp = add_mxArray(SR->atProp,"Frequency", AT_DOUBLE, 1, 1, &Frequency);
			}			
			if (atGetPointer(atElem, "HarmNumber")) {
				double HarmNumber = atGetDouble(atElem, "HarmNumber");
				SR->atProp = add_mxArray(SR->atProp,"HarmNumber", AT_DOUBLE, 1, 1, &HarmNumber);
				SR->atProp = add_mxArray(SR->atProp,"cell_harmnumber", AT_DOUBLE, 1, 1, &HarmNumber);
			}
			if (atGetPointer(atElem, "Voltage")) {
				Voltage += atGetDouble(atElem, "Voltage");
			}
	
			// destroy and recreate rf cavity pass to pass T0 and harmonic number calculated after storage ring initialization	
			destroy_field_mxArray(atElem, "Elem");				
			destroy_field_mxArray(atElem, "TrackFunction");	
		
			track_function_ptr pass_track = (track_function_ptr) get_atPassPtr(atGetPointer(SR->atProp,"PassMethodPath"),atGetPointer(atElem,"PassMethod"));

			atElem = add_mxArray(atElem, "TrackFunction", AT_FUNCTION, 1, 1, (void*)pass_track);

			track_function_ptr fptr = (track_function_ptr) atGetPointer(atElem,"TrackFunction");
			double r_in[6] = {0,0,0,0,0,0};					
			atElem = add_mxArray(atElem, "Elem", AT_POINTER_F, 1, 1, fptr(atElem, NULL, r_in, 0, Param));		
			// Voltage, Frequency
			// store pointer to RF elements with fieldname RFCavity,Address,0...RFCavity,Address,N.....
			char cavity_fieldname[64];sprintf(cavity_fieldname,"RFCavity,Address,%d",cav_cnt);
			SR->atProp = add_mxArray(SR->atProp,cavity_fieldname, AT_POINTER, 1, 1, atElem);
			cav_cnt++;
		}
		cnt++;
		if (atRingIter != NULL)
			atRingIter = atRingIter->next;
	} while (atRingIter != NULL); 	
	
	if (cav_cnt > 0) {
		SR->atProp = add_mxArray(SR->atProp,"cavpts", AT_DOUBLE, 1, cav_cnt, cav_idx_array);
		SR->atProp = add_mxArray(SR->atProp, "NumRFElem", AT_UINT32, 1, 1, &cav_cnt);
	}	
	
	// store the sum of all voltages
	SR->atProp = add_mxArray(SR->atProp,"Voltage", AT_DOUBLE, 1, 1, &Voltage);
	
	free(Param);
	
}


void find_atEnergy(atSuperRing_t *SR) 
{
	mxArray *atRingIter = SR->atRing;
	int cnt = 0;
	
	if (atRingIter == NULL) {
		printf("Ring is empty\n");
		return;
	} 
	
	do {
		char idx[20];sprintf(idx,"%d",cnt);
		mxArray *atElem = (mxArray *)atGetPointer(atRingIter,idx);
		
		if ((atElem != NULL) && (atGetPointer(atElem, "Energy"))) {
			double Energy = atGetDouble(atElem, "Energy");
			SR->atProp = add_mxArray(SR->atProp,"Energy", AT_DOUBLE, 1, 1, &Energy);
			break;
		}

		cnt++;
		if (atRingIter != NULL)
			atRingIter = atRingIter->next;
	} while (atRingIter != NULL); 	

}


void find_atRingLength(atSuperRing_t *SR) 
{
	mxArray *atRingIter = SR->atRing;
	uint32_t cnt = 0;
	double Length = 0;

	if (atRingIter == NULL) {
		printf("Ring is empty\n");
		return;
	} 

	do {
		char idx[10];sprintf(idx,"%d",cnt);
		mxArray *atElem = (mxArray *)atGetPointer(atRingIter,idx);
		atElem = add_mxArray(atElem, "SPos", AT_DOUBLE, 1, 1, &Length);		
		if ((atElem != NULL) && (atGetPointer(atElem, "Length"))) {
			Length += atGetDouble(atElem, "Length");
		}
		cnt++;
		if (atRingIter != NULL)
			atRingIter = atRingIter->next;
	} while (atRingIter != NULL); 	

	// storage ring length
	
	if (SR->atProp != NULL) {
		SR->atProp = add_mxArray(SR->atProp,"Length", AT_DOUBLE, 1, 1, &Length);	
		// number of elements
		SR->atProp = add_mxArray(SR->atProp,"NumElem", AT_UINT32, 1, 1, &cnt);
	}	
	else {
		printf("Ring properties not created\n");	
	}
	
}


void set_atTrackKicks(atSuperRing_t *SR, double XYStep, double DPStep) 
{
	if (SR->atProp != NULL) {
		SR->atProp = add_mxArray(SR->atProp,"XYStep", AT_DOUBLE, 1, 1, &XYStep);	
		SR->atProp = add_mxArray(SR->atProp,"DPStep", AT_DOUBLE, 1, 1, &DPStep);
	}
	else {
		printf("Ring properties not created\n");
	}	
}


void set_atClosedOrbitMinMax(atSuperRing_t *SR, double MinVal, double MaxVal) 
{
	if (SR->atProp != NULL) {
		SR->atProp = add_mxArray(SR->atProp,"ClosedOrbitMinVal", AT_DOUBLE, 1, 1, &MinVal);	
		SR->atProp = add_mxArray(SR->atProp,"ClosedOrbitMaxVal", AT_DOUBLE, 1, 1, &MaxVal);
	}
	else {
		printf("Ring properties not created\n");
	}	
}


void set_atCounter(atSuperRing_t *SR, double Counter) 
{
	if (SR->atProp != NULL) {
		if (atGetPointer(SR->atProp, "Counter") == NULL) {
			SR->atProp = add_front_mxArray(SR->atProp,"Counter", AT_DOUBLE, 1, 1, &Counter);
		}	
		else {
			destroy_field_mxArray(SR->atProp, "Counter");								
			SR->atProp = add_front_mxArray(SR->atProp,"Counter", AT_DOUBLE, 1, 1, &Counter);
		}
	}
	else {
		printf("Ring properties not created\n");
	}	
}


void atRadOn(atSuperRing_t *SR) 
{
	mxArray *atRingIter = SR->atRing;
	double energy = atGetDouble(SR->atProp, "Energy");

	if (atRingIter == NULL) {
		printf("Ring is empty\n");
		return;
	} 
	
	int cnt = 0;
	do {
		char idx[20];sprintf(idx,"%d",cnt);
		mxArray *atElem = (mxArray *)atGetPointer(atRingIter,idx);
		if ((atElem != NULL) && (atGetPointer(atElem, "Class"))) {
			
			if ((strcmp(atGetPointer(atElem, "Class"),"Bend") == 0) ||  
				(strcmp(atGetPointer(atElem, "Class"),"Quadrupole") == 0)) {
			
				if (strcmp(atGetPointer(atElem, "PassMethod"),"BndMPoleSymplectic4Pass") == 0) {					
					destroy_field_mxArray(atElem, "Elem");					
					destroy_field_mxArray(atElem, "PassMethod");					
					destroy_field_mxArray(atElem, "TrackFunction");				
					add_mxArray(atElem,"Energy", AT_DOUBLE, 1, 1, &energy);					
					atElem = init_pass_atElem(SR->atProp, atElem, "BndMPoleSymplectic4RadPass");
				}
				
				if (strcmp(atGetPointer(atElem, "PassMethod"),"StrMPoleSymplectic4Pass") == 0) {				
					destroy_field_mxArray(atElem, "Elem");				
					destroy_field_mxArray(atElem, "PassMethod");					
					destroy_field_mxArray(atElem, "TrackFunction");					
					add_mxArray(atElem,"Energy", AT_DOUBLE, 1, 1, &energy);					
					atElem = init_pass_atElem(SR->atProp, atElem, "StrMPoleSymplectic4RadPass");							
				}	
			}
			else if (strcmp(atGetPointer(atElem, "Class"),"RFCavity") == 0)	{
				destroy_field_mxArray(atElem, "Elem");				
				destroy_field_mxArray(atElem, "PassMethod");					
				destroy_field_mxArray(atElem, "TrackFunction");				
				atElem = init_pass_atElem(SR->atProp, atElem, "RFCavityPass");
			}
		}	
			
		cnt++;
		if (atRingIter != NULL) 
			atRingIter = atRingIter->next;
		
	} while (atRingIter != NULL); 	
	
		

}


void atRadOff(atSuperRing_t *SR) 
{
	mxArray *atRingIter = SR->atRing;

	if (atRingIter == NULL) {
		printf("Ring is empty\n");
		return;
	} 

	int cnt = 0;
	do {
		char idx[20];sprintf(idx,"%d",cnt);
		mxArray *atElem = (mxArray *)atGetPointer(atRingIter,idx);
		if ((atElem != NULL) && (atGetPointer(atElem, "Class"))) {
				
			if (strcmp(atGetPointer(atElem, "PassMethod"),"BndMPoleSymplectic4RadPass") == 0) {		
				destroy_field_mxArray(atElem, "Elem");
				destroy_field_mxArray(atElem, "PassMethod");
				destroy_field_mxArray(atElem, "TrackFunction");
				destroy_field_mxArray(atElem, "Energy");
				uint32_t *NumRadElem = atGetPointer(SR->atProp, "NumRadElem");
				(*NumRadElem) -= 1;
				atElem = init_pass_atElem(SR->atProp, atElem, "BndMPoleSymplectic4Pass");
			}
			
			else if (strcmp(atGetPointer(atElem, "PassMethod"),"StrMPoleSymplectic4RadPass") == 0) {								
				destroy_field_mxArray(atElem, "Elem");					
				destroy_field_mxArray(atElem, "PassMethod");					
				destroy_field_mxArray(atElem, "TrackFunction");					
				destroy_field_mxArray(atElem, "Energy");
				uint32_t *NumRadElem = atGetPointer(SR->atProp, "NumRadElem");
				(*NumRadElem) -= 1;									
				atElem = init_pass_atElem(SR->atProp, atElem, "StrMPoleSymplectic4Pass");					
			}					
			/*
			if ((strcmp(atGetPointer(atElem, "PassMethod"),"RFCavityPass") == 0) || 
				(strcmp(atGetPointer(atElem, "PassMethod"),"CavityPass") == 0)) {								
				destroy_field_mxArray(atElem, "Elem");					
				destroy_field_mxArray(atElem, "PassMethod");					
				destroy_field_mxArray(atElem, "TrackFunction");	
				if (atGetDouble(atElem,"Length") == 0)	{						
					atElem = init_pass_atElem(SR->atProp, atElem, "IdentityPass");
				}		
				else {
					atElem = init_pass_atElem(SR->atProp, atElem, "DriftPass");	
				}
				uint32_t *NumRFElem = atGetPointer(SR->atProp, "NumRFElem");
				(*NumRFElem) -= 1;											
			}
			*/				
	
		}	
			
		cnt++;
		if (atRingIter != NULL) 
			atRingIter = atRingIter->next;
	} while (atRingIter != NULL); 		

}


void atRFOn(atSuperRing_t *SR) 
{
	mxArray *atRingIter = SR->atRing;

	if (atRingIter == NULL) {
		printf("Ring is empty\n");
		return;
	} 
	
	int cnt = 0;
	do {
		char idx[20];sprintf(idx,"%d",cnt);
		mxArray *atElem = (mxArray *)atGetPointer(atRingIter,idx);
		if ((atElem != NULL) && (atGetPointer(atElem, "Class"))) {
			
			if (strcmp(atGetPointer(atElem, "Class"),"RFCavity") == 0)	{
				destroy_field_mxArray(atElem, "Elem");				
				destroy_field_mxArray(atElem, "PassMethod");					
				destroy_field_mxArray(atElem, "TrackFunction");				
				atElem = init_pass_atElem(SR->atProp, atElem, "RFCavityPass");
			}
		}	
			
		cnt++;
		if (atRingIter != NULL) 
			atRingIter = atRingIter->next;
	} while (atRingIter != NULL); 		

}


void atRFOff(atSuperRing_t *SR) 
{
	mxArray *atRingIter = SR->atRing;

	if (atRingIter == NULL) {
		printf("Ring is empty\n");
		return;
	} 

	int cnt = 0;
	do {
		char idx[20];sprintf(idx,"%d",cnt);
		mxArray *atElem = (mxArray *)atGetPointer(atRingIter,idx);
		if ((atElem != NULL) && (atGetPointer(atElem, "Class"))) {
			if (strcmp(atGetPointer(atElem, "Class"),"RFCavity") == 0) {
				if ((strcmp(atGetPointer(atElem, "PassMethod"),"RFCavityPass") == 0) || 
					(strcmp(atGetPointer(atElem, "PassMethod"),"CavityPass") == 0)) {						
					destroy_field_mxArray(atElem, "Elem");					
					destroy_field_mxArray(atElem, "PassMethod");					
					destroy_field_mxArray(atElem, "TrackFunction");	
					if (atGetDouble(atElem,"Length") == 0)	{						
						atElem = init_pass_atElem(SR->atProp, atElem, "IdentityPass");
					}		
					else {
						atElem = init_pass_atElem(SR->atProp, atElem, "DriftPass");	
					}
					uint32_t *NumRFElem = atGetPointer(SR->atProp, "NumRFElem");
					(*NumRFElem) -= 1;									
				}				
			}	
		}	
			
		cnt++;
		if (atRingIter != NULL) 		
			atRingIter = atRingIter->next;
	} while (atRingIter != NULL); 		

}


void atSetRFFrequency(atSuperRing_t *SR, double frequency) 
{
	mxArray *atRingIter = SR->atRing;
	int cnt = 0;

	if (atRingIter == NULL) {
		printf("Ring is empty\n");
		return;
	} 
	
	// change RF cavity property
	do {
		char idx[20];sprintf(idx,"%d",cnt);
		mxArray *atElem = (mxArray *)atGetPointer(atRingIter,idx);
		if ((atElem != NULL) && (atGetPointer(atElem, "Frequency"))) {
			destroy_field_mxArray(atElem, "Frequency");								
			atElem = add_mxArray(atElem,"Frequency", AT_DOUBLE, 1, 1, &frequency);
			init_elem_atElem(atElem);
		}	
			
		cnt++;
		if (atRingIter != NULL) 
			atRingIter = atRingIter->next;
	} while (atRingIter != NULL); 		

	// change ring global property
	if ((SR->atProp != NULL) && (atGetPointer(SR->atProp, "Frequency"))) {
		destroy_field_mxArray(SR->atProp, "Frequency");
		SR->atProp = add_mxArray(SR->atProp,"Frequency", AT_DOUBLE, 1, 1, &frequency);	
	}
}


void atSetEnergy(atSuperRing_t *SR, double energy) 
{
	mxArray *atRingIter = SR->atRing;
	int cnt = 0;

	if (atRingIter == NULL) {
		printf("Ring is empty\n");
		return;
	} 

	do {
		char idx[20];sprintf(idx,"%d",cnt);
		mxArray *atElem = (mxArray *)atGetPointer(atRingIter,idx);
		if ((atElem != NULL) && (atGetPointer(atElem, "Energy"))) {
			destroy_field_mxArray(atElem, "Energy");							
			atElem = add_mxArray(atElem,"Energy", AT_DOUBLE, 1, 1, &energy);
			init_elem_atElem(atElem);
		}
	
		cnt++;
		if (atRingIter != NULL) 
			atRingIter = atRingIter->next;
	} while (atRingIter != NULL); 		

	// change ring global property
	if ((SR->atProp != NULL) && (atGetPointer(SR->atProp, "Energy"))) {
		destroy_field_mxArray(SR->atProp, "Energy");
		SR->atProp = add_mxArray(SR->atProp,"Energy", AT_DOUBLE, 1, 1, &energy);	
	}
}


void transpose(double *src, double *dst, const int N, const int M) 
{
	for(int n = 0; n<N*M; n++) {
		int i = n/N;
		int j = n%N;
		dst[n] = src[M*j + i];
	}
}


void linear_solve(double *Ain, double *bin, double *xo, const int N, const int M, int transposeA) 
{
	double *Atin;
	
	double *Ain_bk = malloc(N*M*sizeof(double));
	memcpy(Ain_bk,Ain,N*M*sizeof(double));
	double *bin_bk = malloc(M*sizeof(double));
	memcpy(bin_bk,bin,M*sizeof(double));	
	
	
	if (transposeA) {
		Atin = (double *) malloc(N*M*sizeof(double));
		// src,dest,rows,cols
		transpose(Ain, Atin, N, M);
	}
	else {
		Atin = Ain;
	}	 	

	// in matlab A\b
  	gsl_matrix_view A = gsl_matrix_view_array(Atin, N, M);
  	gsl_vector_view b = gsl_vector_view_array(bin, N);

	// Allocate space for the solution x
	gsl_vector *xg = gsl_vector_alloc(M);

	// Allocate workspace for the solver
	gsl_permutation *p = gsl_permutation_alloc(M);

	// Solve the system
	int s;
	gsl_linalg_LU_decomp(&A.matrix, p, &s);
	gsl_linalg_LU_solve(&A.matrix, p, &b.vector, xg);
	
	for (int i = 0; i < N; i++) {
		xo[i] = gsl_vector_get(xg,i);
	}
	
	// Clean up
	gsl_permutation_free(p);
	gsl_vector_free(xg);
	
	memcpy(Ain,Ain_bk,N*M*sizeof(double));
	memcpy(bin,bin_bk,M*sizeof(double));
	free(Ain_bk);
	free(bin_bk);
	
	if (transposeA)
		free(Atin);

}


double norm_diff(double *a, double *b, int size) 
{
	double change_tmp = 0;
	for (int i = 0; i < size; i++) {
		change_tmp += (a[i] - b[i]) * (a[i] - b[i]);
	}
	return pow(change_tmp,0.5);
}


double *atpass(atSuperRing_t *SR, double *Ri, uint32_t num_particles, uint32_t max_num_turns, 
	uint32_t *refpts, uint32_t max_num_refpts, struct parameters *Param, int32_t *beam_loss)
{
	double *Rout;
	int64_t turn_cnt = 0, refpts_cnt = 0, offset = 0, tot_elem = 0;
	uint32_t key_cnt;
	uint32_t num_elem = atGetLong(SR->atProp,"NumElem");	
	double min_val = 1e-30;// atGetDouble(SR->atProp,"ClosedOrbitMinVal");	// speed up by 5%
	double max_val = 1e10; //atGetDouble(SR->atProp,"ClosedOrbitMaxVal");

	if (max_num_refpts > 0) {	
		tot_elem = 6 * num_particles * max_num_refpts;
	}

	// store all the progress
	if (max_num_refpts > 0) {
		Rout = (double *) malloc(tot_elem * sizeof(double));
		memset(Rout,0,tot_elem * sizeof(double));
	}
	else {
		Rout = Ri;
	}

	// perform in any case the malloc to avoid segmentation into the upper layers
	if (*beam_loss)
		return;

	// calculate U0 with only one particle
	refpts_cnt = 0;
	int stored = 0;
	while (turn_cnt < max_num_turns) {  
		mxArray *atRingIter = SR->atRing;
	
		key_cnt = 0;	
		stored = 0;
		
		do {
			char idx[5];sprintf(idx,"%d",key_cnt); 

			if ((refpts_cnt < max_num_refpts) && (key_cnt == (refpts[refpts_cnt] % num_elem))) {
				memcpy(Rout+offset, Ri, num_particles * 6 * sizeof(double));
				stored = 1;
				refpts_cnt++;
				offset += num_particles * 6;		
			}

			// generate the fieldname to extract element which is always found immediately
			// because of the shift of the Iter pointer
			mxArray *atElem = (mxArray *)atGetPointer(atRingIter,idx);

			// get pointer to data struct used in pass function
			struct elem *Elem = (struct elem *)atGetPointer(atElem,"Elem");

			// get pointer of pass function
			track_function_ptr pass_track = (track_function_ptr) atGetPointer(atElem,"TrackFunction");
	
			// very small values slow down calculations
			// removed because in the real lattics orbit is never 0	
			/*for (int t = 0; t < num_particles * 6; t++) {
				if (fabs(Ri[t]) < min_val)
					Ri[t] = 0;
			}*/
			
			pass_track(atElem, Elem, Ri, num_particles, Param);
		
			/*
			if ((key_cnt >= 0) && (key_cnt < 39)) {
				double sum = 0;
				for (int i = 0; i < 6; i++) {
					printf("turn=%d el=%d outval[%d]=%.14e\n",turn_cnt,key_cnt,i,Ri[i]);
					sum += fabs(Ri[i]);
				}
				printf("GG1 turn=%d el=%d sum=%.14e %s\n",turn_cnt,key_cnt,sum,  atGetPointer(atElem,"PassMethod"));
			}
			*/

			// moved out of the inner loop
			/*for (int t = 0; t < num_particles * 6; t++) {
				if (isnan(Ri[t]) || (fabs(Ri[t]) > max_val)) {
					*beam_loss = 1;
					break;
				}
			}*/
			
			// copy data in the output if the element corresponds
			if ((stored == 0) && (refpts_cnt < max_num_refpts) && (key_cnt == (refpts[refpts_cnt] % num_elem))) {
				memcpy(Rout+offset, Ri, num_particles * 6 * sizeof(double));
				refpts_cnt++;
				offset += num_particles * 6;
				
				if (refpts_cnt >= max_num_refpts)
					break;
				
			}
			key_cnt++;
			atRingIter = atRingIter->next;
	
		} while (atRingIter != NULL); 	
		turn_cnt++;

		// move beamloss outside
		for (int t = 0; t < num_particles * 6; t++) {
			if (isnan(Ri[t]) || (fabs(Ri[t]) > max_val)) {
				*beam_loss = 1;
				break;
			}
		}
		
		if (*beam_loss)
			break;

		if ((refpts_cnt >= max_num_refpts) && (max_num_refpts > 0)) 
			break;

	}

	return Rout;

}


double *ringpass(atSuperRing_t *SR, uint32_t num_particles, uint32_t num_turns, double *Rout, int32_t *beam_loss)
{
	//double *Rout;
	elem_pass_t *elempass;
	int64_t turn_cnt = 0, offset = 0;

	struct parameters *Param = getSRParam(SR);	
	uint32_t num_elem = atGetLong(SR->atProp,"NumElem");	
	double min_val = atGetDouble(SR->atProp,"ClosedOrbitMinVal");	
	double max_val = atGetDouble(SR->atProp,"ClosedOrbitMaxVal");

	// differently from atpass ringpass function creates a list of trackfunction pointer
	// and input data struct
	elempass = (struct elem_pass_t *) malloc(num_elem * sizeof(struct elem_pass_t));
	mxArray *atElem = SR->atRing;
	for (unsigned int i = 0; i < num_elem; i++) {
		elempass[i].track_function = atGetPointer(atElem->data,"TrackFunction");	
		elempass[i].Elem = atGetPointer(atElem->data,"Elem");	
		atElem = atElem->next;	
	}

	// set closed orbit as starting point
	double *Ri = xorbit_6(SR, beam_loss);

	//Ri[0] += 1e-13;

	// calculate U0 with only one particle
	while (turn_cnt < num_turns) {  
		mxArray *atRingIter = SR->atRing;
	
		for (unsigned int i = 0; i < num_elem; i++) {
			if (i == 0) {
				memcpy(Rout+offset, Ri, num_particles * 6 * sizeof(double));
				offset += num_particles * 6;
			}
			Param->nturn = turn_cnt;	

			for (int t = 0; t < num_particles * 6; t++) {
				if (fabs(Ri[t]) < min_val)
					Ri[t] = 0;
			}

			elempass[i].track_function(atRingIter->data, elempass[i].Elem, Ri, num_particles, Param);

			for (int t = 0; t < num_particles * 6; t++) {
				if (isnan(Ri[t]) || fabs(Ri[t]) > max_val) {
					*beam_loss = 1;
					break;
				}
			}

			atRingIter = atRingIter->next;
		} 

		if (*beam_loss)
			break;

		turn_cnt++;
	}
	
	free(Ri);
	free(Param);
	free(elempass);
	
	return Rout;
}




struct parameters *getSRParam(atSuperRing_t *SR)
{
	struct parameters *Param = (struct parameters *) malloc(sizeof(struct parameters));
	memset(Param,0,sizeof(struct parameters));	

	Param->RingLength = atGetDouble(SR->atProp,"Length");	
	Param->T0 = (double) Param->RingLength / (double)C0;
	Param->charge = -1;

	return Param;
}





/*
 * xorbit_ct() 
 * Closed orbit for fixed path lengthening
 */
double *xorbit_ct(atSuperRing_t *SR, double dct, int32_t *beam_loss) 
{
	struct parameters *Param = getSRParam(SR);

	double XYStep = atGetDouble(SR->atProp,"XYStep");
	uint32_t num_turns = 1, num_particles = 6;	

	double *Ri = (double *) malloc(6 * sizeof(double));
	memset(Ri, 0, 6 * sizeof(double));

	if (*beam_loss) {
		free(Param);
		return Ri;
	}

	double scaling[6] = {XYStep,XYStep,XYStep,XYStep,XYStep,0};
	double theta5[6] = {0,0,0,0,dct,0};

	// send in the ring seven different particles, six of them are perturbed
	// the last one is propagated without offsets
	double *RMATi = (double *) malloc(num_particles * 6 * sizeof(double));
	memset(RMATi, 0, num_particles * 6 * sizeof(double));

	double *eye5_J5 = (double *) malloc((num_particles-1) * (num_particles-1) * sizeof(double));
	double *J5 = (double *) malloc((num_particles-1) * (num_particles-1) * sizeof(double));

	double change = 1e100, dps = 1.0000e-12;
	uint32_t itercount = 0, max_iterations = 20;

	// select subindexes from a {1:6,1:6} matrix to {1:4,1:4} submatrix, MATLAB order 
	int R_idx[25] = {0,1,2,3,5,6,7,8,9,11,12,13,14,15,17,18,19,20,21,23,24,25,26,27,29};
	int R_off[5] = {30,31,32,33,35};	

	while ((change > dps) && (itercount < max_iterations)) {

		// RMATi= Ri(:,ones(1,6)) + D5;
		for (size_t i = 0; i < num_particles * 6; i++) {
			RMATi[i] = Ri[i % 6];
		}	
		for (size_t i = 0; i < num_particles - 1; i++) {
			RMATi[i*6 + i] += scaling[i % 6];
		}
	
		// RMATf = linepass(RING,RMATi,args{:});	
		atpass(SR, RMATi, num_particles, num_turns, NULL, 0, Param, beam_loss);
		if (*beam_loss) 
			break;

		// compute the transverse part of the Jacobian
		// J5 =  (RMATf([1:4,6],1:5)-RMATf([1:4,6],6*ones(1,5)))./scaling;
		for (size_t i = 0; i < (num_particles - 1) * (num_particles-1); i++) {
			J5[i] = RMATi[R_idx[i]] - RMATi[R_off[i % (num_particles-1)]];
			J5[i] /= scaling[i / (num_particles-1)];
		}

		// now isolate the parameters of the unperturbed particle and remove energy terms
		//(Rf([1:4,6])-[Ri(1:4);0]-theta5)
		double Rf[6];
		for (int i = 0; i < 5; i++) {
			if (i < 4)
				Rf[i] = RMATi[R_off[i]]-Ri[i]-theta5[i];
			else
				Rf[i] = RMATi[R_off[i]]-theta5[i];	
		}

		// diag([1 1 1 1 0])
		memset(eye5_J5, 0, (num_particles-1) * (num_particles - 1) * sizeof(double));
		for (size_t i = 0; i < (num_particles - 2); i++) 
			eye5_J5[i*5+i] = 1;
		
		//diag([1 1 1 1 0]) - J5 , we need to transpose it because of the ordering
		for (size_t i = 0; i < (num_particles - 1) * (num_particles - 1); i++) {
			eye5_J5[i] -= J5[i];
		}

		double Ri_next[6] = {0,0,0,0,0,0}; double gsl_x[6];
		// solve linear system (eye(5)-J5)\(Rf-Ri)   ---- Rf = Rf - Ri
		linear_solve(eye5_J5, Rf, gsl_x, (num_particles - 1), (num_particles - 1), 1);

		
		// Ri_next = Ri + (eye(5)-J5)\(Rf-Ri-theta);
		for (unsigned int i = 0; i < (num_particles-1); i++) {
			Ri_next[i] = Ri[i] + gsl_x[i];
		}

		// change = norm(Ri_next - Ri);
		change = norm_diff(Ri_next,Ri, (num_particles-1));

		memcpy(Ri, Ri_next, (num_particles-1) *sizeof(double));

		itercount = itercount+1;
	
	}

	/*
	for (int i = 0; i < 6; i++) {
		printf("ct Ri[%d] = %.18e\n",i,Ri[i]);
	}
	printf("\n");
	*/

	free(RMATi);
	free(eye5_J5);
	free(J5);
	free(Param);

	return Ri;
	
}


/*
 * xorbit_dp() 
 * Closed orbit for fixed off-momentum (dp)
 */
double *xorbit_dp(atSuperRing_t *SR, double dp, int32_t *beam_loss) 
{
	struct parameters *Param = getSRParam(SR);

	double XYStep = atGetDouble(SR->atProp,"XYStep");
	uint32_t num_turns = 1, num_particles = 5;	

	double *Ri = (double *) malloc(6 * sizeof(double));
	memset(Ri, 0, 6 * sizeof(double));

	if (*beam_loss) {
		free(Param);
		return Ri;
	}

	Ri[4] = dp;

	double scaling[6] = {XYStep,XYStep,XYStep,XYStep,0,0};

	// send in the ring seven different particles, six of them are perturbed
	// the last one is propagated without offsets
	double *RMATi = (double *) malloc(num_particles * 6 * sizeof(double));
	memset(RMATi, 0, num_particles * 6 * sizeof(double));

	double *eye4_J4 = (double *) malloc((num_particles-1) * (num_particles-1) * sizeof(double));
	double *J4 = (double *) malloc((num_particles-1) * (num_particles-1) * sizeof(double));

	double change = 1e100, dps = 1.0000e-12;
	uint32_t itercount = 0, max_iterations = 20;

	// select subindexes from a {1:6,1:6} matrix to {1:4,1:4} submatrix, MATLAB order 
	int R_idx[16] = {0,1,2,3,6,7,8,9,12,13,14,15,18,19,20,21};
	int R_off[4] = {24,25,26,27};

	while ((change > dps) && (itercount < max_iterations)) {

		// RMATi= Ri(:,ones(1,7)) + D;
		for (size_t i = 0; i < num_particles * 6; i++) {
			RMATi[i] = Ri[i % 6];
		}	
		for (size_t i = 0; i < num_particles - 1; i++) {
			RMATi[i*6 + i] += scaling[i % 6];
		}
	
		// RMATf = linepass(RING,RMATi,args{:});	
		num_turns = 1;
	
		atpass(SR, RMATi, num_particles, num_turns, NULL, 0, Param, beam_loss);
		if (*beam_loss)
			break;

		// J4 = (RMATf(1:4,1:4)-RMATf(1:4,5))./scaling;
		// memset(J6, 0, (num_particles-1) * 6 * sizeof(double));

		// then subtract the output of the unperturbed particle to the perturbed ones
		// and normalize with the scaling factor
		for (size_t i = 0; i < (num_particles - 1) * (num_particles-1); i++) {
			J4[i] = RMATi[R_idx[i]] - RMATi[R_off[i % (num_particles-1)]];
			J4[i] /= scaling[i / (num_particles-1)];
		}

		// now isolate the parameters of the unperturbed particle and remove energy terms
		double Rf[6];
		for (int i = 0; i < 6; i++) {
			Rf[i] = RMATi[6*(num_particles-1)+i]-Ri[i];
		}

		// eye(4)
		memset(eye4_J4, 0, (num_particles-1) * (num_particles-1) * sizeof(double));
		for (size_t i = 0; i < (num_particles - 1); i++) 
			eye4_J4[i*4+i] = 1;
		
		//(eye(4)-J4) , we need to transpose it because of the ordering
		for (size_t i = 0; i < (num_particles - 1) * (num_particles-1); i++) {
			eye4_J4[i] -= J4[i];
		}

		double Ri_next[6] = {0,0,0,0,0,0}; double gsl_x[6];
		// solve linear system (eye(4)-J4)\(Rf-Ri)   ---- Rf = Rf - Ri
		linear_solve(eye4_J4, Rf, gsl_x, (num_particles - 1), (num_particles - 1), 1);

		// Ri_next = Ri + (eye(4)-J4)\(Rf-Ri);
		for (unsigned int i = 0; i < (num_particles-1); i++) {
			Ri_next[i] = Ri[i] + gsl_x[i];
		}

		// change = norm(Ri_next - Ri);
		change = norm_diff(Ri_next,Ri, (num_particles-1));

		memcpy(Ri, Ri_next, (num_particles-1) *sizeof(double));

		itercount = itercount+1;
	
	}

	free(RMATi);
	free(eye4_J4);
	free(J4);
	free(Param);

	return Ri;
	
}




/*
 * atMCF() 
 * It computes the momentum compaction for off-momentum dpp
 */
double atMCF(atSuperRing_t *SR, double dp0, int32_t *beam_loss)
{
	double dpp = 1e-6;
	double a;
	
	struct parameters *Param = getSRParam(SR);

#ifdef OLD_SERIAL_CODE
	double *fpdown, *fpup;

	fpdown = xorbit_dp(SR,dp0-0.5*dpp, beam_loss);	
	fpup = xorbit_dp(SR,dp0+0.5*dpp, beam_loss);	

	atpass(SR, fpdown, 1, 1, NULL, 0, Param, beam_loss);
	atpass(SR, fpup, 1, 1, NULL, 0, Param, beam_loss);

	if (*beam_loss)
		a = 0;
	else
		a = (fpup[5] - fpdown[5]) / (dpp * atGetDouble(SR->atProp,"Length"));

	free(fpdown);
	free(fpup);	
	free(Param);
#endif


	double *fp[2]; //0=down 1=up

	#pragma omp parallel num_threads(2) 
	{
	#pragma omp for
	for (int i = 0; i < 2; i++) {
		fp[i] = xorbit_dp(SR,dp0-(0.5+(double)i)*dpp, beam_loss);		
		atpass(SR, fp[i], 1, 1, NULL, 0, Param, beam_loss);
	}

	} //#pragma omp parallel num_threads(2)	

	if (*beam_loss)
		a = 0;
	else
		a = (fp[0][5] - fp[1][5]) / (dpp * atGetDouble(SR->atProp,"Length")); // inverted 1 with 0 due OMP??
	

	free(fp[0]);
	free(fp[1]);	
	free(Param);


	return a;
}	


/*
 * atchromaticity() 
 * It computes the SR chromaticity 
 */
void atchromaticity(atSuperRing_t *SR, int32_t *beam_loss)
{
	int N = 6;
	double Frf = atGetDouble(SR->atProp,"Frequency");
	double DPStep = atGetDouble(SR->atProp,"DPStep");	

	uint32_t num_elem = 1;
	unsigned int *refpts = malloc(sizeof(int) * num_elem);
	for (unsigned int i = 0; i < num_elem; i++) {
		refpts[i] = i;
	}
	
	atSuperRing_t *SR_rfoff = clone_atSuperRing(SR);
	double a;
	a = atMCF(SR_rfoff, 0, beam_loss);
	destroy_atSuperRing(SR_rfoff);

	SR->ringdata.mcf = a;

	// DFStep=-DPStep*mcf(atradoff(ring))*frf;
	double DFStep = - DPStep * a * Frf;

	double o1v[2];
	double theta_t_v[2][3];
	
	double freq_set[2];
	freq_set[0] = Frf + 0.5*DFStep;
	freq_set[1] = Frf - 0.5*DFStep;

	#pragma omp parallel num_threads(2) 
	{
	//int id = omp_get_thread_num();  // Get thread ID
	#pragma omp for
	for (int i = 0; i < 2; i++) {
	
		atSuperRing_t *SR_tmp = clone_atSuperRing(SR);

		double *M66 = malloc(6 * 6 * sizeof(double));
		double *M66_refpts = malloc(6 * 6 * num_elem * sizeof(double));
		double *M66t = malloc(N*N*sizeof(double));
		double *o1;
		
		gsl_matrix *A; gsl_vector_complex *lambda;
		amat_alloc(N, &A, &lambda);
		
		atSetRFFrequency(SR_tmp, freq_set[i]); 
			
		// [~,o1P]=findorbit(rgup,'guess',orbitin,varargs{:}); 
		
		o1 = xorbit_6(SR_tmp, beam_loss);
	
		findm66(SR_tmp, o1, refpts, num_elem, M66, M66_refpts, NULL, beam_loss); 
		transpose(M66,M66t,N,N);
		amat(M66t, NULL, N, A, lambda);	

		gsl_vector *r = gsl_vector_alloc(lambda->size);
		gsl_vector *theta_t = gsl_vector_alloc(lambda->size); 

		//tunes=mod(angle(vals)/2/pi,1);
		gsl_convert_vector_complex_rect_to_polar(lambda, r, theta_t);

		for (unsigned int j = 0; j < theta_t->size; j++) 
			gsl_vector_set(theta_t, j, fmod(gsl_vector_get(theta_t, j) / 2 / PI+1,1));
				
		
	
		o1v[i] = o1[4];
		for (int j = 0; j < 3; j++) {
			theta_t_v[i][j] = gsl_vector_get(theta_t,j);
		}

		gsl_vector_free(r);
		gsl_vector_free(theta_t);

		amat_free(&A, &lambda);	
		free(o1);
		free(M66t);
		free(M66_refpts);
		free(M66);
		destroy_atSuperRing(SR_tmp);		

	}	

	} //#pragma omp parallel num_threads(2)

	// store chromaticity
	for (int j = 0; j < 3; j++) {
		if (*beam_loss)
			SR->ringdata.chromaticity[j] = 0;
		else
			SR->ringdata.chromaticity[j] = (theta_t_v[0][j] - theta_t_v[1][j]) / (o1v[0] - o1v[1]);
	}

	free(refpts);

}




/*
 * xorbit_6() 
 * Closed orbit for 6D motion
 */
double *xorbit_6(atSuperRing_t *SR, int *beam_loss) 
{
	struct parameters *Param = getSRParam(SR);

	double XYStep = atGetDouble(SR->atProp,"XYStep");
	double DPStep = atGetDouble(SR->atProp,"DPStep");
	double Frf = atGetDouble(SR->atProp,"Frequency");
	double Vcell = atGetDouble(SR->atProp,"Voltage"); // sum of all rf plant voltages 
	double energy = 0;

	if (atGetPointer(SR->atProp, "Energy")) 
		energy = atGetDouble(SR->atProp, "Energy");


	double *Ri = (double *) malloc(6 * sizeof(double));
	memset(Ri, 0, 6 * sizeof(double));

	if (*beam_loss) {
		free(Param);
		return Ri;
	}

	uint32_t num_turns = 1, num_particles = 1;	
	double *r_in;
	r_in = (double *) malloc(6 * sizeof(double));
	memset(r_in, 0, 6 * sizeof(double));

	// calculate U0 with only one particle with zero offset
	//ringtmp=atdisable_6d(ring,'allpass','','cavipass','auto');
	// o6=ringpass(ringtmp,o0);
	// DO NOT DISABLE RF, ACTUALLY IT IS NOT EFFECTIVE IN ALL THE CALCULATIONS
	atSuperRing_t *SR_6d_off = clone_atSuperRing(SR);
	atRFOff(SR_6d_off);
	atpass(SR_6d_off, r_in, num_particles, num_turns, NULL, 0, Param, beam_loss);
	destroy_atSuperRing(SR_6d_off);

	//exit(0); // GG printf(""""

	//atpass(SR, r_in, num_particles, num_turns, NULL, 0, Param, beam_loss);

	double U0 = -r_in[4]  * energy;
	double HarmNumber = round(Frf * Param->RingLength / C0);


	/*/for (int i = 0; i < 6; i++) {
		printf("r_in[%d]=%1.12e beam_loss=%d U0=%f\n",i,r_in[i],*beam_loss,U0);
	}
	exit(0);*/
	


	Ri[5] = -Param->RingLength/(2 * PI * HarmNumber) * asin(U0/Vcell);

	double theta[6] = {0,0,0,0,0,C0 *(HarmNumber/Frf - Param->T0)};



	double scaling[6] = {XYStep,XYStep,XYStep,XYStep,DPStep,DPStep};

	// send in the ring seven different particles, six of them are perturbed
	// the last one is propagated without offsets
	num_particles = 6 + 1;
	double *RMATi = (double *) malloc(num_particles * 6 * sizeof(double));
	memset(RMATi, 0, num_particles * 6 * sizeof(double));

	double *eye6_J6 = (double *) malloc((num_particles-1) * 6 * sizeof(double));
	double *J6 = (double *) malloc((num_particles-1) * 6 * sizeof(double));

	double change = 1e100, dps = 1.0000e-12;
	uint32_t itercount = 0, max_iterations = 20;

	/*for (int i = 0; i < 6; i++) {
		printf("Ri[%d]=%1.12e\n",i,Ri[i]);
	}*/	

	while ((change > dps) && (itercount < max_iterations)) {

		//RMATi= Ri(:,ones(1,7)) + D;
		for (size_t i = 0; i < num_particles * 6; i++) {
			RMATi[i] = Ri[i % 6];
		}	
		for (size_t i = 0; i < num_particles - 1; i++) {
			RMATi[i*6 + i] += scaling[i % 6];
		}
			
		// RMATf = linepass(RING,RMATi,args{:});	
		num_turns = 1;	

		atpass(SR, RMATi, num_particles, num_turns, NULL, 0, Param, beam_loss);
		if (*beam_loss)
			break;

		// J6 = (RMATf(:,1:6)-RMATf(:,7))./scaling;
		// memset(J6, 0, (num_particles-1) * 6 * sizeof(double));

		// then subtract the output of the unperturbed particle to the perturbed ones
		// and normalize with the scaling factor
		for (size_t i = 0; i < (num_particles - 1) * 6; i++) {
			J6[i] = RMATi[i] - RMATi[(num_particles - 1) * 6 + i % 6];
			J6[i] /= scaling[i / 6];
		}

		// now isolate the parameters of the unperturbed particle and remove energy terms
		double Rf[6];
		for (int i = 0; i < 6; i++) {
			Rf[i] = RMATi[6*(num_particles-1)+i] - Ri[i] - theta[i];
		}

		// eye(6)
		memset(eye6_J6, 0, (num_particles-1) * 6 * sizeof(double));
		for (size_t i = 0; i < (num_particles - 1); i++) 
			eye6_J6[i*6+i] = 1;
		
		//(eye(6)-J6) , we need to transpose it because of the ordering
		for (size_t i = 0; i < (num_particles - 1) * 6; i++) {
			eye6_J6[i] -= J6[i];
		}

		double Ri_next[6] = {0,0,0,0,0,0}; double gsl_x[6];
		// solve linear system (eye(6)-J6)\(Rf-Ri-theta)
		linear_solve(eye6_J6, Rf, gsl_x, (num_particles - 1), (num_particles - 1), 1);

		
		// Ri_next = Ri + (eye(6)-J6)\(Rf-Ri-theta);
		for (int i = 0; i < 6; i++) {
			Ri_next[i] = Ri[i] + gsl_x[i];
		}

		// change = norm(Ri_next - Ri);
		change = norm_diff(Ri_next,Ri, 6);

		memcpy(Ri, Ri_next, 6 *sizeof(double));

		itercount = itercount+1;
	
	}
	
	//exit(0);

	free(r_in);
	free(RMATi);
	free(eye6_J6);
	free(J6);
	free(Param);
	
	return Ri;
	
}


double *findorbit(atSuperRing_t *SR, double *Ri, unsigned int *refpts, unsigned int size_refpts, int32_t *beam_loss)
{
	double *closed_orbit;
	struct parameters *Param = getSRParam(SR);
	int num_particles = 1, num_turns = 1;
	closed_orbit = atpass(SR, Ri, num_particles, num_turns, refpts, size_refpts, Param, beam_loss);

	free(Param);

	return closed_orbit;

}


void findm66(atSuperRing_t *SR, double *Ri, unsigned int *refpts, unsigned int size_refpts, double *M66, double *M66_refpts, double *closed_orbit, int32_t *beam_loss) 
{

//	printf("*********** FINDM66 *************\n");

	struct parameters *Param = getSRParam(SR);
	double XYStep = atGetDouble(SR->atProp,"XYStep");
	double DPStep = atGetDouble(SR->atProp,"DPStep");
	double scaling[6] = {XYStep,XYStep,XYStep,XYStep,DPStep,DPStep};

	// 6+, 6- particles plus one particle with zero offsets
	unsigned int num_particles = 13;
	double *D = (double *) malloc(6 * num_particles * sizeof(double));
	memset(D, 0, 6 * num_particles * sizeof(double));



	//scaling=XYStep*[1 1 1 1 0 0] + DPStep*[0 0 0 0 1 1];
	//D6 = 0.5*diag(scaling);
	for (int i = 0; i < 6; i++)
		D[i*6+i] = scaling[i] / 2;
	for (int i = 0; i < 6; i++)
		D[6*6+i*6+i] = -scaling[i] / 2;	

	// Add to the orbit_in. First 12 columns for derivative
	// 13-th column is for closed orbit
	// RIN = orbitin + [D6 -D6 zeros(6,1)];	
	// use the Ri_ref out of the Jacobian	
	for (unsigned int i = 0; i < 6 * num_particles; i++) {
		D[i] += Ri[i % 6];	
	}

	//When REFPTS= [  .. length(RING)+1] the last point is the exit of the last element
    	//and the entrance of the first element after 1
    	unsigned int *refpts_one = malloc(sizeof(int)*(size_refpts+1));
    	unsigned int size_refpts_one = size_refpts + 1;
    	
    	for (unsigned int i = 0; i < size_refpts; i++) {
    		refpts_one[i] = refpts[i];
    	}
	refpts_one[size_refpts] = 0;

	// perform two turns, the last one just to collect the first element of the second turn
	double *orbs_rout = atpass(SR, D, num_particles, 2, refpts_one, size_refpts_one, Param, beam_loss);

	int offset = (6 * num_particles * (size_refpts_one - 1));

	for (int i = 0; i < 6 * 6; i++) {
		// first turn - second turn ... (6) is the close orbit
		//M66 = (TMAT3(:,1:6,end)-TMAT3(:,7:12,end))./scaling;
		M66[i] = (orbs_rout[i+offset] - orbs_rout[i+offset+6*6]) / scaling[i / 6];	
	}

	// Calculate matrices at all REFPTS
	if (M66_refpts != NULL) {
		for (unsigned int j = 0; j < size_refpts; j++) {
			offset = j * num_particles * 6;
			for (int i = 0; i < 6 * 6; i++) {
				// first turn - second turn ... (6) is the close orbit
				//varargout{1} = (TMAT3(:,1:6,reqs)-TMAT3(:,7:12,reqs))./scaling
				M66_refpts[i + j*6*6] = (orbs_rout[i+offset] - orbs_rout[i+6*6+offset]) / scaling[i / 6];	
			}	
		}
	}		
	
	// Return closed orbit if requested
	int cnt = 0;
	if (closed_orbit != NULL) {
		for (unsigned int j = 0; j < size_refpts; j++) {
			offset = j * num_particles * 6;
			for (unsigned int i = 0; i < 6; i++) {
				closed_orbit[cnt++] = orbs_rout[i+6*(num_particles-1)+offset];
			}
		}
	}			

	free(orbs_rout);
	free(refpts_one);
	free(Param);
	free(D);
	
}


double *atpass_elems(atSuperRing_t *SR, double *Ri,  
	uint32_t refpts_start, uint32_t refpts_end, struct parameters *Param, int32_t *beam_loss)
{
	double *Rout;
	uint32_t key_cnt;
	uint32_t num_particles = 1;

	double min_val = atGetDouble(SR->atProp,"ClosedOrbitMinVal");	
	double max_val = atGetDouble(SR->atProp,"ClosedOrbitMaxVal");
	
	Rout = Ri;
	mxArray *atRingIter = SR->atRing;
	key_cnt = refpts_start;	

	do {
		char idx[5];sprintf(idx,"%d",key_cnt); 

		// generate the fieldname to extract element which is always found immediately
		// because of the shift of the Iter pointer
		mxArray *atElem = (mxArray *)atGetPointer(atRingIter,idx);

		// get pointer to data struct used in pass function
		struct elem *Elem = (struct elem *)atGetPointer(atElem,"Elem");
		// get pointer of pass function
		track_function_ptr pass_track = (track_function_ptr) atGetPointer(atElem,"TrackFunction");

		// very small values slow down calculations
		for (int t = 0; t < num_particles * 6; t++) {
			if (fabs(Ri[t]) < min_val)
				Ri[t] = 0;
		}
		
		pass_track(atElem, Elem, Ri, num_particles, Param);

		for (int t = 0; t < num_particles * 6; t++) {
			if (isnan(Ri[t]) || fabs(Ri[t]) > max_val) {
				*beam_loss = 1;
				break;
			}
		}
	
		key_cnt++;
		atRingIter = atRingIter->next;

	} while (key_cnt <= refpts_end); 	

	return Rout;

}






void build_1turn_map(atSuperRing_t *SR, double *M66, double *M66_refpts, int refpts_size, double *phi_out, double *bks_out, double *as_out, int32_t *beam_loss)
{
	int N = 6;
	double L0 = atGetDouble(SR->atProp,"Length");
	// locate amat
	gsl_matrix *A; gsl_vector_complex *lambda;
	amat_alloc(N, &A, &lambda);
	
	double *M66t = malloc(6*6*sizeof(double));
	
	// GSL works in matrix C order, transpose data 
	transpose(M66,M66t,6,6);	
	
	
	int err= amat(M66t, NULL, N, A, lambda);	

	if (err == -1) {
		*beam_loss = 1;
		amat_free(&A, &lambda);
		free(M66t);
		return;
	}

	#pragma omp parallel num_threads(2) //shared(A, lambda, N, M66_refpts, refpts_size, phi_out, bks_out, as_out, L0, SR)
	{

	gsl_vector *phi;gsl_matrix **bk;gsl_matrix *astdi;
	r_analysis_alloc(N, &phi, &bk, &astdi);	

	//int id = omp_get_thread_num();  // Get thread ID
	//#pragma omp for
	for (int i = 0; i < refpts_size; i++) {		
		gsl_matrix *mi = gsl_matrix_alloc_p_trasp(M66_refpts + 6*6*i , N, N); 

		//print_matrix(A); GG qua la matrice ha già i segni invertiti rispetto AT

		r_analysis(A, mi, phi, bk, astdi);
		gsl_matrix_free(mi);	

		// phi is 3 element size
		// bk is 3x6x6 element size
		// astdi is 6x6 element size
		memcpy(phi_out + 3*i, phi->data, 3*sizeof(double));

		// bisogna trasporre i dati (???)
		for (int j = 0; j < 3; j++) {
			memcpy(bks_out + 3*6*6*i + 6*6*j, bk[j]->data, 6*6*sizeof(double));
		}	

		// bisogna trasporre i dati (???) siamo fuori da dominio matlab
		memcpy(as_out + 6*6*i, astdi->data, 6*6*sizeof(double));

		// get ringdata at first iteration
		if (i == 0) {
			gsl_vector *r = gsl_vector_alloc(phi->size);
			gsl_vector *theta_t = gsl_vector_alloc(phi->size); 
			gsl_convert_vector_complex_rect_to_polar(lambda, r, theta_t);

			for (unsigned int j = 0; j < theta_t->size; j++) 
				gsl_vector_set(theta_t, j, fmod(gsl_vector_get(theta_t, j) / 2 / PI+1,1));

			gsl_vector *damping_rates = gsl_vector_alloc(lambda->size); 
			gsl_vector *damping_times = gsl_vector_alloc(lambda->size);
			gsl_vector_complex_logabs(lambda, damping_rates);
	
			for (unsigned int j = 0; j < damping_times->size; j++) {
				gsl_vector_set(damping_times, j, -L0/C0/gsl_vector_get(damping_rates,j));
			}

			for (unsigned int j = 0; j < 3; j++) {
				SR->ringdata.tunes[j] = gsl_vector_get(theta_t,j);
				SR->ringdata.damping_times[j] = gsl_vector_get(damping_times ,j);
				SR->ringdata.damping_rates[j] = gsl_vector_get(damping_rates ,j);
			}

			gsl_vector_free(damping_times);
			gsl_vector_free(damping_rates);
			gsl_vector_free(r);
			gsl_vector_free(theta_t);		
			
		}
		
	}	

	r_analysis_free(N, &phi, &bk, &astdi);

	} // #pragma omp parallel num_threads(2)

	free(M66t);

	amat_free(&A, &lambda);

}
 



void atlinopt6(atSuperRing_t *SR, int32_t *beam_loss) 
{

	uint32_t num_elem = atGetLong(SR->atProp,"NumElem");
	uint32_t *refpts = malloc(sizeof(int) * num_elem);

	for (unsigned int i = 0; i < num_elem; i++) {
		refpts[i] = i;
	}	

	// Closed orbit for 6D motion after one turn
	double *Ri = xorbit_6(SR, beam_loss); 
	double Ri_orbit[6];
	memcpy(Ri_orbit, Ri, sizeof(double) * 6);	

	// Find the closed orbit
	double *orbs; 
	orbs = findorbit(SR, Ri_orbit, refpts, num_elem, beam_loss);

	// findm66 finds the full one-turn 6-by-6
	// matrix at the entrance of the first element
	double *M66 = malloc(6 * 6 * sizeof(double));
	double *M66_refpts = malloc(6 * 6 * num_elem * sizeof(double));
	findm66(SR, Ri, refpts, num_elem, M66, M66_refpts, NULL, beam_loss); 

	double *phi_out = malloc(3 * num_elem * sizeof(double));
	double *bks_out = malloc(3 * 6 * 6 * num_elem * sizeof(double));
	double *as_out = malloc(6 * 6 * num_elem * sizeof(double));	
		
	// phi_out, bks_out, as_out matrices now are in C order	
	build_1turn_map(SR, M66, M66_refpts, num_elem, phi_out, bks_out, as_out, beam_loss);

	double *mu_x_raw = malloc(num_elem*sizeof(double));
	double *mu_y_raw = malloc(num_elem*sizeof(double));

	if (!(*beam_loss)) {
		// [mux, muy] 	Betatron phase advances
		for (unsigned int i = 0; i < num_elem; i++) {
			mu_x_raw[i] = phi_out[3*i];
			mu_y_raw[i] = phi_out[3*i+1];		
		}
		unwrap_array(mu_x_raw, SR->elemdata.mu_x, num_elem);	
		unwrap_array(mu_y_raw, SR->elemdata.mu_y, num_elem);	


		for (unsigned int i = 0; i < num_elem; i++) {
			// alpha=-[ri(2,1,1) ri(4,3,2)];
			SR->elemdata.alpha_x[i] = -bks_out[6*6*3*i+1];
			SR->elemdata.alpha_y[i] = -bks_out[3 + 6*2 + 6*6 + 6*6*3*i];
			// beta=[ri(1,1,1) ri(3,3,2)];	
			SR->elemdata.beta_x[i] = bks_out[6*6*3*i];
			SR->elemdata.beta_y[i] = bks_out[2 + 6*2 + 6*6 + 6*6*3*i];
			// disp=ri(1:4,5,3)/ri(5,5,3); 
			SR->elemdata.eta_x[i] = bks_out[0 + 6*4 + 6*6*2 + 6*6*3*i] / bks_out[4 + 6*4 + 6*6*2 + 6*6*3*i];
			SR->elemdata.eta_xp[i] = bks_out[1 + 6*4 + 6*6*2 + 6*6*3*i] / bks_out[4 + 6*4 + 6*6*2 + 6*6*3*i];	
			SR->elemdata.eta_y[i] = bks_out[2 + 6*4 + 6*6*2 + 6*6*3*i] / bks_out[4 + 6*4 + 6*6*2 + 6*6*3*i];
			SR->elemdata.eta_yp[i] = bks_out[3 + 6*4 + 6*6*2 + 6*6*3*i] / bks_out[4 + 6*4 + 6*6*2 + 6*6*3*i];
			SR->elemdata.x[i] = orbs[6*i];
			SR->elemdata.xp[i] = orbs[6*i+1];		
			SR->elemdata.y[i] = orbs[6*i+2];
			SR->elemdata.yp[i] = orbs[6*i+3];
			SR->elemdata.dpd[i] = orbs[6*i+4];
			SR->elemdata.ct[i] = orbs[6*i+5];
			
			char idx[5];sprintf(idx,"%d",refpts[i]); 
			mxArray *atRingIter = SR->atRing;
			mxArray *atElem = (mxArray *)atGetPointer(atRingIter,idx);
			SR->elemdata.spos[i] = atGetDouble(atElem,"SPos");
					
		}

		// set integer tunes
		// horizontal
		SR->ringdata.int_tunes[0] =  (int)(SR->elemdata.mu_x[num_elem-1] / 2 / PI + 0.5);
		// vertical
		SR->ringdata.int_tunes[1] =  (int)(SR->elemdata.mu_y[num_elem-1] / 2 / PI + 0.5);
	}
	else {
		// [mux, muy] 	Betatron phase advances
		for (unsigned int i = 0; i < num_elem; i++) {
			SR->elemdata.mu_x[i] = 0;
			SR->elemdata.mu_y[i] = 0;		
		}

		for (unsigned int i = 0; i < num_elem; i++) {
			// alpha=-[ri(2,1,1) ri(4,3,2)];
			SR->elemdata.alpha_x[i] = 0;
			SR->elemdata.alpha_y[i] = 0;
			// beta=[ri(1,1,1) ri(3,3,2)];	
			SR->elemdata.beta_x[i] = 0;
			SR->elemdata.beta_y[i] = 0;
			// disp=ri(1:4,5,3)/ri(5,5,3); 
			SR->elemdata.eta_x[i] = 0;
			SR->elemdata.eta_xp[i] = 0;	
			SR->elemdata.eta_y[i] = 0;
			SR->elemdata.eta_yp[i] = 0;
			SR->elemdata.x[i] = 0;
			SR->elemdata.xp[i] = 0;		
			SR->elemdata.y[i] = 0;
			SR->elemdata.yp[i] = 0;
			SR->elemdata.dpd[i] = 0;
			SR->elemdata.ct[i] = 0;
			
			char idx[5];sprintf(idx,"%d",refpts[i]); 
			mxArray *atRingIter = SR->atRing;
			mxArray *atElem = (mxArray *)atGetPointer(atRingIter,idx);
			SR->elemdata.spos[i] = atGetDouble(atElem,"SPos");
					
		}

		// set integer tunes
		// horizontal
		SR->ringdata.int_tunes[0] =  0;
		// vertical
		SR->ringdata.int_tunes[1] =  0;

	}

	free(mu_x_raw);
	free(mu_y_raw);		
	
	free(phi_out);
	free(bks_out);
	free(as_out);
	
	free(M66_refpts);
	free(M66);
	free(orbs);
	free(Ri);
	free(refpts);

}


uint32_t *find_radOn_idx(atSuperRing_t *SR) 
{
	mxArray *atRingIter = SR->atRing;
	uint32_t num_elem = atGetLong(SR->atProp,"NumElem");
	
	uint32_t *radon_idx = malloc(num_elem * sizeof(uint32_t));
	for (uint32_t i = 0; i < num_elem; i++) {
		radon_idx[i] = 0xffffffff;
	}

	if (atRingIter == NULL) {
		printf("Ring is empty\n");
		return NULL;
	} 
	
	int rad_on_cnt = 0;
	int cnt = 0;
	do {
		char idx[20];sprintf(idx,"%d",cnt);
		mxArray *atElem = (mxArray *)atGetPointer(atRingIter,idx);
		if ((atElem != NULL) && (atGetPointer(atElem, "PassMethod"))) {
			if (substr_match(atGetPointer(atElem, "PassMethod"),"Rad") == 1) {					
				radon_idx[rad_on_cnt++] = cnt;		
			}	
		}	
		cnt++;
		if (atRingIter != NULL) 
			atRingIter = atRingIter->next;
		
	} while (atRingIter != NULL); 	
	
	return radon_idx;	

}


void findelemm66(double XYStep, struct parameters *Param, mxArray *atElem, double *Ri, double *M66, double min_val, double max_val, int32_t *beam_loss)
{
	double scaling[6] = {XYStep,XYStep,XYStep,XYStep,XYStep,XYStep};
	
	int num_particles = 12;
	double *D = (double *) calloc(6 * num_particles, sizeof(double));
	
	//D6 = 0.5*diag(scaling);
	for (int i = 0; i < 6; i++)
		D[i*6+i] = scaling[i] / 2;
	for (int i = 0; i < 6; i++)
		D[6*6+i*6+i] = -scaling[i] / 2;		

	for (int i = 0; i < 6 * num_particles; i++) {
		D[i] += Ri[i % 6];	
	}

	// get pointer to data struct used in pass function
	struct elem *Elem = (struct elem *)atGetPointer(atElem,"Elem");
	// get pointer of pass function
	track_function_ptr pass_track = (track_function_ptr) atGetPointer(atElem,"TrackFunction");

	// very small values slow down calculations
	for (int t = 0; t < num_particles * 6; t++) {
		if (fabs(D[t]) < min_val)
			D[t] = 0;
	}

	pass_track(atElem, Elem, D, num_particles, Param);

	for (int t = 0; t < num_particles * 6; t++) {
		if (isnan(D[t]) || fabs(D[t]) > max_val) {
			*beam_loss = 1;
		}
	}

	for (int i = 0; i < 6 * 6; i++) {
		//M66 = (ROUT(:,1:6)-ROUT(:,7:12))./scaling;
		M66[i] = (D[i] - D[i+6*6]) / scaling[i / 6];	
	}
	
	free(D);

}


void atohmienvelope(atSuperRing_t *SR, int32_t *beam_loss) 
{
	uint32_t num_elem = atGetLong(SR->atProp,"NumElem");
	double energy = atGetDouble(SR->atProp, "Energy");
	double XYStep = atGetDouble(SR->atProp,"XYStep");
	double min_val = atGetDouble(SR->atProp,"ClosedOrbitMinVal");	
	double max_val = atGetDouble(SR->atProp,"ClosedOrbitMaxVal");

	struct parameters *Param = getSRParam(SR);
	
	uint32_t *refpts = malloc(sizeof(int) * num_elem);

	uint32_t *radon_idx = find_radOn_idx(SR);
	
	for (unsigned int i = 0; i < num_elem; i++) {
		refpts[i] = i;
	}	

	// Closed orbit for 6D motion after one turn
	double *Ri = xorbit_6(SR, beam_loss); 

	// findm66 finds the full one-turn 6-by-6
	// matrix at the entrance of the first element


	
	double *M66 = malloc(6 * 6 * sizeof(double));
	double *M66_elem = malloc(6 * 6 * sizeof(double));
	double *M66_refpts = malloc(6 * 6 * num_elem * sizeof(double));
	double *closed_orbit  = (double *) malloc(6 * sizeof(double) *  num_elem);
	
	findm66(SR, Ri, refpts, num_elem, M66, M66_refpts, closed_orbit, beam_loss); 

	double *B = calloc(6 * 6, sizeof(double));
	double *BCum = calloc(6 * 6, sizeof(double));
	double *BCUM_elem_t = calloc(6 * 6, sizeof(double));
	double *Batbeg = malloc(6 * 6 * num_elem * sizeof(double));
	
	double closed_orbit_elem[6];
	mxArray *atRingIter = SR->atRing;
	
	int rad_on_cnt = 0;
	unsigned int cnt = 0;

	// questo si può ridurre se invece che gli indici vengono restituiti i puntatori agli elementi
	do {
		char idx[20];sprintf(idx,"%d",cnt);
		mxArray *ElemData = (mxArray *)atGetPointer(atRingIter,idx);

		memcpy(closed_orbit_elem, &closed_orbit[6*cnt], 6 * sizeof(double));	

		if (radon_idx[rad_on_cnt] == cnt) {
			memset(B, 0, 6 * 6 * sizeof(double));
			diffmatrix((const atElem *)ElemData, &closed_orbit[6*cnt], energy, B);
			rad_on_cnt++;
		}	
		else {
			memset(B, 0, 6 * 6 * sizeof(double));
		}
	
		//m=findelemm66(elem,elem.PassMethod,orbit);
		findelemm66(XYStep, Param, (atElem *)ElemData, closed_orbit_elem, M66_elem, min_val, max_val, beam_loss);

		if (*beam_loss)
			break;

		//CUM = m*BCUM*m' + b;
		//matrix_mul_t1_colord(M66_elem,  BCum, 6, BCUM_elem_t);
		//matrix_mul(BCUM_elem_t, M66_elem, 6, BCum);
		matrix_mul_t1(M66_elem,  BCum, 6, BCUM_elem_t);
		matrix_mul(BCUM_elem_t, M66_elem, 6, BCum);
		
		//matrix_mul_t1_tout(BCum,  M66_elem, 6, BCUM_elem_t);
		//matrix_mul_t2(M66_elem,  BCUM_elem_t, 6, BCUM_elem_t);

		for (int i = 0; i < 36; i++) 
			BCum[i] += B[i];

		//btx=BCUM;
		memcpy(&Batbeg[cnt*6*6], BCum, 6 * 6 * sizeof(double)); 

		cnt++;
		// orb(1:end-1) 
		if (atRingIter != NULL) 
			atRingIter = atRingIter->next;
			
	} while (atRingIter != NULL); 	


	if (*beam_loss) {
		for (unsigned int i = 0; i < num_elem; i++) {
			SR->elemdata.tilt[i] = 0;
			SR->elemdata.sigma_x[i] = 0;
			SR->elemdata.sigma_y[i] = 0;
		}
		free(Batbeg);
		free(BCUM_elem_t);
		free(BCum);
		free(B);
		free(M66);
		free(M66_elem);
		free(M66_refpts);
		free(closed_orbit);
		free(refpts);
		free(Param);
		free(Ri);
		free(radon_idx);
		return;
	}


	gsl_matrix *mring = gsl_matrix_alloc_p_trasp(M66, 6, 6);	
	gsl_matrix *bcum = gsl_matrix_alloc_p_trasp(BCum, 6, 6);	
	
	gsl_matrix *AA = gsl_matrix_alloc(6, 6);
	gsl_matrix *BB = gsl_matrix_alloc(6, 6);
	gsl_matrix *CC = gsl_matrix_alloc(6, 6);
	
	gsl_matrix_inv(mring, AA);
	gsl_matrix_transpose2_memcpy(mring, BB);
	gsl_matrix_scale(BB, -1);
	gsl_matrix_mul(AA,  bcum, CC);

	double *R = lapack_solve_sylvester(AA->data, BB->data, CC->data, 6, LAPACK_ROW_MAJOR);

	SR->ringdata.rmsdp = pow(R[28],0.5);  // sqrt(R(5,5));   % R.M.S. energy spread
	SR->ringdata.rmsbl = pow(R[35],0.5);  // sqrt(R(6,6));   % R.M.S. bunch lenght	

	//#pragma omp parallel num_threads(1) shared(R,M66_refpts,Batbeg)
	//{
	gsl_vector_complex *dr = gsl_vector_complex_alloc (2);
	gsl_matrix_complex *u = gsl_matrix_complex_alloc (2, 2);	
	double RM66[36], M66RM66[36], r1313[4];	

	//#pragma omp for 
	for (unsigned int i = 0; i < num_elem; i++) {
		
		memset(RM66,0,36*sizeof(double));
		memset(M66RM66,0,36*sizeof(double));
				
		matrix_mul_t1_tout(R, &M66_refpts[36*i], 6, RM66); 
		matrix_mul(RM66, &M66_refpts[36*i], 6, M66RM66);

		for (int j = 0; j < 6 * 6; j++) {
			M66RM66[j] += Batbeg[i*36+j];	
		}		

		r1313[0] = M66RM66[0];
		r1313[1] = M66RM66[12];
		r1313[2] = M66RM66[2];
		r1313[3] = M66RM66[14];		

		eig(r1313, dr, u, 2, 0, 0); 
		gsl_complex diff = gsl_complex_sub(gsl_matrix_complex_get(u,1,0), gsl_matrix_complex_get(u,0,1));
		gsl_complex arcsindiff = gsl_complex_arcsin(gsl_complex_mul_real(diff, 0.5));
		SR->elemdata.tilt[i] = GSL_REAL(arcsindiff);
		gsl_complex sigma_x = gsl_vector_complex_get(dr,0),
			sigma_y = gsl_vector_complex_get(dr,1);
		SR->elemdata.sigma_x[i] = pow(GSL_REAL(sigma_x),0.5);
		SR->elemdata.sigma_y[i] = pow(GSL_REAL(sigma_y),0.5);

	}

	gsl_vector_complex_free(dr);
	gsl_matrix_complex_free(u);
	
	//} //#pragma omp parallel num_threads(1)

	free(R);	
	gsl_matrix_free(mring);
	gsl_matrix_free(bcum);
	gsl_matrix_free(AA);
	gsl_matrix_free(BB);
	gsl_matrix_free(CC);
	
	free(Batbeg);
	free(BCUM_elem_t);
	free(BCum);
	free(B);
	

	free(M66);
	free(M66_elem);
	free(M66_refpts);
	free(closed_orbit);

	free(refpts);
	free(Param);
	free(Ri);
	free(radon_idx);

}


double *testorbit(atSuperRing_t *SR, int32_t *beam_loss) 
{
	uint32_t num_elem = atGetLong(SR->atProp,"NumElem");
	struct parameters *Param = getSRParam(SR);
	Param->RingLength = atGetDouble(SR->atProp,"Length");
	Param->T0 = Param->RingLength / C0;
	Param->charge = -1;

	uint32_t max_num_turns = 1, num_particles = 1;
	double *r_in;
	r_in = (double *) malloc(6 * sizeof(double));
	memset(r_in, 0, 6 * sizeof(double));
	
	// calculate U0 with only one particle
	// o6=ringpass(ring,o0);
	atpass(SR, r_in, num_particles, max_num_turns, NULL, 0, Param, beam_loss);
	
	double energy = 0; // undefined 
	double U0 = r_in[5]  * energy;
	
	double XYStep = 3.e-8; // min step used for differentiation
	double DPStep = 3.e-6;
	
	double Frf = atGetDouble(SR->atProp,"Frequency");
	
	double L0 = atGetDouble(SR->atProp,"Length");
	double T0 = Param->T0;
	double HarmNumber = round(Frf * L0 / C0);

	double Vcell = atGetDouble(SR->atProp,"Voltage"); // sum of all rf plant voltages 
	
	double *Ri = (double *) malloc(6 * sizeof(double));
	memset(Ri, 0, 6 * sizeof(double));

	Ri[5] = -L0/(2 * PI * HarmNumber) * asin(U0/Vcell);

	double theta[6] = {0,0,0,0,0,C0 *(HarmNumber/Frf - T0)};

	//printf("THETA[5]=%.8e length=%.8e\n",theta[5],Param->RingLength);

	double scaling[6] = {XYStep,XYStep,XYStep,XYStep,DPStep,DPStep};

	// send in the ring seven different particles, six of them are perturbed
	// the last one is propagated without offsets
	num_particles = 6 + 1;
	double *RMATi = (double *) malloc(num_particles * 6 * sizeof(double));
	memset(RMATi, 0, num_particles * 6 * sizeof(double));

	double *eye6_J6 = (double *) malloc((num_particles-1) * 6 * sizeof(double));
	double *eye6_J6_t = (double *) malloc((num_particles-1) * 6 * sizeof(double));	
	double *J6 = (double *) malloc((num_particles-1) * 6 * sizeof(double));

	double change = 1e100, dps = 1.0000e-12;
	uint32_t itercount = 0, max_iterations = 20;

	while ((change > dps) && (itercount < max_iterations)) {

		// RMATi= Ri(:,ones(1,7)) + D;
		for (size_t i = 0; i < num_particles * 6; i++) {
			RMATi[i] = Ri[i % 6];
		}	
		for (size_t i = 0; i < num_particles - 1; i++) {
			RMATi[i*6 + i] = scaling[i % 6];
		}
			
		// RMATf = linepass(RING,RMATi,args{:});	
		max_num_turns = 1;
		atpass(SR, RMATi, num_particles, max_num_turns, NULL, 0, Param, beam_loss);
		if (*beam_loss)
			break;
		
		// J6 = (RMATf(:,1:6)-RMATf(:,7))./scaling;
		// memset(J6, 0, (num_particles-1) * 6 * sizeof(double));

		// then subtract the output of the unperturbed particle to the perturbed ones
		// and normalize with the scaling factor
		for (size_t i = 0; i < (num_particles - 1) * 6; i++) {
			J6[i] = RMATi[i] - RMATi[(num_particles - 1) * 6 + i % 6];
			J6[i] /= scaling[i / 6];
		}

		// now isolate the parameters of the unperturbed particle and remove energy terms
		double Rf[6];
		for (int i = 0; i < 6; i++) {
			Rf[i] = RMATi[6*(num_particles-1)+i] - Ri[i] - theta[i];
		}

		// eye(6)
		memset(eye6_J6, 0, (num_particles-1) * 6 * sizeof(double));
		for (size_t i = 0; i < (num_particles - 1); i++) 
			eye6_J6[i*6+i] = 1;
		
		//(eye(6)-J6) , we need to transpose it because of the ordering
		for (size_t i = 0; i < (num_particles - 1) * 6; i++) {
			eye6_J6[i] -= J6[i];
		}

		double Ri_next[6] = {0,0,0,0,0,0}; double gsl_x[6];
		// solve linear system (eye(6)-J6)\(Rf-Ri-theta)
		linear_solve(eye6_J6, Rf, gsl_x, (num_particles - 1), (num_particles - 1), 1);

		
		// Ri_next = Ri + (eye(6)-J6)\(Rf-Ri-theta);
		for (int i = 0; i < 6; i++) {
			Ri_next[i] = Ri[i] + gsl_x[i];
		}

		// change = norm(Ri_next - Ri);
		change = norm_diff(Ri_next,Ri, 6);

		memcpy(Ri, Ri_next, 6 *sizeof(double));

		itercount = itercount+1;
	
	}

	double Ri_ref[6];
	memcpy(Ri_ref,Ri,6*sizeof(double));

	////////////////////// findorbit.m
	
	uint32_t *refpts = (uint32_t *) malloc(sizeof(uint32_t) *  num_elem);
	for (size_t i = 0; i < num_elem; i++)
		refpts[i] = i;
		
	
	double *orbs = atpass(SR, Ri, 1, 1, refpts, num_elem, Param, beam_loss);
	orbs[0] += 0;

	////////////////////// findm66.m

	num_particles = 13;
	double *D = (double *) malloc(6 * num_particles * sizeof(double));
	memset(D, 0, 6 * num_particles * sizeof(double));
	
	//scaling=XYStep*[1 1 1 1 0 0] + DPStep*[0 0 0 0 1 1];
	//D6 = 0.5*diag(scaling);
	for (int i = 0; i < 6; i++)
		D[i*6+i] = scaling[i] / 2;
	for (int i = 0; i < 6; i++)
		D[6*6+i*6+i] = -scaling[i] / 2;	

	// Add to the orbit_in. First 12 columns for derivative
	// 13-th column is for closed orbit
	// RIN = orbitin + [D6 -D6 zeros(6,1)];	
	// use the Ri_ref out of the Jacobian	
	for (size_t i = 0; i < 6 * num_particles; i++) {
		D[i] += Ri_ref[i % 6];	
		//printf("D[%d]=%.8e\n",i,D[i]);
	}

	//When REFPTS= [  .. length(RING)+1] the last point is the exit of the last element
    	//and the entrance of the first element after 1
	refpts[0] = 0;
	refpts[1] = num_elem;

	// perform two turns, the last one just to collect the first element of the second turn
	double *orbs_rout = atpass(SR, D, 13, 2, refpts, 2, Param, beam_loss);

	double *M66 = (double *) malloc(6 * 6 * sizeof(double));
	
	for (int i = 0; i < 6 * 6; i++) {
		// first turn - second turn ... (6) is the close orbit
		//M66 = (TMAT3(:,1:6,end)-TMAT3(:,7:12,end))./scaling;
		M66[i] = (orbs_rout[i+6*13] - orbs_rout[i+6*13+6*6]) / scaling[i / 6];
		//printf("M66[%d]=%.15e M66_esrf[%d]=%.15e diff[%d]=%.15e\n",i,M66[i],i,M66_esrf_t[i],i,M66[i]-M66_esrf_t[i]);
		//printf("orbs_rout[%d]=%.15e orbs_rout[%d]=%.15e M66[%d]=%.15e\n",i+6*13,orbs_rout[i+6*13],i+6*13+6*6,orbs_rout[i+6*13+6*6],i,M66[i]);	
	}
	
	// Calculate matrices at all REFPTS
	double *M66_pre  = (double *) malloc(6 * 6 * sizeof(double));
	for (int i = 0; i < 6 * 6; i++) {
		// first turn - second turn ... (6) is the close orbit
		//varargout{1} = (TMAT3(:,1:6,reqs)-TMAT3(:,7:12,reqs))./scaling
		M66_pre[i] = (orbs_rout[i] - orbs_rout[i+6*6]) / scaling[i / 6];	
	}	
	
	// Return closed orbit if requested
	double *closed_orbit  = (double *) malloc(6 * 6 * sizeof(double));
	for (int i = 0; i < 6; i++) {
		closed_orbit[i] = orbs_rout[i+6*12];
	}	


	int N = 6;
	// locate amat
	gsl_matrix *A; gsl_vector_complex *lambda;
	amat_alloc(N, &A, &lambda);
	
	double *M66t = malloc(6*6*sizeof(double));
	transpose(M66,M66t,6,6);
	
	int err = amat(M66t, NULL, N, A, lambda);
	if (err)
		*beam_loss = 1;	

	gsl_vector *phi;gsl_matrix **bk;gsl_matrix *astdi;
	r_analysis_alloc(N, &phi, &bk, &astdi);
	
	r_analysis(A, NULL, phi, bk, astdi);

	/*
	print_matrix(astdi);
	for (int i = 0; i < N/2; i++) {
		print_matrix(bk[i]);
	}
	*/

	gsl_vector *r = gsl_vector_alloc(phi->size);
	gsl_vector *theta_t = gsl_vector_alloc(phi->size); 
	gsl_convert_vector_complex_rect_to_polar(lambda, r, theta_t);

	for (unsigned int i = 0; i < theta_t->size; i++) 
		gsl_vector_set(theta_t, i, gsl_vector_get(theta_t, i) / 2 / PI);

	gsl_vector *damping_rates = gsl_vector_alloc(lambda->size); 
	gsl_vector *damping_times = gsl_vector_alloc(lambda->size);
	gsl_vector_complex_logabs(lambda, damping_rates);

	
	for (unsigned int i = 0; i < damping_times->size; i++) {
		gsl_vector_set(damping_times, i, -L0/C0/gsl_vector_get(damping_rates,i));
	}

	gsl_vector_free(damping_times);
	gsl_vector_free(damping_rates);
	gsl_vector_free(r);
	gsl_vector_free(theta_t);
	

	
	r_analysis_free(N, &phi, &bk, &astdi);
	amat_free(&A, &lambda);

	free(closed_orbit);
	free(M66_pre);
	free(M66);
	free(M66t);
	free(orbs_rout);
	free(D);
	free(eye6_J6);
	free(eye6_J6_t);
	free(J6);
	free(RMATi);
	free(Param);
	free(r_in);
	free(refpts);
	
	return Ri;
		
}




