
#include "r_analysis.h"

void r_analysis_alloc(int size, gsl_vector **phi, gsl_matrix ***bk, gsl_matrix **astdi)
{
	(*phi) = gsl_vector_alloc(size/2);
	(*bk) = (gsl_matrix **) malloc(size/2 * sizeof(gsl_matrix *));
	for (int i = 0; i < size/2; i++) {
		(*bk)[i] = gsl_matrix_alloc(size,size);
	}		
	(*astdi) = gsl_matrix_alloc(size,size);
}


void r_analysis_free(int size, gsl_vector **phi, gsl_matrix ***bk, gsl_matrix **astdi)
{
	gsl_vector_free((*phi));
	for (int i = 0; i < size/2; i++) 
		gsl_matrix_free((*bk)[i]);
	gsl_matrix_free((*astdi));
	free((*bk));
}


double r_analysis_getphase(gsl_matrix *a22)
{
	return atan2(gsl_matrix_get(a22, 0, 1),gsl_matrix_get(a22, 0, 0));

}


void getphase_matrix(gsl_matrix *A, int dms, gsl_vector *phi)
{
	gsl_matrix *a0_22 = gsl_matrix_alloc(2,2);
	for (int i = 0; i < dms; i++) {
		int range[2];
		range[0] = 0+2*i; range[1] = 1+2*i;
		
		gsl_matrix_square_submatrix(A, range, a0_22);
		
		double rot = r_analysis_getphase(a0_22);

		gsl_vector_set(phi, i, rot);
	}
	gsl_matrix_free(a0_22);		
}


void r_analysis_standardize(gsl_matrix *A, int dms, gsl_matrix *Astd, gsl_vector *phi)
{
	gsl_matrix *a0_22 = gsl_matrix_alloc(2,2);
	gsl_matrix *cs_sn = gsl_matrix_alloc(2,2);
	gsl_matrix *v = gsl_matrix_alloc(A->size1, cs_sn->size2);
	gsl_matrix *A_red = gsl_matrix_alloc(A->size1, cs_sn->size2);



	for (int i = 0; i < dms; i++) {
		int range[2];
		range[0] = 0+2*i; range[1] = 1+2*i;

		gsl_matrix_square_submatrix(A, range, a0_22);
	
		double rot = -r_analysis_getphase(a0_22);
		
		//cs=cos(rot);
		//sn=sin(rot);
		//v=a0(:,range)*[cs sn;-sn cs];		
		double cs = cos(rot);
		double sn = sin(rot);
		gsl_matrix_set(cs_sn, 0, 0, cs);gsl_matrix_set(cs_sn, 0, 1, sn);
		gsl_matrix_set(cs_sn, 1, 0, -sn);gsl_matrix_set(cs_sn, 1, 1, cs);
		
		gsl_matrix_copy_columns(A, range, A_red);
		gsl_matrix_mul(A_red,  cs_sn, v);
		gsl_matrix_copyall_columns(v, range, Astd);
	}

	// calculate also the phase og the standardized A matrix
	// phi=cellfun(@(slc) getphase(ai(slc,slc)),slices);
	for (int i = 0; i < dms; i++) {
		int range[2];
		range[0] = 0+2*i; range[1] = 1+2*i;
		
		gsl_matrix_square_submatrix(Astd, range, a0_22);
		
		double rot = -r_analysis_getphase(a0_22);

		gsl_vector_set(phi, i, rot);
	}		

	gsl_matrix_free(v);
	gsl_matrix_free(A_red);
	gsl_matrix_free(a0_22);
	gsl_matrix_free(cs_sn);
	
}


void r_analysis(gsl_matrix *A, gsl_matrix *mi, gsl_vector *phi, gsl_matrix **bk, gsl_matrix *astdi)
{
	int N = A->size1;
	int M = A->size2;
	int dms = A->size1 / 2;
	int mi_alloc = 0;

	if (mi == NULL) {
		mi_alloc = 1;
	 	mi = gsl_matrix_alloc (N, M);
		gsl_matrix_identity(mi);
	}

	// standardized A
	gsl_matrix *astd = gsl_matrix_alloc (N, M);

	r_analysis_standardize(A, dms, astd, phi);	

	gsl_matrix_mul(mi,  astd, astdi);	

	gsl_matrix *s = jmat_alloc(dms);
	gsl_matrix *t = kmat_alloc(dms);
	
	gsl_matrix *astdis = gsl_matrix_alloc(N,N);
	// ais=ai*t;
	gsl_matrix_mul(astdi,  t, astdis);

	// s';
	gsl_matrix *st = gsl_matrix_alloc(N,N);
	gsl_matrix_transpose2_memcpy(s,st);

	gsl_matrix *invastdi = gsl_matrix_alloc(N,N);
	// invai=ai\s';
	// multiple linear system solver
	gsl_linear_solve_matrix(astdi,st,invastdi);

	
        // bk=cellfun(@(slc) ais(:,slc)*invai(slc,:),slices,'UniformOutput',false);
        // bk=cat(3,bk{:});
        // 2x2 diagonal matrix product
	for (int i = 0; i < dms; i++) {
		int select[2] = {0,1};
		select[0] = 2*i; select[1] = 2*i+1;
		gsl_matrix_selection_mul(astdis, invastdi, select, 2, bk[i]);
	}

	getphase_matrix(astdi, dms, phi);

	gsl_matrix_free(astd);
	gsl_matrix_free(s);
	gsl_matrix_free(t);
	gsl_matrix_free(astdis);
	gsl_matrix_free(st);
	gsl_matrix_free(invastdi);
	if (mi_alloc)
		gsl_matrix_free(mi);
	
}


